#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "scan.h"
#include "dis_peers.h"
#include "network_util.h"


extern struct peer_config config;
extern struct adhoc_windows window_box;

#ifdef USE_IPW2100
extern struct pci_ipw2100 ipw2100;
#endif

void make_pixmappath(struct peers_info_entry *peer, char *pixmappath);
void popup_curr_wlan_info(struct interface_info_entry *iface);
void make_MY_INFO_dev(int cnt, GtkWidget *MY_INFO_dialog, GtkWidget *MY_INFO_notebook, struct interface_info_entry *iface);


int proxy_id;
struct wlan_info		temp_wlan_info;


const char * const iw_wireless_mode[] = { "Wire",
					"Wireless"};


const char * const iw_rechable_mode[] = { "link-local",
					"Global"};


const char * const iw_adhoc_mode[] = { "Poor mode",
					"Link Only Peer mode",
					"Scatter ADGW mode",
					"Proxy ADGW mdoe"};


const char * const iw_op_mode[] = {"Auto",
					"Ad-Hoc",
					"Infrastructure",
					"Master",
					"Repeater",
					"Secondary",
					"Monitor" };

const char * const conference_tool_type[] = { "",
					"VIC&RAT",
					"VIC",
					"RAT"};


void
on_MAIN_QUIT_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
        if( !list_empty(&config.devices) )
        {
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)
		{
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);
			
			init_global_address(entry->dev_name);
			
			cleanup_route_info(entry->dev_name, (struct in6_addr *)entry->s_address.address, 64);
			if((entry->scope > REACHABLE_LINK_SCOPE) || (config.mode > ADHOC_LINK_MODE))
				cleanup_route_info(entry->dev_name, (struct in6_addr *)entry->g_address.address, 64);

			default_router_del(entry->dev_name, NULL);

			change_autoconf_status(entry->dev_name, AUTOCONF_ENABLE);
		}
        }
        gtk_main_quit();
}


void
on_MAIN_window_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	if( !list_empty(&config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)	
		{
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

			init_global_address(entry->dev_name);
			cleanup_route_info(entry->dev_name, (struct in6_addr *)entry->s_address.address, 64);
			if((entry->scope > REACHABLE_LINK_SCOPE) || (config.mode > ADHOC_LINK_MODE))
				cleanup_route_info(entry->dev_name, (struct in6_addr *)entry->g_address.address, 64);

			default_router_del(entry->dev_name, NULL);

			change_autoconf_status(entry->dev_name, AUTOCONF_ENABLE);						
		}
	}
	gtk_main_quit();
}



void
on_CONFIG_DEVICE_combo_entry_insert_text
                                        (GtkEditable     *editable,
                                        gchar           *new_text,
                                        gint             new_text_length,
                                        gpointer         position,
                                        gpointer         user_data)
{
	struct interface_info_entry *entry;
	if( (entry = search_interface_by_name(new_text)) == NULL )
	{
		char buf[32]={0x00,};

		sprintf(buf, "Not found Device [%s]\n", new_text);
		popup_error(buf);
		return ;
	}

	config.curr_if_focus = entry;

	if(entry->ifmode == WIRELESS_INTERFACE )
		popup_curr_wlan_info(entry);

	refresh_adhoc_list();

	return ;
}


void
on_CONFIG_APPLY_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *ap_clist = NULL;
	GtkWidget *APLIST_list = NULL;	
	gchar *essid, *channel;
	struct ap_entry *ap=NULL;

	APLIST_list = lookup_widget( GTK_WIDGET(window_box.CONFIG_dialog), "CONFIG_APLIST_clist" );
	ap_clist = GTK_CLIST (APLIST_list);
		
	if( ap_clist != NULL )
	{
		if (ap_clist->selection)
		{
			gtk_clist_get_text(ap_clist, GPOINTER_TO_INT(ap_clist->selection->data), 0, &essid);
			gtk_clist_get_text(ap_clist, GPOINTER_TO_INT(ap_clist->selection->data), 1, &channel);

			if( (ap=search_adhoc_by_essid(essid)) == NULL )
			{
				popup_error("Not found Ad-hoc AP entry\n");
				return ;
			}
		}
		else
		{
			popup_error("Not found select Ad-hoc AP entry\n");
			return ;
		}
	}
	else
		return ;

	temp_wlan_info.mode = ap->mode_t;
	temp_wlan_info.channel = ap->channel;
	memcpy(temp_wlan_info.essid, ap->essid, strlen(ap->essid));
	
	window_box.NICK_dialog = create_NICK_dialog();
	gtk_widget_show (window_box.NICK_dialog);	

	return ;	
}



void
on_CONFIG_NEW_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	window_box.CREATE_dialog = create_CREATE_dialog ();
 	gtk_widget_show (window_box.CREATE_dialog);

	return;
}



void
on_CONFIG_SCAN_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	if( config.curr_if_focus == NULL )
	{
		popup_error("Select Device \n");
		return ;
	}
	
	if( config.curr_if_focus->ifmode == WIRE_INTERFACE )
	{
		char buf[32]={0x00,};

		sprintf(buf, "Not support wireless scan. [%s]\n", config.curr_if_focus->dev_name);
		popup_error(buf);
		return ;
	}

	if( !scanning( IW_MODE_ADHOC, config.scan_sock, &config.adhoc_network_list, config.curr_if_focus->dev_name) )
		rebuild_adhoc_list();
	else
		popup_error("Scanning fail\n");
		
	return;
}



void
on_CONFIG_QUIT_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	clean_config_dialog();
}

void
on_CONFIG_dialog_destroy               (GtkObject       *object,
                                        gpointer         user_data)
{
	clean_config_dialog();
}









void
on_PROXY_DEVICE_combo_entry_insert_text
                                        (GtkEditable     *editable,
                                        gchar           *new_text,
                                        gint             new_text_length,
                                        gpointer         position,
                                        gpointer         user_data)
{
	struct interface_info_entry *entry;
	if( (entry = search_interface_by_name(new_text)) == NULL )
	{
		char buf[32]={0x00,};

		sprintf(buf, "Not found Device [%s]\n", new_text);
		popup_error(buf);
		return ;
	}

	config.curr_if_focus = entry;

	if(entry->ifmode == WIRELESS_INTERFACE )
		popup_curr_wlan_info(entry);

	refresh_infra_list();

	return ;
}



void
on_PROXY_ENABLE_checkbutton_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *PROXY_frame;

	
	if( GTK_TOGGLE_BUTTON (togglebutton)->active )
	{
		if(config.mode < ADHOC_SCATTER_MODE)
		{
			popup_error("Not support Proxy mode\n");
			gtk_toggle_button_set_active ( togglebutton, FALSE);
			return ;
		}
		
		PROXY_frame = lookup_widget(window_box.PROXY_dialog, "PROXY_frame");
		gtk_widget_set_sensitive (PROXY_frame, TRUE);
	}
	else
	{
		if( config.mode == ADHOC_PROXY_MODE )
		{
			window_box.PROXY_OUT_dialog =  create_PROXY_OUT_dialog();
			gtk_widget_show(window_box.PROXY_OUT_dialog);
		}
	}

}


void
on_PROXY_SCAN_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{	
	if( config.curr_if_focus == NULL )
	{
		popup_error("Select Device \n");
		return ;
	}
	
	if( config.curr_if_focus->ifmode == WIRE_INTERFACE )
	{
		char buf[32]={0x00,};

		sprintf(buf, "Not support wireless scan. [%s]\n", config.curr_if_focus->dev_name);
		popup_error(buf);
		return ;
	}
	
	if( !scanning( IW_MODE_MASTER, config.scan_sock, &config.infra_network_list, config.curr_if_focus->dev_name) )
		rebuild_infra_list();
	else
		popup_error("Scanning fail\n");
	
}


void
on_PROXY_APPLY_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *ap_clist = NULL;
	GtkWidget *APLIST_list = NULL;	
 	GtkWidget *WLAN_radiobutton;
	gchar *essid;
	struct ap_entry *ap=NULL;
	char buffer[32] = {0x00,};	
	int cnt;

	if(config.mode == ADHOC_PROXY_MODE)
	{
		initialize_scatter_network(config.proxy_if);
		config.proxy_if = NULL;
	}

	config.mode = ADHOC_DELAY_MODE;

	if( config.curr_if_focus->ifmode == WIRELESS_INTERFACE )
	{
		APLIST_list = lookup_widget( GTK_WIDGET(window_box.PROXY_dialog), "PROXY_APLIST_clist" );
		ap_clist = GTK_CLIST (APLIST_list);
			
		if( ap_clist != NULL )
		{
			if (ap_clist->selection)
			{
				gtk_clist_get_text(ap_clist, GPOINTER_TO_INT(ap_clist->selection->data), 0, &essid);

				if( (ap = search_infra_by_essid(essid)) == NULL )
				{
					popup_error("Not found AP entry\n");
					return ;
				}
			}
			else
			{
				popup_error("Select AP entry\n");
				return ;
			}
		}
		else
		{
			popup_error("Not found AP entry\n");
			return ;
		}

#ifdef USE_IPW2100
		if( (ipw2100.use_enable == TRUE) && (ipw2100.curr_mode == IW_MODE_ADHOC) )
		{
			if( !strcmp(ipw2100.dev, config.curr_if_focus->dev_name) )
			{
				char buf[128]={0x00,};

				sprintf(buf, "ifdown %s\n", ipw2100.dev);
				system(buf);
				system("modprobe -r ipw2100");
				system("modprobe ipw2100");
			}
		}
#endif
		if( ap_link_change(config.scan_sock, ap->mode_t,  ap->essid, ap->channel, NULL, config.curr_if_focus->dev_name) )
		{
			char buf[128]={0x00,};

			sprintf(buf, "Not change Wireless Infrastructure networking.[%s]\n", config.curr_if_focus->dev_name);	
			popup_error(buf);						
			return;
		}
		else
			sleep(SAFE_WIRELESS_WAIT_TIME);

	}

	if( initialize_proxy_network(config.curr_if_focus) < 0 )
 		return;

	if( !list_empty(&config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)	
		{
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

			change_network(entry);
		}
	}
 
	config.mode = ADHOC_PROXY_MODE;
	config.proxy_if = config.curr_if_focus;

	if(1)
	{
		char buf[128]={0x00,};

		sprintf(buf, "Success Proxy connetion[%s]\n", config.proxy_if->dev_name);
		popup_info(buf);
	}
	return;
}

void
on_PROXY_QUIT_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	clean_proxy_dialog();
}

void
on_PROXY_dialog_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
	clean_proxy_dialog();
}


void
on_PROXY_OUT_dialog_destroy            (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.PROXY_OUT_dialog);
}


void
on_PROXY_OUT_ok_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *PROXY_frame;

	PROXY_frame = lookup_widget(window_box.PROXY_dialog, "PROXY_frame");
	gtk_widget_set_sensitive (PROXY_frame, FALSE);

	initialize_scatter_network(config.proxy_if);
	
	config.proxy_if = NULL;

	gtk_widget_destroy(window_box.PROXY_OUT_dialog);

	clean_proxy_dialog();
	
	return ;
}


void
on_PROXY_OUT_cancel_button_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.PROXY_OUT_dialog);
}





void
on_NICK_ok_button_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *ADGW_frame;
	GtkWidget *BUDDY_frame;
	GtkWidget *NICK_entry;
	char *nick=NULL;

	NICK_entry = lookup_widget(GTK_WIDGET(window_box.NICK_dialog), "NICK_entry");
	nick = (char *) gtk_entry_get_text(GTK_ENTRY(NICK_entry));

	if( !strlen(nick) )
	{
		popup_error("Enter youre Nickname\n");
		return ;
	}
	else
	{
		int mode = temp_wlan_info.mode;
		char *essid = temp_wlan_info.essid;
		int channel = temp_wlan_info.channel;
		strcpy(temp_wlan_info.nickname, nick);

#ifdef USE_IPW2100
		if( (ipw2100.use_enable == TRUE) && (ipw2100.curr_mode != IW_MODE_ADHOC) )
		{
			if( !strcmp(ipw2100.dev, config.curr_if_focus->dev_name) )
			{
				char buf[128]={0x00,};

				sprintf(buf, "ifdown %s\n", ipw2100.dev);
				system(buf);
				system("modprobe -r ipw2100");
				system("modprobe ipw2100 mode=1");
			}
		}
#endif
		
		if( ap_link_change(config.scan_sock, mode, essid, channel, nick, config.curr_if_focus->dev_name) )
		{
			char buf[128]={0x00,};

			sprintf(buf, "Ad-hoc networking configuration fail.[%s]\n", config.curr_if_focus->dev_name);	
			popup_error(buf);						
			memset(&temp_wlan_info, 0, sizeof(struct wlan_info));		
			return;
		}
		else
		{
			sleep(SAFE_WIRELESS_WAIT_TIME);
			ifup_ipv6_restart(config.curr_if_focus->dev_name);
		}	
		
		memcpy(&config.curr_if_focus->curr_wlan_info, &temp_wlan_info, sizeof(struct wlan_info));
		memset(&temp_wlan_info, 0, sizeof(struct wlan_info));
		strcpy(config.curr_if_focus->nickname.nick, config.curr_if_focus->curr_wlan_info.nickname);
		config.curr_if_focus->nickname.nicklen = strlen(config.curr_if_focus->nickname.nick);					
	}

	change_network(config.curr_if_focus);

	rebuild_peer_list();

	popup_curr_wlan_info(config.curr_if_focus);

	send_dis_peers_req(config.curr_if_focus, NULL);

	gtk_widget_destroy(window_box.NICK_dialog);
}

void
on_NICK_cancel_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.NICK_dialog);
}


void
on_NICK_dialog_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.NICK_dialog);
}




void
on_ERROR_dialog_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.ERROR_dialog);
}

void
on_ERROR_button_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.ERROR_dialog);
}



void
on_INFO_dialog_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.INFO_dialog);
}


void
on_INFO_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.INFO_dialog);
}



void
on_CREATE_dialog_destroy               (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.CREATE_dialog);
}

void
on_CREATE_cancel_button_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.CREATE_dialog);
}


void
on_CREATE_ok_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *ADGW_frame;
	GtkWidget *BUDDY_frame;
	GtkWidget *ESSID_entry;
	GtkWidget *NICK_entry;	
	GtkWidget *CHANNEL_entry;
	gchar *essid, *nick;
	gint channel=0;

	ESSID_entry = lookup_widget(GTK_WIDGET(window_box.CREATE_dialog), "CREATE_ESSID_entry");
	essid = (char *) gtk_entry_get_text(GTK_ENTRY(ESSID_entry));	

	if( !strlen(essid) )
	{
		popup_error("ESSID entry is empty\n");
		return ;
	}
	NICK_entry = lookup_widget(GTK_WIDGET(window_box.CREATE_dialog), "CREATE_NICK_entry");
	nick = (char *) gtk_entry_get_text(GTK_ENTRY(NICK_entry));	
	if( !strlen(nick) )
	{
		popup_error("NICKNAME entry is empty\n");
		return ;
	}
			
	CHANNEL_entry = lookup_widget(GTK_WIDGET(window_box.CREATE_dialog), "CREATE_CHANNEL_spinbutton");
	channel = gtk_spin_button_get_value_as_int((GtkSpinButton *)CHANNEL_entry);

#ifdef USE_IPW2100
		if( (ipw2100.use_enable == TRUE) && (ipw2100.curr_mode != IW_MODE_ADHOC) )
		{
			if( !strcmp(ipw2100.dev, config.curr_if_focus->dev_name) )
			{
				char buf[128]={0x00,};

				sprintf(buf, "ifdown %s\n", ipw2100.dev);
				system(buf);
				system("modprobe -r ipw2100");
				system("modprobe ipw2100 mode=1");
			}
		}
#endif
	
	if( ap_link_change(config.scan_sock, IW_MODE_ADHOC, essid, channel, nick, config.curr_if_focus->dev_name) < 0)
	{
		char buf[128]={0x00,};

		sprintf(buf, "Ad-hoc networking create fail.[%s]\n", config.curr_if_focus->dev_name);	
		popup_error(buf);						
		return;
	}
	else
	{
		sleep(SAFE_WIRELESS_WAIT_TIME);
		ifup_ipv6_restart(config.curr_if_focus->dev_name);
	}

	memset( &config.curr_if_focus->curr_wlan_info, 0, sizeof(struct wlan_info));
	get_curr_wireless_info(config.scan_sock, config.curr_if_focus->dev_name, &config.curr_if_focus->curr_wlan_info);

	strcpy(config.curr_if_focus->nickname.nick, config.curr_if_focus->curr_wlan_info.nickname);
	config.curr_if_focus->nickname.nicklen = strlen(config.curr_if_focus->nickname.nick);	
	
	change_network(config.curr_if_focus);

	rebuild_peer_list();

	popup_curr_wlan_info(config.curr_if_focus);	

	send_dis_peers_req(config.curr_if_focus, NULL);

	gtk_widget_destroy(window_box.CREATE_dialog);
}




void
on_CURR_INFO_dialog_destroy            (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.CURR_INFO_dialog);
}



void
on_CURR_INFO_OK_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.CURR_INFO_dialog);
}



void
on_PEER_INFO_dialog_destroy            (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.PEER_INFO_dialog);
}


void
on_PEER_INFO_ok_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.PEER_INFO_dialog);
}


void
on_PEER_INFO_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *peer_clist = NULL;
	GtkWidget *PEER_LIST = NULL, *NICK, *RM, *S_ADDR, *G_ADDR, *G_LABEL, *DEV, *MODE;	
	gchar *nickname, addr_buf[128];
	struct peers_info_entry *peer=NULL;
	
	PEER_LIST = lookup_widget( GTK_WIDGET(window_box.MAIN_window), "PEER_LIST_clist" );
	peer_clist = GTK_CLIST (PEER_LIST);
		
	if( peer_clist != NULL )
	{
		if (peer_clist->selection)
		{
			gtk_clist_get_text(peer_clist, GPOINTER_TO_INT(peer_clist->selection->data), 1, &nickname);

				if( (peer = search_peer_by_nick(nickname)) == NULL )
				{

					if( (peer = search_adgw_by_nick(nickname)) == NULL )
					{
						char buf[128]={0x00,};

						sprintf(buf, "Not foune peer[%s]\n", nickname);
						popup_error(buf);
						return;
					}
				}
		}
		else
		{
			popup_error(" Selecte Peer entry\n");
			return ;
		}
	}
	else
	{
		popup_error("Not found peer entry [empty]\n");
		return ;
	}

	window_box.PEER_INFO_dialog = create_PEER_INFO_dialog();
	gtk_widget_show (window_box.PEER_INFO_dialog);	

	NICK = lookup_widget(GTK_WIDGET(window_box.PEER_INFO_dialog), "PEER_INFO_NICK_entry");	
	RM = lookup_widget(GTK_WIDGET(window_box.PEER_INFO_dialog), "PEER_INFO_RM_entry");
	S_ADDR = lookup_widget(GTK_WIDGET(window_box.PEER_INFO_dialog), "PEER_INFO_S_ADDR_textview");
	G_ADDR = lookup_widget(GTK_WIDGET(window_box.PEER_INFO_dialog), "PEER_INFO_G_ADDR_textview");
	MODE = lookup_widget(GTK_WIDGET(window_box.PEER_INFO_dialog), "PEER_INFO_MODE_entry");
	DEV = lookup_widget(GTK_WIDGET(window_box.PEER_INFO_dialog), "PEER_INFO_DEV_entry");

	gtk_entry_set_text (GTK_ENTRY (MODE), _(iw_adhoc_mode[peer->mode]));
	gtk_entry_set_text (GTK_ENTRY (DEV), _(peer->dev_index));
	gtk_entry_set_text (GTK_ENTRY (NICK), _(peer->nickname.nick));
	gtk_entry_set_text (GTK_ENTRY (RM), _(iw_rechable_mode[peer->status]));

	inet_ntop(AF_INET6, peer->s_address.address, addr_buf, 128);
	gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (S_ADDR)), addr_buf, -1);

	if( (config.mode == ADHOC_LINK_MODE)&&(peer->mode == UNCONFIRMED_LINK_PEER) )
	{
		G_LABEL = lookup_widget(GTK_WIDGET(window_box.PEER_INFO_dialog), "PEER_INFO_G_ADDR_label");
		gtk_widget_set_sensitive (G_LABEL, FALSE);
		gtk_widget_set_sensitive (G_ADDR, FALSE);
	}
	else
	{
		memset(addr_buf, 0, 128);
		inet_ntop(AF_INET6, peer->g_address.address, addr_buf, 128);		
		gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (G_ADDR)), addr_buf, -1);
	}
	
	return ;	
}




void
on_MSG_SEND_dialog_destroy             (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.MSG_SEND_dialog);
}


void
on_MSG_SEND_ok_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *MSG_SEND_NAME_label;	
	GtkWidget *MSG_SEND_entry;
	gchar *nick, *msg;

	int msg_len;
	struct in6_addr		dst_addr;
	unsigned char			buf[MAX_MSG_LEN]={0x00,};
	unsigned char			*pos = buf;
	Option_header		*hdr;

	struct peers_info_entry *peer=NULL;
	struct interface_info_entry *iface=NULL;	
	
	MSG_SEND_NAME_label = lookup_widget( GTK_WIDGET(window_box.MSG_SEND_dialog), "MSG_SEND_NAME_label" );
	nick = (gchar *)gtk_label_get_text(GTK_LABEL(MSG_SEND_NAME_label));

	if( !strlen(nick) )
	{
		popup_error("Not found NickName\n");
		return ;
	}

	MSG_SEND_entry = lookup_widget(GTK_WIDGET(window_box.MSG_SEND_dialog), "MSG_SEND_entry");
	msg = (char *) gtk_entry_get_text(GTK_ENTRY(MSG_SEND_entry));

	if( !strlen(msg) )
	{
		popup_error("Please. Input Message\n");
		return ;
	}
	else
		msg_len = strlen(msg)+1;

	if( (peer = search_peer_by_nick(nick)) == NULL )
	{
		if(( peer = search_adgw_by_nick(nick)) == NULL )
		{
			char buf[128]={0x00,};

			sprintf(buf, "Not foune peer[%s]\n", nick);
			popup_error(buf);
			return;
		}
	}	

	if( (iface = search_interface_by_name(peer->dev_index) ) == NULL )
	{
			char buf[128]={0x00,};

			sprintf(buf, "Not foune interface[%s]\n", peer->dev_index);
			popup_error(buf);
			return;	
	}
	
	create_opt_header(pos, CHAT_MESSAGE, OPTOIN_HEADER_SIZE+NICKNAME_INFO_SIZE+msg_len);
	hdr = (Option_header *)pos;
	pos += OPTOIN_HEADER_SIZE;

	create_chat_message_opt(pos, &iface->nickname, msg);

	if( peer->status == CONFIRMED_SCATTER_PEER )
		memcpy(&dst_addr, peer->g_address.address, sizeof(struct in6_addr));
	else
		memcpy(&dst_addr, peer->s_address.address, sizeof(struct in6_addr));										
		
	debug_hexdump("SEND CHAT_MESSAGE", buf, hdr->length);

//	if( send_udp_packet(config.chat_sock, config.chat_port, buf, hdr->length, &dst_addr) <0 )
	if( send_udp_msg(config.chat_sock, config.chat_port, buf, hdr->length, iface->ifindex, NULL, &dst_addr) <0 )
	{
		char buf[128]={0x00,};

		sprintf(buf, "Message Send fail [%s]\n", peer->nickname.nick);
		popup_error(buf);
	}
	
	gtk_widget_destroy(window_box.MSG_SEND_dialog);	

	return ;
}


void
on_MSG_SEND_cancel_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.MSG_SEND_dialog);
}


void
on_MSG_RCV_dialog_destroy              (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.MSG_RCV_dialog);
}


void
on_MSG_RCV_reponse_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *MSG_RCV_NAME_label;	
	GtkWidget *MSG_SEND_NAME_label;		
	char *nickname;

	MSG_RCV_NAME_label = lookup_widget( GTK_WIDGET(window_box.MSG_RCV_dialog), "MSG_RCV_NAME_label" );
	nickname = (char *)gtk_label_get_text(GTK_LABEL(MSG_RCV_NAME_label));

	if( !strlen(nickname) )
	{
		popup_error("Not found NickName\n");
		return ;
	}

	window_box.MSG_SEND_dialog = create_MSG_SEND_dialog();
	gtk_widget_show (window_box.MSG_SEND_dialog);	

	MSG_SEND_NAME_label = lookup_widget( GTK_WIDGET(window_box.MSG_SEND_dialog), "MSG_SEND_NAME_label" );
	gtk_label_set_text(GTK_LABEL(MSG_SEND_NAME_label), nickname);

	gtk_widget_destroy(window_box.MSG_RCV_dialog);
}


void
on_MSG_RCV_cancel_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.MSG_RCV_dialog);
}


void
on_Send_Message_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_MENU_PROXY_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{

	if( config.window.active_proxy_window == TRUE )
		clean_proxy_dialog();
	
	make_proxy_dialog();

	if( config.mode == ADHOC_PROXY_MODE )
	{
		char buf[128]={0x00,};
		
		GtkWidget *PROXY_ENABLE_checkbutton;

		PROXY_ENABLE_checkbutton = lookup_widget( GTK_WIDGET(window_box.PROXY_dialog), "PROXY_ENABLE_checkbutton" );
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(PROXY_ENABLE_checkbutton), TRUE);

		sprintf(buf, "Current Proxy Mode. Interface [%s]\n", config.proxy_if->dev_name);
		popup_info(buf);
	}
	
}


void
on_MENU_CONFIG_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	if( config.window.active_config_window == TRUE )
		clean_config_dialog();

	make_config_dialog();
}









void
on_PEER_MSG_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *peer_clist = NULL;
	GtkWidget *PEER_LIST = NULL;	
	GtkWidget *MSG_SEND_NAME_label;
	char *nickname;
	struct peers_info_entry *peer=NULL;
	
	PEER_LIST = lookup_widget( GTK_WIDGET(window_box.MAIN_window), "PEER_LIST_clist" );
	peer_clist = GTK_CLIST (PEER_LIST);
		
	if( peer_clist != NULL )
	{
		if (peer_clist->selection)
		{
			gtk_clist_get_text(peer_clist, GPOINTER_TO_INT(peer_clist->selection->data), 1, &nickname);

				if( (peer = search_peer_by_nick(nickname)) == NULL )
				{
					if((peer= search_adgw_by_nick(nickname)) == NULL )
					{
						char buf[128]={0x00,};

						sprintf(buf, "Not foune peer[%s]\n", nickname);
						popup_error(buf);
						return;
					}
				}
		}
		else
		{
			popup_error(" Selecte Peer entry\n");
			return ;
		}
	}
	else
	{
		popup_error("Not found peer entry [empty]\n");
		return ;
	}

	window_box.MSG_SEND_dialog = create_MSG_SEND_dialog();
	gtk_widget_show (window_box.MSG_SEND_dialog);

	MSG_SEND_NAME_label = lookup_widget( GTK_WIDGET(window_box.MSG_SEND_dialog), "MSG_SEND_NAME_label" );
	gtk_label_set_text(GTK_LABEL(MSG_SEND_NAME_label), nickname);
	
	return ;
}

void
on_RAT_button_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *peer_clist = NULL;
	GtkWidget *PEER_LIST = NULL;	
	gchar *nickname;
	int cnt;
	struct peers_info_entry *peer;
	struct interface_info_entry *iface;
	struct in6_addr		dst_addr;

	int msg_len;
	unsigned char			buf[MAX_MSG_LEN]={0x00,};
	unsigned char			*pos = buf;
	Option_header		*hdr;
		
	PEER_LIST = lookup_widget( GTK_WIDGET(window_box.MAIN_window), "PEER_LIST_clist" );
	peer_clist = GTK_CLIST (PEER_LIST);
		
	if( peer_clist != NULL )
	{
		if (peer_clist->selection)
		{
			gtk_clist_get_text(peer_clist, GPOINTER_TO_INT(peer_clist->selection->data), 1, &nickname);

				if( (peer = search_peer_by_nick(nickname)) == NULL )
				{
					if((peer= search_adgw_by_nick(nickname)) == NULL )
					{
						char buf[128]={0x00,};

						sprintf(buf, "Not foune peer[%s]\n", nickname);
						popup_error(buf);
						return;
					}
				}
		}
		else
		{
			popup_error(" Selecte Peer entry\n");
			return ;
		}
	}
	else
	{
		popup_error("Not found peer entry [empty]\n");
		return ;
	}

	if( (iface = search_interface_by_name(peer->dev_index) ) == NULL )
	{
			char buf[128]={0x00,};

			sprintf(buf, "Not foune interface[%s]\n", peer->dev_index);
			popup_error(buf);
			return;	
	}

	send_conference_request(CONFERENCE_RAT, iface, peer);
	
	return;
}



void
on_CONFERNCE_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *peer_clist = NULL;
	GtkWidget *PEER_LIST = NULL;	
	gchar *nickname;
	int cnt;
	struct peers_info_entry *peer;
	struct interface_info_entry *iface;
	struct in6_addr		dst_addr;

	int msg_len;
	unsigned char			buf[MAX_MSG_LEN]={0x00,};
	unsigned char			*pos = buf;
	Option_header		*hdr;
		
	PEER_LIST = lookup_widget( GTK_WIDGET(window_box.MAIN_window), "PEER_LIST_clist" );
	peer_clist = GTK_CLIST (PEER_LIST);
		
	if( peer_clist != NULL )
	{
		if (peer_clist->selection)
		{
			gtk_clist_get_text(peer_clist, GPOINTER_TO_INT(peer_clist->selection->data), 1, &nickname);

				if( (peer = search_peer_by_nick(nickname)) == NULL )
				{
					if((peer= search_adgw_by_nick(nickname)) == NULL )
					{
						char buf[128]={0x00,};

						sprintf(buf, "Not foune peer[%s]\n", nickname);
						popup_error(buf);
						return;
					}
				}
		}
		else
		{
			popup_error(" Selecte Peer entry\n");
			return ;
		}
	}
	else
	{
		popup_error("Not found peer entry [empty]\n");
		return ;
	}

	if( (iface = search_interface_by_name(peer->dev_index) ) == NULL )
	{
			char buf[128]={0x00,};

			sprintf(buf, "Not foune interface[%s]\n", peer->dev_index);
			popup_error(buf);
			return;	
	}

	send_conference_request(CONFERENCE_ALL, iface, peer);
	
	return;
}


void
on_PEER_JOIN_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *peer_clist = NULL;
	GtkWidget *PEER_LIST = NULL;	
	char *nickname;

	struct peers_info_entry *peer;
	struct interface_info_entry *iface;
	
	PEER_LIST = lookup_widget( GTK_WIDGET(window_box.MAIN_window), "PEER_LIST_clist" );
	peer_clist = GTK_CLIST (PEER_LIST);
		
	if( peer_clist != NULL )
	{
		if (peer_clist->selection)
		{
			gtk_clist_get_text(peer_clist, GPOINTER_TO_INT(peer_clist->selection->data), 1, &nickname);

			if( (peer = search_adgw_by_nick(nickname)) == NULL )
			{
				popup_error("Not found ADGW entry\n");
				return ;
			}
		}
		else
		{
			popup_error("Select ADGW entry\n");
			return ;
		}
			
	}
	else
	{
		popup_error("Not found ADGW LIST entry\n");
		return ;
	}

	if( peer->mode < ADHOC_SCATTER_MODE )
	{
		char buf[32]={0x00,};

		sprintf(buf, "Not support Scatter Networking. [%s]\n", nickname);
		popup_error(buf);
		return ;
	}

	if( (iface = search_interface_by_name(peer->dev_index)) == NULL )
	{
		char buf[32]={0x00,};

		sprintf(buf, "Not found Out going Interface [%s]\n", peer->dev_index);
		popup_error(buf);
		return ;	
	}
	
	send_adgw_join_req(iface, peer);

	return;
}








void
popup_error(char *msg)
{
	GtkWidget *ERR_LABEL = NULL;	
	
	window_box.ERROR_dialog = create_ERROR_dialog();
	gtk_widget_show (window_box.ERROR_dialog);	

	ERR_LABEL = lookup_widget( GTK_WIDGET(window_box.ERROR_dialog), "ERROR_label" );
	gtk_label_set_text(GTK_LABEL(ERR_LABEL), msg);
	return;
}

void
thread_popup_error(char *msg)
{
	GtkWidget *ERR_LABEL = NULL;	

	gdk_threads_enter();
	
	window_box.ERROR_dialog = create_ERROR_dialog();
	gtk_widget_show (window_box.ERROR_dialog);	

	ERR_LABEL = lookup_widget( GTK_WIDGET(window_box.ERROR_dialog), "ERROR_label" );
	gtk_label_set_text(GTK_LABEL(ERR_LABEL), msg);

	gdk_threads_leave();
	
	return;
}




void
popup_info(char *msg)
{
	GtkWidget *INFO_label = NULL;	
	
	window_box.INFO_dialog = create_INFO_dialog();
	gtk_widget_show (window_box.INFO_dialog);	

	INFO_label = lookup_widget( GTK_WIDGET(window_box.INFO_dialog), "INFO_label" );
	gtk_label_set_text(GTK_LABEL(INFO_label), msg);
	return;
}

void
thread_popup_info(char *msg)
{
	GtkWidget *INFO_label = NULL;	

	gdk_threads_enter();
	
	window_box.INFO_dialog = create_INFO_dialog();
	gtk_widget_show (window_box.INFO_dialog);	

	INFO_label = lookup_widget( GTK_WIDGET(window_box.INFO_dialog), "INFO_label" );
	gtk_label_set_text(GTK_LABEL(INFO_label), msg);
	
	gdk_threads_leave();
	
	return;
}



void popup_curr_wlan_info(struct interface_info_entry *iface)
{
	GtkWidget *CMODE;
	GtkWidget *ESSID;	
	GtkWidget *NICK;
	GtkWidget *WMODE;
	GtkWidget *CHANNEL;
	char s_chnl[3]={0x0,};
	int wireless=TRUE;

	if( iface->ifmode == WIRE_INTERFACE )
		wireless=FALSE;
	
	window_box.CURR_INFO_dialog = create_CURR_INFO_dialog();
	gtk_widget_show (window_box.CURR_INFO_dialog);

	CMODE = lookup_widget(GTK_WIDGET(window_box.CURR_INFO_dialog), "CURR_INFO_CMODE_entry");
	ESSID = lookup_widget(GTK_WIDGET(window_box.CURR_INFO_dialog), "CURR_INFO_ESSID_entry");
	NICK = lookup_widget(GTK_WIDGET(window_box.CURR_INFO_dialog), "CURR_INFO_NICK_entry");	
	WMODE = lookup_widget(GTK_WIDGET(window_box.CURR_INFO_dialog), "CURR_INFO_WMODE_entry");
	CHANNEL = lookup_widget(GTK_WIDGET(window_box.CURR_INFO_dialog), "CURR_INFO_CH_entry");

	gtk_entry_set_text (GTK_ENTRY (CMODE), _(iw_adhoc_mode[config.mode]));

	if(wireless)
	{
		if( get_curr_wireless_info(config.scan_sock, iface->dev_name, &iface->curr_wlan_info) < 0 )
		{
			char buf[128]={0x00,};

			sprintf(buf, "Get current wireless information fail.[%s]\n", iface->dev_name);
			popup_error(buf);
			gtk_widget_destroy(window_box.CURR_INFO_dialog);
			return ;
		}
	
		gtk_entry_set_text (GTK_ENTRY (ESSID), _(iface->curr_wlan_info.essid));
		gtk_entry_set_text (GTK_ENTRY (NICK), _(iface->curr_wlan_info.nickname));
		gtk_entry_set_text (GTK_ENTRY (WMODE), _(iw_op_mode[iface->curr_wlan_info.mode]));
		sprintf(s_chnl, "%d", iface->curr_wlan_info.channel);
		gtk_entry_set_text (GTK_ENTRY (CHANNEL), _(s_chnl));		
	}
	else
	{
		gtk_widget_set_sensitive (ESSID, FALSE);
		gtk_widget_set_sensitive (NICK, FALSE);
		gtk_widget_set_sensitive (WMODE, FALSE);		
		gtk_widget_set_sensitive (CHANNEL, FALSE);		
	}
	return;
}



thread_popup_conference(int type, struct peers_info_entry *peer)
{
	GtkWidget *CFRC_NICK_label;
	GtkWidget *CFRC_TYPE_label;	
	
	gdk_threads_enter();
	
	window_box.CFRC_dialog = create_CFRC_dialog();
	gtk_widget_show (window_box.CFRC_dialog);	

	CFRC_NICK_label = lookup_widget( GTK_WIDGET(window_box.CFRC_dialog), "CFRC_NICK_label" );
	gtk_label_set_text(GTK_LABEL(CFRC_NICK_label), peer->nickname.nick);

	CFRC_TYPE_label = lookup_widget( GTK_WIDGET(window_box.CFRC_dialog), "CFRC_TYPE_label" );
	gtk_label_set_text(GTK_LABEL(CFRC_TYPE_label), conference_tool_type[type]);
	
	gdk_threads_leave();
}

void popup_rcv_message(char *nick, char *msg)
{
	GtkWidget *MSG_RCV_textview = NULL;	
	GtkWidget *MSG_RCV_NAME_label;
	
	gdk_threads_enter();
	
	window_box.MSG_RCV_dialog = create_MSG_RCV_dialog();
	gtk_widget_show (window_box.MSG_RCV_dialog);	

	MSG_RCV_textview = lookup_widget( GTK_WIDGET(window_box.MSG_RCV_dialog), "MSG_RCV_textview" );
	gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (MSG_RCV_textview)), msg, -1);

	MSG_RCV_NAME_label = lookup_widget( GTK_WIDGET(window_box.MSG_RCV_dialog), "MSG_RCV_NAME_label" );
	gtk_label_set_text(GTK_LABEL(MSG_RCV_NAME_label), nick);
	
	gdk_threads_leave();
	
	return;
}


void refresh_adhoc_list()
{
	GtkWidget *APLIST_list = NULL;

	APLIST_list = lookup_widget( GTK_WIDGET(window_box.CONFIG_dialog), "CONFIG_APLIST_clist" );
	
	gtk_clist_clear(GTK_CLIST(APLIST_list));
	gtk_clist_freeze(GTK_CLIST(APLIST_list));

	gtk_clist_thaw(GTK_CLIST(APLIST_list));

	return ;
}

void rebuild_adhoc_list()
{
	int row;
	GtkWidget *APLIST_list = NULL;
	gchar *listitem[2]={0x00,};

	APLIST_list = lookup_widget( GTK_WIDGET(window_box.CONFIG_dialog), "CONFIG_APLIST_clist" );
	
	gtk_clist_clear(GTK_CLIST(APLIST_list));
	gtk_clist_freeze(GTK_CLIST(APLIST_list));

	if( !list_empty(&config.adhoc_network_list) )
	{
		struct list_head* lead = &config.adhoc_network_list;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)	
		{
			char s_chnl[3]={0x00,};
			struct ap_entry *entry = list_entry(p, typeof(*entry), list);

			row = gtk_clist_append(GTK_CLIST(APLIST_list), listitem );
			gtk_clist_set_text(GTK_CLIST(APLIST_list), row, 0, entry->essid);
			sprintf(s_chnl, "%d", entry->channel);
			gtk_clist_set_text(GTK_CLIST(APLIST_list), row, 1, s_chnl);
		}	
	}

	gtk_clist_sort(GTK_CLIST(APLIST_list));
	gtk_clist_thaw(GTK_CLIST(APLIST_list));

	return ;
}

void refresh_infra_list()
{
	GtkWidget *APLIST_list = NULL;

	APLIST_list = lookup_widget( GTK_WIDGET(window_box.PROXY_dialog), "PROXY_APLIST_clist" );
	
	gtk_clist_clear(GTK_CLIST(APLIST_list));
	gtk_clist_freeze(GTK_CLIST(APLIST_list));

	gtk_clist_thaw(GTK_CLIST(APLIST_list));

	return ;
}

void rebuild_infra_list()
{
	int row;
	GtkWidget *APLIST_list = NULL;
	gchar *listitem[2]={0x00,};

	APLIST_list = lookup_widget( GTK_WIDGET(window_box.PROXY_dialog), "PROXY_APLIST_clist" );
	
	gtk_clist_clear(GTK_CLIST(APLIST_list));
	gtk_clist_freeze(GTK_CLIST(APLIST_list));

	if( !list_empty(&config.infra_network_list) )
	{
		struct list_head* lead = &config.infra_network_list;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)		
		{
			struct ap_entry *entry = list_entry(p, typeof(*entry), list);

			row = gtk_clist_append(GTK_CLIST(APLIST_list), listitem );
			gtk_clist_set_text(GTK_CLIST(APLIST_list), row, 0, entry->essid);
			printf("[rebuild_infra_list] ap essid [%s]\n", entry->essid);
		}	
	}

	gtk_clist_sort(GTK_CLIST(APLIST_list));
	gtk_clist_thaw(GTK_CLIST(APLIST_list));
	
	return ;
}

void refresh_peer_list()
{
	GtkWidget *PEER_LIST = NULL;

	PEER_LIST = lookup_widget( GTK_WIDGET(window_box.MAIN_window), "PEER_LIST_clist" );

	gtk_clist_clear(GTK_CLIST(PEER_LIST));
	gtk_clist_freeze(GTK_CLIST(PEER_LIST));

	gtk_clist_thaw(GTK_CLIST(PEER_LIST));

	return ;
}



void rebuild_peer_list( )
{
	int row, ref = 0;
	GtkWidget *PEER_LIST = NULL;
	gchar *listitem[2]={0x00,};

	GtkStyle *user_style = NULL;
	GdkPixmap *st_pixmap = NULL;
	GdkBitmap *st_mask = NULL;
	char status_pixmap[128]={0x00,};
	
	PEER_LIST = lookup_widget( GTK_WIDGET(window_box.MAIN_window), "PEER_LIST_clist" );

	gtk_clist_clear(GTK_CLIST(PEER_LIST));
	gtk_clist_freeze(GTK_CLIST(PEER_LIST));

	user_style = gtk_widget_get_style(PEER_LIST);

	if( !list_empty(&config.peers_list) )
	{
		struct list_head* lead = &config.peers_list;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)		
		{
			struct peers_info_entry *entry = list_entry(p, typeof(*entry), list);

			make_pixmappath(entry, status_pixmap);

			st_pixmap = gdk_pixmap_create_from_xpm(PEER_LIST->window, &st_mask, &user_style->bg[GTK_STATE_NORMAL], status_pixmap);

			row = gtk_clist_append(GTK_CLIST(PEER_LIST), listitem );

			gtk_clist_set_pixmap(GTK_CLIST(PEER_LIST), row, 0, st_pixmap, st_mask );	
			gtk_clist_set_text(GTK_CLIST(PEER_LIST), row, 1, entry->nickname.nick);

			ref = TRUE;		
		}	
	}
	
	if( !list_empty(&config.adgws_list) )
	{
		struct list_head* lead = &config.adgws_list;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)		
		{
			struct peers_info_entry *entry = list_entry(p, typeof(*entry), list);

			make_pixmappath(entry, status_pixmap);

			st_pixmap = gdk_pixmap_create_from_xpm(PEER_LIST->window, &st_mask, &user_style->bg[GTK_STATE_NORMAL], status_pixmap);

			row = gtk_clist_append(GTK_CLIST(PEER_LIST), listitem );

			gtk_clist_set_pixmap(GTK_CLIST(PEER_LIST), row, 0, st_pixmap, st_mask );	
			gtk_clist_set_text(GTK_CLIST(PEER_LIST), row, 1, entry->nickname.nick);

			ref = TRUE;
		}	
	}		

	if( ref == TRUE )
	{
		gdk_pixmap_unref( st_pixmap );
		gdk_bitmap_unref( st_mask );		
	}
	gtk_clist_sort(GTK_CLIST(PEER_LIST));
	gtk_clist_thaw(GTK_CLIST(PEER_LIST));
	
	return ;
}



void thread_rebuild_peer_list( )
{
	int row, ref=0;
	GtkWidget *PEER_LIST = NULL;
	gchar *listitem[2]={0x00,};

	GtkStyle *user_style = NULL;
	GdkPixmap *st_pixmap = NULL;
	GdkBitmap *st_mask = NULL;
	char status_pixmap[128]={0x00,};

	printf("[thread_rebuild_peer_list] start\n");	

	gdk_threads_enter();

	PEER_LIST = lookup_widget( GTK_WIDGET(window_box.MAIN_window), "PEER_LIST_clist" );

	gtk_clist_clear(GTK_CLIST(PEER_LIST));
	gtk_clist_freeze(GTK_CLIST(PEER_LIST));

	user_style = gtk_widget_get_style(PEER_LIST);

	if( !list_empty(&config.peers_list) )
	{
		struct list_head* lead = &config.peers_list;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)		
		{
			struct peers_info_entry *entry = list_entry(p, typeof(*entry), list);

			make_pixmappath(entry, status_pixmap);

			st_pixmap = gdk_pixmap_create_from_xpm(PEER_LIST->window, &st_mask, &user_style->bg[GTK_STATE_NORMAL], status_pixmap);

			row = gtk_clist_append(GTK_CLIST(PEER_LIST), listitem );

			gtk_clist_set_pixmap(GTK_CLIST(PEER_LIST), row, 0, st_pixmap, st_mask );	
			gtk_clist_set_text(GTK_CLIST(PEER_LIST), row, 1, entry->nickname.nick);

			printf("thread_rebuild_peer_list : peer entry[%s]\n", entry->nickname.nick);
			ref = TRUE;		
		}	
	}
	
	if( !list_empty(&config.adgws_list) )
	{
		struct list_head* lead = &config.adgws_list;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)		
		{
			struct peers_info_entry *entry = list_entry(p, typeof(*entry), list);

			make_pixmappath(entry, status_pixmap);

			st_pixmap = gdk_pixmap_create_from_xpm(PEER_LIST->window, &st_mask, &user_style->bg[GTK_STATE_NORMAL], status_pixmap);

			row = gtk_clist_append(GTK_CLIST(PEER_LIST), listitem );

			gtk_clist_set_pixmap(GTK_CLIST(PEER_LIST), row, 0, st_pixmap, st_mask );	
			gtk_clist_set_text(GTK_CLIST(PEER_LIST), row, 1, entry->nickname.nick);

			printf("thread_rebuild_peer_list : adgw entry[%s]\n", entry->nickname.nick);
			ref = TRUE;
		}	
	}		

	if( ref == TRUE )
	{
		gdk_pixmap_unref( st_pixmap );
		gdk_bitmap_unref( st_mask );		
	}
	gtk_clist_sort(GTK_CLIST(PEER_LIST));
	gtk_clist_thaw(GTK_CLIST(PEER_LIST));

	gdk_threads_leave();

	printf("[thread_rebuild_peer_list] end\n");	
	
	return ;
}

void rebuild_config_devicelist()
{
	GtkWidget *CONFIG_DEVICE_combo;
	GList *CONFIG_DEVICE_combo_items = NULL;
	
	CONFIG_DEVICE_combo = lookup_widget( GTK_WIDGET(window_box.CONFIG_dialog), "CONFIG_DEVICE_combo" );

	if( !list_empty( &config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)
		{	
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

			if( (entry->ifmode == WIRELESS_INTERFACE) && (entry->proxy_enable == ADHOC_PROXY_DISABLE) )
				CONFIG_DEVICE_combo_items = g_list_append (CONFIG_DEVICE_combo_items, (gpointer) _(entry->dev_name));
		}

		gtk_combo_set_popdown_strings (GTK_COMBO (CONFIG_DEVICE_combo), CONFIG_DEVICE_combo_items);
		g_list_free (CONFIG_DEVICE_combo_items);	
	}
	
	return ;
}

void rebuild_proxy_devicelist()
{
	GtkWidget *PROXY_DEVICE_combo;
	GList *PROXY_DEVICE_combo_items = NULL;
	
	PROXY_DEVICE_combo = lookup_widget( GTK_WIDGET(window_box.PROXY_dialog), "PROXY_DEVICE_combo" );

	if( !list_empty( &config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)
		{	
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

			PROXY_DEVICE_combo_items = g_list_append (PROXY_DEVICE_combo_items, (gpointer) _(entry->dev_name));
		}

		gtk_combo_set_popdown_strings (GTK_COMBO (PROXY_DEVICE_combo), PROXY_DEVICE_combo_items);
		g_list_free (PROXY_DEVICE_combo_items);	
	}
	
	return ;
}




void make_pixmappath(struct peers_info_entry *peer, char *pixmappath)
{
	if( peer->mode == ADHOC_LINK_MODE )
	{
		if(peer->status == CONFIRMED_SCATTER_PEER )
			sprintf( pixmappath, "%s/%s/pixmaps/%s", PACKAGE_DATA_DIR, PACKAGE, "scatter_peer.xpm");
		else if(peer->status == UNCONFIRMED_LINK_PEER )
			sprintf( pixmappath, "%s/%s/pixmaps/%s", PACKAGE_DATA_DIR, PACKAGE, "linklocal_peer.xpm");
	}
	else if( peer->mode == ADHOC_SCATTER_MODE )
		sprintf( pixmappath, "%s/%s/pixmaps/%s", PACKAGE_DATA_DIR, PACKAGE, "scatter_adgw.xpm");
	else if(peer->mode == ADHOC_PROXY_MODE )
		sprintf( pixmappath, "%s/%s/pixmaps/%s", PACKAGE_DATA_DIR, PACKAGE, "proxy_adgw.xpm");
	else
		sprintf( pixmappath, "%s/%s/pixmaps/%s", PACKAGE_DATA_DIR, PACKAGE, "poor_peer.xpm");

//	printf("[make_pixmappath] : %s\n", pixmappath);
	
	return ;
}





void make_config_dialog(void)
{
	GtkWidget *CONFIG_DEVICE_combo_entry;
	char *dev_name;
	struct interface_info_entry *iface;
	
	window_box.CONFIG_dialog =  create_CONFIG_dialog();
	gtk_widget_show(window_box.CONFIG_dialog);

	rebuild_config_devicelist();

	CONFIG_DEVICE_combo_entry = lookup_widget(GTK_WIDGET(window_box.CONFIG_dialog), "CONFIG_DEVICE_combo_entry");
	dev_name = (char*) gtk_entry_get_text(GTK_ENTRY(CONFIG_DEVICE_combo_entry));	

	if( (iface = search_interface_by_name(dev_name)) == NULL)
	{
		char buf[32]={0x00,};

		clean_config_dialog();
		
		sprintf(buf, "Not found Wireless Interface\n");
		popup_error(buf);
		return ;
	}

	config.curr_if_focus = iface;
	config.window.active_config_window = TRUE;

	return;
}

void clean_config_dialog(void)
{
	gtk_widget_destroy(window_box.CONFIG_dialog);
	config.window.active_config_window = FALSE;

	return;
}



void make_proxy_dialog(void)
{
	GtkWidget *PROXY_DEVICE_combo_entry;
	char *dev_name;
	struct interface_info_entry *iface;
	
	window_box.PROXY_dialog =  create_PROXY_dialog();
	gtk_widget_show(window_box.PROXY_dialog);

	rebuild_proxy_devicelist();

	PROXY_DEVICE_combo_entry = lookup_widget(GTK_WIDGET(window_box.PROXY_dialog), "PROXY_DEVICE_combo_entry");
	dev_name = (char*) gtk_entry_get_text(GTK_ENTRY(PROXY_DEVICE_combo_entry));	

	if( (iface = search_interface_by_name(dev_name)) == NULL)
	{
		char buf[32]={0x00,};

		clean_proxy_dialog();
		
		sprintf(buf, "Not found Interface. [%s]\n", dev_name);
		popup_error(buf);
		return ;
	}

	config.curr_if_focus = iface;
	config.window.active_proxy_window = TRUE;

	return;
}


void clean_proxy_dialog(void)
{
	gtk_widget_destroy(window_box.PROXY_dialog);
	config.window.active_proxy_window = FALSE;

	return;
}




void
on_CONFIG_info_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	if(config.curr_if_focus->ifmode == WIRELESS_INTERFACE )
		popup_curr_wlan_info(config.curr_if_focus);
}


void
on_PROXY_info_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	if(config.curr_if_focus->ifmode == WIRELESS_INTERFACE )
		popup_curr_wlan_info(config.curr_if_focus);
	else
	{
		char buf[128]={0x00,};

		sprintf(buf, "Wire Interface[%s]. Not Supported Scan!\n", config.curr_if_focus->dev_name);
		popup_info(buf);
	}
}




void
on_CFRC_dialog_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	return ;
}


void
on_CFRC_cancel_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *CFRC_NICK_label;	
	GtkWidget *CFRC_TYPE_label;		
	char *nick, *s_type;
	int type;

	struct peers_info_entry *peer=NULL;
	struct interface_info_entry *iface=NULL;	
	
	CFRC_NICK_label = lookup_widget( GTK_WIDGET(window_box.CFRC_dialog), "CFRC_NICK_label" );
	nick = (char *)gtk_label_get_text(GTK_LABEL(CFRC_NICK_label));

	if( !strlen(nick) )
	{
		popup_error("Not found NickName\n");
		return ;
	}

	CFRC_TYPE_label = lookup_widget( GTK_WIDGET(window_box.CFRC_dialog), "CFRC_TYPE_label" );
	s_type = (char *)gtk_label_get_text(GTK_LABEL(CFRC_TYPE_label));
	if( !strlen(s_type) )
	{
		popup_error("Not found type\n");
		return ;
	}

	if( !strcmp(s_type, conference_tool_type[1]) )
		type = CONFERENCE_ALL;
	else if( !strcmp(s_type, conference_tool_type[2]) )
		type = CONFERENCE_VIC;
	else if( !strcmp(s_type, conference_tool_type[3]) )
		type = CONFERENCE_RAT;
	else
		return ;

	
	if( (peer = search_peer_by_nick(nick)) == NULL )
	{
		if(( peer = search_adgw_by_nick(nick)) == NULL )
		{
			char buf[128]={0x00,};

			sprintf(buf, "Not foune peer[%s]\n", nick);
			popup_error(buf);
			return;
		}
	}	

	if( (iface = search_interface_by_name(peer->dev_index) ) == NULL )
	{
			char buf[128]={0x00,};

			sprintf(buf, "Not foune interface[%s]\n", peer->dev_index);
			popup_error(buf);
			return;	
	}

	send_conference_response(type,CONNECTION_REFUSE, iface, peer);

	gtk_widget_destroy(window_box.CFRC_dialog);

	return ;
}


void
on_CFRC_ok_button_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *CFRC_NICK_label;	
	GtkWidget *CFRC_TYPE_label;		
	char *nick, *s_type;
	int type;

	struct peers_info_entry *peer=NULL;
	struct interface_info_entry *iface=NULL;	

	struct in6_addr addr;
	
	CFRC_NICK_label = lookup_widget( GTK_WIDGET(window_box.CFRC_dialog), "CFRC_NICK_label" );
	nick = (char *)gtk_label_get_text(GTK_LABEL(CFRC_NICK_label));

	if( !strlen(nick) )
	{
		popup_error("Not found NickName\n");
		return ;
	}

	CFRC_TYPE_label = lookup_widget( GTK_WIDGET(window_box.CFRC_dialog), "CFRC_TYPE_label" );
	s_type = (char *)gtk_label_get_text(GTK_LABEL(CFRC_TYPE_label));
	if( !strlen(s_type) )
	{
		popup_error("Not found type\n");
		return ;
	}

	if( !strcmp(s_type, conference_tool_type[1]) )
		type = CONFERENCE_ALL;
	else if( !strcmp(s_type, conference_tool_type[2]) )
		type = CONFERENCE_VIC;
	else if( !strcmp(s_type, conference_tool_type[3]) )
		type = CONFERENCE_RAT;
	else
		return ;

	
	if( (peer = search_peer_by_nick(nick)) == NULL )
	{
		if(( peer = search_adgw_by_nick(nick)) == NULL )
		{
			char buf[128]={0x00,};

			sprintf(buf, "Not foune peer[%s]\n", nick);
			popup_error(buf);
			return;
		}
	}	

	if( (iface = search_interface_by_name(peer->dev_index) ) == NULL )
	{
			char buf[128]={0x00,};

			sprintf(buf, "Not foune interface[%s]\n", peer->dev_index);
			popup_error(buf);
			return;	
	}

	send_conference_response(type, CONNECTION_ACCEPT, iface, peer);

	if(peer->status == CONFIRMED_SCATTER_PEER)
		memcpy(&addr, peer->g_address.address, sizeof(struct in6_addr));
	else
		memcpy(&addr, peer->s_address.address, sizeof(struct in6_addr));

	switch(type)
	{
		case CONFERENCE_ALL :
			run_skt_conference_rat(&addr, DEFAULT_RAT_PORT);
			run_skt_conference_vic(&addr, DEFAULT_VIC_PORT);
			break;
		case CONFERENCE_VIC:
			run_skt_conference_vic(&addr, DEFAULT_VIC_PORT);
			break;
		case CONFERENCE_RAT :
			run_skt_conference_rat(&addr, DEFAULT_RAT_PORT);
			break;
		default :
			break;
	}

	gtk_widget_destroy(window_box.CFRC_dialog);
	
	return ;
}


void
on_MENU_MY_INFO_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	window_box.MY_INFO_dialog = make_MY_INFO_dialog();
	gtk_widget_show(window_box.MY_INFO_dialog);

	return ;
}


void
on_MY_INFO_dialog_destroy              (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.MY_INFO_dialog);
}


void
on_MY_INFO_dev_ok_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(window_box.MY_INFO_dialog);
}


#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)



GtkWidget* make_MY_INFO_dialog (void)
{
	GtkWidget *MY_INFO_dialog;
	GtkWidget *MY_INFO_vbox;
	GtkWidget *MY_INFO_notebook;
	GtkWidget *MY_INFO_hbuttonbox;

	MY_INFO_dialog = gtk_dialog_new ();
	gtk_widget_set_size_request (MY_INFO_dialog, 220, 265);
	GTK_WIDGET_SET_FLAGS (MY_INFO_dialog, GTK_CAN_FOCUS);
	GTK_WIDGET_SET_FLAGS (MY_INFO_dialog, GTK_CAN_DEFAULT);
	gtk_window_set_title (GTK_WINDOW (MY_INFO_dialog), _("SKT IPv6 Adhoc"));
	gtk_window_set_position (GTK_WINDOW (MY_INFO_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_modal (GTK_WINDOW (MY_INFO_dialog), TRUE);
	gtk_window_set_resizable (GTK_WINDOW (MY_INFO_dialog), FALSE);
	gtk_window_set_destroy_with_parent (GTK_WINDOW (MY_INFO_dialog), TRUE);
	gtk_dialog_set_has_separator (GTK_DIALOG (MY_INFO_dialog), FALSE);

	MY_INFO_vbox = GTK_DIALOG (MY_INFO_dialog)->vbox;
	gtk_widget_show (MY_INFO_vbox);

	MY_INFO_notebook = gtk_notebook_new ();
	gtk_widget_show (MY_INFO_notebook);
	gtk_box_pack_start (GTK_BOX (MY_INFO_vbox), MY_INFO_notebook, TRUE, TRUE, 0);


	g_signal_connect ((gpointer) MY_INFO_dialog, "destroy",
	                G_CALLBACK (on_MY_INFO_dialog_destroy),
	                NULL);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (MY_INFO_dialog, MY_INFO_dialog, "MY_INFO_dialog");
	GLADE_HOOKUP_OBJECT_NO_REF (MY_INFO_dialog, MY_INFO_vbox, "MY_INFO_vbox");
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_notebook, "MY_INFO_notebook");
	GLADE_HOOKUP_OBJECT_NO_REF (MY_INFO_dialog, MY_INFO_hbuttonbox, "MY_INFO_hbuttonbox");

	if( !list_empty( &config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;
		int cnt=0;

		list_for_each_safe(p, n, lead)
		{	
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

			printf("=====================================================%s\n", entry->dev_name);
			make_MY_INFO_dev(cnt, MY_INFO_dialog, MY_INFO_notebook, entry);
			cnt++;
		}
	}

	gtk_widget_grab_focus (MY_INFO_dialog);
	gtk_widget_grab_default (MY_INFO_dialog);
	return MY_INFO_dialog;
}


void make_MY_INFO_dev(int cnt, GtkWidget *MY_INFO_dialog, GtkWidget *MY_INFO_notebook, struct interface_info_entry *iface)
{
	GtkWidget *MY_INFO_dev_frame;
	GtkWidget *MY_INFO_dev_fixed;
	GtkWidget *MY_INFO_dev_nick_entry;
	GtkWidget *MY_INFO_dev_g_addr_textview;
	GtkWidget *MY_INFO_dev_mode_entry;
	GtkWidget *MY_INFO_dev_l_addr_textview;
	GtkWidget *MY_INFO_dev_mode_label;
	GtkWidget *MY_INFO_dev_nick_label;
	GtkWidget *MY_INFO_dev_l_addr_label;
	GtkWidget *MY_INFO_dev_s_addr_textview;
	GtkWidget *MY_INFO_dev_g_addr_label;
	GtkWidget *MY_INFO_dev_s_addr_label;
	GtkWidget *MY_INFO_dev_ok_button;
	GtkWidget *MY_INFO_dev_alignment;
	GtkWidget *MY_INFO_dev_hbox;
	GtkWidget *MY_INFO_dev_image;
	GtkWidget *MY_INFO_dev_label;
	GtkWidget *MY_INFO_dev_frame_hbox;
	GtkWidget *MY_INFO_dev_frame_image;
	GtkWidget *MY_INFO_dev_frame_label;
	GtkWidget *MY_INFO_dev_menu_label;

	char g_buf[128]={0x00,}, s_buf[128]={0x00,}, l_buf[128]={0x00,}, g_name[128]={0x00,};

	MY_INFO_dev_frame = gtk_frame_new (NULL);
	gtk_widget_show (MY_INFO_dev_frame);
	gtk_container_add (GTK_CONTAINER (MY_INFO_notebook), MY_INFO_dev_frame);


	
	MY_INFO_dev_frame_hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (MY_INFO_dev_frame_hbox);
	gtk_frame_set_label_widget (GTK_FRAME (MY_INFO_dev_frame), MY_INFO_dev_frame_hbox);

	MY_INFO_dev_frame_image = gtk_image_new_from_stock ("gtk-dialog-question", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (MY_INFO_dev_frame_image);
	gtk_box_pack_start (GTK_BOX (MY_INFO_dev_frame_hbox), MY_INFO_dev_frame_image, TRUE, TRUE, 0);

	MY_INFO_dev_frame_label = gtk_label_new (_(" Ad-hoc peer information "));
	gtk_widget_show (MY_INFO_dev_frame_label);
	gtk_box_pack_start (GTK_BOX (MY_INFO_dev_frame_hbox), MY_INFO_dev_frame_label, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (MY_INFO_dev_frame_label), GTK_JUSTIFY_LEFT);

	MY_INFO_dev_menu_label = gtk_label_new (_(iface->dev_name));
	gtk_widget_show (MY_INFO_dev_menu_label);
	gtk_notebook_set_tab_label (GTK_NOTEBOOK (MY_INFO_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (MY_INFO_notebook), cnt), MY_INFO_dev_menu_label);
	gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev_menu_label), TRUE);

	MY_INFO_dev_fixed = gtk_fixed_new ();
	gtk_widget_show (MY_INFO_dev_fixed);
	gtk_container_add (GTK_CONTAINER (MY_INFO_dev_frame), MY_INFO_dev_fixed);

	if( iface->ifmode == WIRELESS_INTERFACE )
	{
		MY_INFO_dev_nick_label = gtk_label_new (_("NickName"));
		gtk_widget_show (MY_INFO_dev_nick_label);
		gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_nick_label, 95, 0);
		gtk_widget_set_size_request (MY_INFO_dev_nick_label, 90, 16);
		gtk_label_set_justify (GTK_LABEL (MY_INFO_dev_nick_label), GTK_JUSTIFY_LEFT);
		gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev_nick_label), TRUE);

		MY_INFO_dev_nick_entry = gtk_entry_new ();
		gtk_widget_show (MY_INFO_dev_nick_entry);
		gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_nick_entry, 90, 16);
		gtk_widget_set_size_request (MY_INFO_dev_nick_entry, 120, 16);
		GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev_nick_entry, GTK_CAN_FOCUS);
		gtk_entry_set_max_length (GTK_ENTRY (MY_INFO_dev_nick_entry), 32);
		gtk_editable_set_editable (GTK_EDITABLE (MY_INFO_dev_nick_entry), FALSE);
		gtk_entry_set_text (GTK_ENTRY (MY_INFO_dev_nick_entry), _(iface->nickname.nick));
	}

	MY_INFO_dev_mode_label = gtk_label_new (_("Mode"));
	gtk_widget_show (MY_INFO_dev_mode_label);
	gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_mode_label, 0, 0);
	gtk_widget_set_size_request (MY_INFO_dev_mode_label, 50, 16);
	gtk_label_set_justify (GTK_LABEL (MY_INFO_dev_mode_label), GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev_mode_label), TRUE);

	MY_INFO_dev_mode_entry = gtk_entry_new ();
	gtk_widget_show (MY_INFO_dev_mode_entry);
	gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_mode_entry, 0, 16);
	gtk_widget_set_size_request (MY_INFO_dev_mode_entry, 80, 16);
	GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev_mode_entry, GTK_CAN_FOCUS);
	gtk_editable_set_editable (GTK_EDITABLE (MY_INFO_dev_mode_entry), FALSE);
	gtk_entry_set_text (GTK_ENTRY (MY_INFO_dev_mode_entry), _(iw_wireless_mode[iface->ifmode]));

	MY_INFO_dev_l_addr_label = gtk_label_new (_("IPv6 address"));
	gtk_widget_show (MY_INFO_dev_l_addr_label);
	gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_l_addr_label, 0, 32);
	gtk_widget_set_size_request (MY_INFO_dev_l_addr_label, 104, 16);
	gtk_label_set_justify (GTK_LABEL (MY_INFO_dev_l_addr_label), GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev_l_addr_label), TRUE);

	MY_INFO_dev_l_addr_textview = gtk_text_view_new ();
	gtk_widget_show (MY_INFO_dev_l_addr_textview);
	gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_l_addr_textview, 0, 48);
	gtk_widget_set_size_request (MY_INFO_dev_l_addr_textview, 210, 33);
	GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev_l_addr_textview, GTK_CAN_FOCUS);
	gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev_l_addr_textview), FALSE);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev_l_addr_textview), GTK_WRAP_CHAR);
	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev_l_addr_textview), FALSE);
	inet_ntop(AF_INET6, (struct in6_addr *)iface->ll_address.address, l_buf, 128);
	gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (MY_INFO_dev_l_addr_textview)),
	    _(l_buf), -1);

	MY_INFO_dev_s_addr_label = gtk_label_new (_("IPv6 Adhoc address"));
	gtk_widget_show (MY_INFO_dev_s_addr_label);
	gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_s_addr_label, 0, 80);
	gtk_widget_set_size_request (MY_INFO_dev_s_addr_label, 131, 16);
	gtk_label_set_justify (GTK_LABEL (MY_INFO_dev_s_addr_label), GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev_s_addr_label), TRUE);

	MY_INFO_dev_s_addr_textview = gtk_text_view_new ();
	gtk_widget_show (MY_INFO_dev_s_addr_textview);
	gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_s_addr_textview, 0, 96);
	gtk_widget_set_size_request (MY_INFO_dev_s_addr_textview, 210, 33);
	GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev_s_addr_textview, GTK_CAN_FOCUS);
	gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev_s_addr_textview), FALSE);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev_s_addr_textview), GTK_WRAP_CHAR);
	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev_s_addr_textview), FALSE);
	inet_ntop(AF_INET6, (struct in6_addr *)iface->s_address.address, s_buf, 128);	
	gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (MY_INFO_dev_s_addr_textview)),
	    _(s_buf), -1);


	if( iface->scope == REACHABLE_GLOBAL_SCOPE )
	{
		MY_INFO_dev_g_addr_label = gtk_label_new (_("IPv6 Global address"));
		gtk_widget_show (MY_INFO_dev_g_addr_label);
		gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_g_addr_label, 0, 128);
		gtk_widget_set_size_request (MY_INFO_dev_g_addr_label, 131, 16);
		gtk_label_set_justify (GTK_LABEL (MY_INFO_dev_g_addr_label), GTK_JUSTIFY_LEFT);
		gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev_g_addr_label), TRUE);

		MY_INFO_dev_g_addr_textview = gtk_text_view_new ();
		gtk_widget_show (MY_INFO_dev_g_addr_textview);
		gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_g_addr_textview, 0, 144);
		gtk_widget_set_size_request (MY_INFO_dev_g_addr_textview, 210, 33);
		GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev_g_addr_textview, GTK_CAN_FOCUS);
		gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev_g_addr_textview), FALSE);
		gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev_g_addr_textview), GTK_WRAP_CHAR);
		gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev_g_addr_textview), FALSE);

		inet_ntop(AF_INET6, (struct in6_addr *)iface->g_address.address, g_buf, 128);
		
		gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (MY_INFO_dev_g_addr_textview)),  _(g_buf), -1);
	}
	

	MY_INFO_dev_ok_button = gtk_button_new ();
	gtk_widget_show (MY_INFO_dev_ok_button);
	gtk_fixed_put (GTK_FIXED (MY_INFO_dev_fixed), MY_INFO_dev_ok_button, 136, 180);
	gtk_widget_set_size_request (MY_INFO_dev_ok_button, 72, 24);
	GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev_ok_button, GTK_CAN_FOCUS);

	MY_INFO_dev_alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (MY_INFO_dev_alignment);
	gtk_container_add (GTK_CONTAINER (MY_INFO_dev_ok_button), MY_INFO_dev_alignment);

	MY_INFO_dev_hbox = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (MY_INFO_dev_hbox);
	gtk_container_add (GTK_CONTAINER (MY_INFO_dev_alignment), MY_INFO_dev_hbox);

	MY_INFO_dev_image = gtk_image_new_from_stock ("gtk-ok", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (MY_INFO_dev_image);
	gtk_box_pack_start (GTK_BOX (MY_INFO_dev_hbox), MY_INFO_dev_image, FALSE, FALSE, 0);

	MY_INFO_dev_label = gtk_label_new_with_mnemonic (_("OK"));
	gtk_widget_show (MY_INFO_dev_label);
	gtk_box_pack_start (GTK_BOX (MY_INFO_dev_hbox), MY_INFO_dev_label, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (MY_INFO_dev_label), GTK_JUSTIFY_LEFT);




	g_signal_connect ((gpointer) MY_INFO_dev_ok_button, "clicked",
	                G_CALLBACK (on_MY_INFO_dev_ok_button_clicked),
	                NULL);


	sprintf(g_name, "MY_INFO_DEV_frame%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_fixed%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_nick_entry%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_gaddr_textview%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_mode%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_laddr_textview%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_mode_label%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_nick_label%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_laddr_label%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_saddr_textview%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_gaddr_label%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_saddr_label%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_ok_button%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_alignement%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_hbox%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_image%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_label%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_frame_hbox%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_frame_image%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_frame_label%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
	memset(g_name, 0, 128);
	sprintf(g_name, "MY_INFO_DEV_menu_label%d", cnt);
	GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev_frame, g_name);
}

/*

GtkWidget*
create_MY_INFO_dialog (void)
{
  GtkWidget *MY_INFO_dialog;
  GtkWidget *MY_INFO_vbox;
  GtkWidget *MY_INFO_notebook;
  GtkWidget *MY_INFO_dev1_frame;
  GtkWidget *MY_INFO_dev1_fixed;
  GtkWidget *MY_INFO_dev1_nick_entry;
  GtkWidget *MY_INFO_dev1_g_addr_textview;
  GtkWidget *MY_INFO_dev1_mode_entry;
  GtkWidget *MY_INFO_dev1_l_addr_textview;
  GtkWidget *MY_INFO_dev1_mode_label;
  GtkWidget *MY_INFO_dev1_nick_label;
  GtkWidget *MY_INFO_dev1_l_addr_label;
  GtkWidget *MY_INFO_dev1_s_addr_textview;
  GtkWidget *MY_INFO_dev1_g_addr_label;
  GtkWidget *MY_INFO_dev1_s_addr_label;
  GtkWidget *MY_INFO_dev1_ok_button;
  GtkWidget *alignment58;
  GtkWidget *hbox74;
  GtkWidget *image90;
  GtkWidget *label152;
  GtkWidget *MY_INFO_dev1_frame_hbox;
  GtkWidget *MY_INFO_dev1_frame_image;
  GtkWidget *MY_INFO_dev1_frame_label;
  GtkWidget *MY_INFO_dev1_label;
  GtkWidget *MY_INFO_dev2_frame;
  GtkWidget *MY_INFO_dev2_fixed;
  GtkWidget *MY_INFO_dev2_nick_entry;
  GtkWidget *MY_INFO_dev2_g_addr_textview;
  GtkWidget *MY_INFO_dev2_mode_entry;
  GtkWidget *MY_INFO_dev2_l_addr_textview;
  GtkWidget *MY_INFO_dev2_mode_label;
  GtkWidget *MY_INFO_dev2_nick_label;
  GtkWidget *MY_INFO_dev2_l_addr_label;
  GtkWidget *MY_INFO_dev2_s_addr_textview;
  GtkWidget *MY_INFO_dev2_g_addr_label;
  GtkWidget *MY_INFO_dev2_s_addr_label;
  GtkWidget *MY_INFO_dev2_ok_button;
  GtkWidget *alignment63;
  GtkWidget *hbox84;
  GtkWidget *image100;
  GtkWidget *label186;
  GtkWidget *MY_INFO_dev2_frame_hbox;
  GtkWidget *MY_INFO_dev2_frame_image;
  GtkWidget *MY_INFO_dev2_frame_label;
  GtkWidget *MY_INFO_dev2_label;
  GtkWidget *MY_INFO_dev3_frame;
  GtkWidget *MY_INFO_dev3_fixed;
  GtkWidget *MY_INFO_dev3_nick_entry;
  GtkWidget *MY_INFO_dev3_g_addr_textview;
  GtkWidget *MY_INFO_dev3_mode_entry;
  GtkWidget *MY_INFO_dev3_l_addr_textview;
  GtkWidget *MY_INFO_dev3_mode_label;
  GtkWidget *MY_INFO_dev3_nick_label;
  GtkWidget *MY_INFO_dev3_l_addr_label;
  GtkWidget *MY_INFO_dev3_s_addr_textview;
  GtkWidget *MY_INFO_dev3_g_addr_label;
  GtkWidget *MY_INFO_dev3_s_addr_label;
  GtkWidget *MY_INFO_dev3_ok_button;
  GtkWidget *alignment64;
  GtkWidget *hbox86;
  GtkWidget *image102;
  GtkWidget *label193;
  GtkWidget *MY_INFO_dev3_frame_hbox;
  GtkWidget *MY_INFO_dev3_frame_image;
  GtkWidget *MY_INFO_dev3_frame_label;
  GtkWidget *MY_INFO_dev3_label;
  GtkWidget *MY_INFO_hbuttonbox;

  MY_INFO_dialog = gtk_dialog_new ();
  gtk_widget_set_size_request (MY_INFO_dialog, 220, 265);
  GTK_WIDGET_SET_FLAGS (MY_INFO_dialog, GTK_CAN_FOCUS);
  GTK_WIDGET_SET_FLAGS (MY_INFO_dialog, GTK_CAN_DEFAULT);
  gtk_window_set_title (GTK_WINDOW (MY_INFO_dialog), _("SKT IPv6 Adhoc"));
  gtk_window_set_position (GTK_WINDOW (MY_INFO_dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (MY_INFO_dialog), TRUE);
  gtk_window_set_resizable (GTK_WINDOW (MY_INFO_dialog), FALSE);
  gtk_window_set_destroy_with_parent (GTK_WINDOW (MY_INFO_dialog), TRUE);
  gtk_dialog_set_has_separator (GTK_DIALOG (MY_INFO_dialog), FALSE);

  MY_INFO_vbox = GTK_DIALOG (MY_INFO_dialog)->vbox;
  gtk_widget_show (MY_INFO_vbox);

  MY_INFO_notebook = gtk_notebook_new ();
  gtk_widget_show (MY_INFO_notebook);
  gtk_box_pack_start (GTK_BOX (MY_INFO_vbox), MY_INFO_notebook, TRUE, TRUE, 0);

  MY_INFO_dev1_frame = gtk_frame_new (NULL);
  gtk_widget_show (MY_INFO_dev1_frame);
  gtk_container_add (GTK_CONTAINER (MY_INFO_notebook), MY_INFO_dev1_frame);

  MY_INFO_dev1_fixed = gtk_fixed_new ();
  gtk_widget_show (MY_INFO_dev1_fixed);
  gtk_container_add (GTK_CONTAINER (MY_INFO_dev1_frame), MY_INFO_dev1_fixed);

  MY_INFO_dev1_nick_entry = gtk_entry_new ();
  gtk_widget_show (MY_INFO_dev1_nick_entry);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_nick_entry, 90, 16);
  gtk_widget_set_size_request (MY_INFO_dev1_nick_entry, 120, 16);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev1_nick_entry, GTK_CAN_FOCUS);
  gtk_entry_set_max_length (GTK_ENTRY (MY_INFO_dev1_nick_entry), 32);
  gtk_editable_set_editable (GTK_EDITABLE (MY_INFO_dev1_nick_entry), FALSE);

  MY_INFO_dev1_g_addr_textview = gtk_text_view_new ();
  gtk_widget_show (MY_INFO_dev1_g_addr_textview);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_g_addr_textview, 0, 144);
  gtk_widget_set_size_request (MY_INFO_dev1_g_addr_textview, 210, 33);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev1_g_addr_textview, GTK_CAN_FOCUS);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev1_g_addr_textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev1_g_addr_textview), GTK_WRAP_CHAR);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev1_g_addr_textview), FALSE);

  MY_INFO_dev1_mode_entry = gtk_entry_new ();
  gtk_widget_show (MY_INFO_dev1_mode_entry);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_mode_entry, 0, 16);
  gtk_widget_set_size_request (MY_INFO_dev1_mode_entry, 80, 16);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev1_mode_entry, GTK_CAN_FOCUS);
  gtk_editable_set_editable (GTK_EDITABLE (MY_INFO_dev1_mode_entry), FALSE);

  MY_INFO_dev1_l_addr_textview = gtk_text_view_new ();
  gtk_widget_show (MY_INFO_dev1_l_addr_textview);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_l_addr_textview, 0, 48);
  gtk_widget_set_size_request (MY_INFO_dev1_l_addr_textview, 210, 33);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev1_l_addr_textview, GTK_CAN_FOCUS);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev1_l_addr_textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev1_l_addr_textview), GTK_WRAP_CHAR);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev1_l_addr_textview), FALSE);

  MY_INFO_dev1_mode_label = gtk_label_new (_("Mode"));
  gtk_widget_show (MY_INFO_dev1_mode_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_mode_label, 0, 0);
  gtk_widget_set_size_request (MY_INFO_dev1_mode_label, 50, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev1_mode_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev1_mode_label), TRUE);

  MY_INFO_dev1_nick_label = gtk_label_new (_("NickName"));
  gtk_widget_show (MY_INFO_dev1_nick_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_nick_label, 95, 0);
  gtk_widget_set_size_request (MY_INFO_dev1_nick_label, 90, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev1_nick_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev1_nick_label), TRUE);

  MY_INFO_dev1_l_addr_label = gtk_label_new (_("IPv6 address"));
  gtk_widget_show (MY_INFO_dev1_l_addr_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_l_addr_label, 0, 32);
  gtk_widget_set_size_request (MY_INFO_dev1_l_addr_label, 104, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev1_l_addr_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev1_l_addr_label), TRUE);

  MY_INFO_dev1_s_addr_textview = gtk_text_view_new ();
  gtk_widget_show (MY_INFO_dev1_s_addr_textview);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_s_addr_textview, 0, 96);
  gtk_widget_set_size_request (MY_INFO_dev1_s_addr_textview, 210, 33);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev1_s_addr_textview, GTK_CAN_FOCUS);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev1_s_addr_textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev1_s_addr_textview), GTK_WRAP_CHAR);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev1_s_addr_textview), FALSE);

  MY_INFO_dev1_g_addr_label = gtk_label_new (_("IPv6 Global address"));
  gtk_widget_show (MY_INFO_dev1_g_addr_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_g_addr_label, 0, 128);
  gtk_widget_set_size_request (MY_INFO_dev1_g_addr_label, 131, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev1_g_addr_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev1_g_addr_label), TRUE);

  MY_INFO_dev1_s_addr_label = gtk_label_new (_("IPv6 Adhoc address"));
  gtk_widget_show (MY_INFO_dev1_s_addr_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_s_addr_label, 0, 80);
  gtk_widget_set_size_request (MY_INFO_dev1_s_addr_label, 131, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev1_s_addr_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev1_s_addr_label), TRUE);

  MY_INFO_dev1_ok_button = gtk_button_new ();
  gtk_widget_show (MY_INFO_dev1_ok_button);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev1_fixed), MY_INFO_dev1_ok_button, 136, 180);
  gtk_widget_set_size_request (MY_INFO_dev1_ok_button, 72, 24);
  GTK_WIDGET_SET_FLAGS (MY_INFO_dev1_ok_button, GTK_CAN_DEFAULT);

  alignment58 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment58);
  gtk_container_add (GTK_CONTAINER (MY_INFO_dev1_ok_button), alignment58);

  hbox74 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox74);
  gtk_container_add (GTK_CONTAINER (alignment58), hbox74);

  image90 = gtk_image_new_from_stock ("gtk-ok", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image90);
  gtk_box_pack_start (GTK_BOX (hbox74), image90, FALSE, FALSE, 0);

  label152 = gtk_label_new_with_mnemonic (_("OK"));
  gtk_widget_show (label152);
  gtk_box_pack_start (GTK_BOX (hbox74), label152, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label152), GTK_JUSTIFY_LEFT);

  MY_INFO_dev1_frame_hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (MY_INFO_dev1_frame_hbox);
  gtk_frame_set_label_widget (GTK_FRAME (MY_INFO_dev1_frame), MY_INFO_dev1_frame_hbox);

  MY_INFO_dev1_frame_image = gtk_image_new_from_stock ("gtk-dialog-question", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (MY_INFO_dev1_frame_image);
  gtk_box_pack_start (GTK_BOX (MY_INFO_dev1_frame_hbox), MY_INFO_dev1_frame_image, TRUE, TRUE, 0);

  MY_INFO_dev1_frame_label = gtk_label_new (_(" Ad-hoc peer information "));
  gtk_widget_show (MY_INFO_dev1_frame_label);
  gtk_box_pack_start (GTK_BOX (MY_INFO_dev1_frame_hbox), MY_INFO_dev1_frame_label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev1_frame_label), GTK_JUSTIFY_LEFT);

  MY_INFO_dev1_label = gtk_label_new (_("eth0"));
  gtk_widget_show (MY_INFO_dev1_label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (MY_INFO_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (MY_INFO_notebook), 0), MY_INFO_dev1_label);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev1_label), TRUE);

  MY_INFO_dev2_frame = gtk_frame_new (NULL);
  gtk_widget_show (MY_INFO_dev2_frame);
  gtk_container_add (GTK_CONTAINER (MY_INFO_notebook), MY_INFO_dev2_frame);

  MY_INFO_dev2_fixed = gtk_fixed_new ();
  gtk_widget_show (MY_INFO_dev2_fixed);
  gtk_container_add (GTK_CONTAINER (MY_INFO_dev2_frame), MY_INFO_dev2_fixed);

  MY_INFO_dev2_nick_entry = gtk_entry_new ();
  gtk_widget_show (MY_INFO_dev2_nick_entry);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_nick_entry, 90, 16);
  gtk_widget_set_size_request (MY_INFO_dev2_nick_entry, 120, 16);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev2_nick_entry, GTK_CAN_FOCUS);
  gtk_entry_set_max_length (GTK_ENTRY (MY_INFO_dev2_nick_entry), 32);
  gtk_editable_set_editable (GTK_EDITABLE (MY_INFO_dev2_nick_entry), FALSE);

  MY_INFO_dev2_g_addr_textview = gtk_text_view_new ();
  gtk_widget_show (MY_INFO_dev2_g_addr_textview);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_g_addr_textview, 0, 144);
  gtk_widget_set_size_request (MY_INFO_dev2_g_addr_textview, 210, 33);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev2_g_addr_textview, GTK_CAN_FOCUS);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev2_g_addr_textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev2_g_addr_textview), GTK_WRAP_CHAR);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev2_g_addr_textview), FALSE);

  MY_INFO_dev2_mode_entry = gtk_entry_new ();
  gtk_widget_show (MY_INFO_dev2_mode_entry);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_mode_entry, 0, 16);
  gtk_widget_set_size_request (MY_INFO_dev2_mode_entry, 80, 16);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev2_mode_entry, GTK_CAN_FOCUS);
  gtk_editable_set_editable (GTK_EDITABLE (MY_INFO_dev2_mode_entry), FALSE);

  MY_INFO_dev2_l_addr_textview = gtk_text_view_new ();
  gtk_widget_show (MY_INFO_dev2_l_addr_textview);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_l_addr_textview, 0, 48);
  gtk_widget_set_size_request (MY_INFO_dev2_l_addr_textview, 210, 33);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev2_l_addr_textview, GTK_CAN_FOCUS);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev2_l_addr_textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev2_l_addr_textview), GTK_WRAP_CHAR);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev2_l_addr_textview), FALSE);

  MY_INFO_dev2_mode_label = gtk_label_new (_("Mode"));
  gtk_widget_show (MY_INFO_dev2_mode_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_mode_label, 0, 0);
  gtk_widget_set_size_request (MY_INFO_dev2_mode_label, 50, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev2_mode_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev2_mode_label), TRUE);

  MY_INFO_dev2_nick_label = gtk_label_new (_("NickName"));
  gtk_widget_show (MY_INFO_dev2_nick_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_nick_label, 95, 0);
  gtk_widget_set_size_request (MY_INFO_dev2_nick_label, 90, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev2_nick_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev2_nick_label), TRUE);

  MY_INFO_dev2_l_addr_label = gtk_label_new (_("IPv6 address"));
  gtk_widget_show (MY_INFO_dev2_l_addr_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_l_addr_label, 0, 32);
  gtk_widget_set_size_request (MY_INFO_dev2_l_addr_label, 104, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev2_l_addr_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev2_l_addr_label), TRUE);

  MY_INFO_dev2_s_addr_textview = gtk_text_view_new ();
  gtk_widget_show (MY_INFO_dev2_s_addr_textview);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_s_addr_textview, 0, 96);
  gtk_widget_set_size_request (MY_INFO_dev2_s_addr_textview, 210, 33);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev2_s_addr_textview, GTK_CAN_FOCUS);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev2_s_addr_textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev2_s_addr_textview), GTK_WRAP_CHAR);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev2_s_addr_textview), FALSE);

  MY_INFO_dev2_g_addr_label = gtk_label_new (_("IPv6 Global address"));
  gtk_widget_show (MY_INFO_dev2_g_addr_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_g_addr_label, 0, 128);
  gtk_widget_set_size_request (MY_INFO_dev2_g_addr_label, 131, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev2_g_addr_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev2_g_addr_label), TRUE);

  MY_INFO_dev2_s_addr_label = gtk_label_new (_("IPv6 Adhoc address"));
  gtk_widget_show (MY_INFO_dev2_s_addr_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_s_addr_label, 0, 80);
  gtk_widget_set_size_request (MY_INFO_dev2_s_addr_label, 131, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev2_s_addr_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev2_s_addr_label), TRUE);

  MY_INFO_dev2_ok_button = gtk_button_new ();
  gtk_widget_show (MY_INFO_dev2_ok_button);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev2_fixed), MY_INFO_dev2_ok_button, 136, 180);
  gtk_widget_set_size_request (MY_INFO_dev2_ok_button, 72, 24);
  GTK_WIDGET_SET_FLAGS (MY_INFO_dev2_ok_button, GTK_CAN_DEFAULT);

  alignment63 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment63);
  gtk_container_add (GTK_CONTAINER (MY_INFO_dev2_ok_button), alignment63);

  hbox84 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox84);
  gtk_container_add (GTK_CONTAINER (alignment63), hbox84);

  image100 = gtk_image_new_from_stock ("gtk-ok", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image100);
  gtk_box_pack_start (GTK_BOX (hbox84), image100, FALSE, FALSE, 0);

  label186 = gtk_label_new_with_mnemonic (_("OK"));
  gtk_widget_show (label186);
  gtk_box_pack_start (GTK_BOX (hbox84), label186, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label186), GTK_JUSTIFY_LEFT);

  MY_INFO_dev2_frame_hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (MY_INFO_dev2_frame_hbox);
  gtk_frame_set_label_widget (GTK_FRAME (MY_INFO_dev2_frame), MY_INFO_dev2_frame_hbox);

  MY_INFO_dev2_frame_image = gtk_image_new_from_stock ("gtk-dialog-question", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (MY_INFO_dev2_frame_image);
  gtk_box_pack_start (GTK_BOX (MY_INFO_dev2_frame_hbox), MY_INFO_dev2_frame_image, TRUE, TRUE, 0);

  MY_INFO_dev2_frame_label = gtk_label_new (_(" Ad-hoc peer information "));
  gtk_widget_show (MY_INFO_dev2_frame_label);
  gtk_box_pack_start (GTK_BOX (MY_INFO_dev2_frame_hbox), MY_INFO_dev2_frame_label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev2_frame_label), GTK_JUSTIFY_LEFT);

  MY_INFO_dev2_label = gtk_label_new (_("eth1"));
  gtk_widget_show (MY_INFO_dev2_label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (MY_INFO_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (MY_INFO_notebook), 1), MY_INFO_dev2_label);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev2_label), TRUE);

  MY_INFO_dev3_frame = gtk_frame_new (NULL);
  gtk_widget_show (MY_INFO_dev3_frame);
  gtk_container_add (GTK_CONTAINER (MY_INFO_notebook), MY_INFO_dev3_frame);

  MY_INFO_dev3_fixed = gtk_fixed_new ();
  gtk_widget_show (MY_INFO_dev3_fixed);
  gtk_container_add (GTK_CONTAINER (MY_INFO_dev3_frame), MY_INFO_dev3_fixed);

  MY_INFO_dev3_nick_entry = gtk_entry_new ();
  gtk_widget_show (MY_INFO_dev3_nick_entry);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_nick_entry, 90, 16);
  gtk_widget_set_size_request (MY_INFO_dev3_nick_entry, 120, 16);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev3_nick_entry, GTK_CAN_FOCUS);
  gtk_entry_set_max_length (GTK_ENTRY (MY_INFO_dev3_nick_entry), 32);
  gtk_editable_set_editable (GTK_EDITABLE (MY_INFO_dev3_nick_entry), FALSE);

  MY_INFO_dev3_g_addr_textview = gtk_text_view_new ();
  gtk_widget_show (MY_INFO_dev3_g_addr_textview);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_g_addr_textview, 0, 144);
  gtk_widget_set_size_request (MY_INFO_dev3_g_addr_textview, 210, 33);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev3_g_addr_textview, GTK_CAN_FOCUS);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev3_g_addr_textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev3_g_addr_textview), GTK_WRAP_CHAR);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev3_g_addr_textview), FALSE);

  MY_INFO_dev3_mode_entry = gtk_entry_new ();
  gtk_widget_show (MY_INFO_dev3_mode_entry);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_mode_entry, 0, 16);
  gtk_widget_set_size_request (MY_INFO_dev3_mode_entry, 80, 16);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev3_mode_entry, GTK_CAN_FOCUS);
  gtk_editable_set_editable (GTK_EDITABLE (MY_INFO_dev3_mode_entry), FALSE);

  MY_INFO_dev3_l_addr_textview = gtk_text_view_new ();
  gtk_widget_show (MY_INFO_dev3_l_addr_textview);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_l_addr_textview, 0, 48);
  gtk_widget_set_size_request (MY_INFO_dev3_l_addr_textview, 210, 33);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev3_l_addr_textview, GTK_CAN_FOCUS);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev3_l_addr_textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev3_l_addr_textview), GTK_WRAP_CHAR);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev3_l_addr_textview), FALSE);

  MY_INFO_dev3_mode_label = gtk_label_new (_("Mode"));
  gtk_widget_show (MY_INFO_dev3_mode_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_mode_label, 0, 0);
  gtk_widget_set_size_request (MY_INFO_dev3_mode_label, 50, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev3_mode_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev3_mode_label), TRUE);

  MY_INFO_dev3_nick_label = gtk_label_new (_("NickName"));
  gtk_widget_show (MY_INFO_dev3_nick_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_nick_label, 95, 0);
  gtk_widget_set_size_request (MY_INFO_dev3_nick_label, 90, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev3_nick_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev3_nick_label), TRUE);

  MY_INFO_dev3_l_addr_label = gtk_label_new (_("IPv6 address"));
  gtk_widget_show (MY_INFO_dev3_l_addr_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_l_addr_label, 0, 32);
  gtk_widget_set_size_request (MY_INFO_dev3_l_addr_label, 104, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev3_l_addr_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev3_l_addr_label), TRUE);

  MY_INFO_dev3_s_addr_textview = gtk_text_view_new ();
  gtk_widget_show (MY_INFO_dev3_s_addr_textview);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_s_addr_textview, 0, 96);
  gtk_widget_set_size_request (MY_INFO_dev3_s_addr_textview, 210, 33);
  GTK_WIDGET_UNSET_FLAGS (MY_INFO_dev3_s_addr_textview, GTK_CAN_FOCUS);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (MY_INFO_dev3_s_addr_textview), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (MY_INFO_dev3_s_addr_textview), GTK_WRAP_CHAR);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (MY_INFO_dev3_s_addr_textview), FALSE);

  MY_INFO_dev3_g_addr_label = gtk_label_new (_("IPv6 Global address"));
  gtk_widget_show (MY_INFO_dev3_g_addr_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_g_addr_label, 0, 128);
  gtk_widget_set_size_request (MY_INFO_dev3_g_addr_label, 131, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev3_g_addr_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev3_g_addr_label), TRUE);

  MY_INFO_dev3_s_addr_label = gtk_label_new (_("IPv6 Adhoc address"));
  gtk_widget_show (MY_INFO_dev3_s_addr_label);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_s_addr_label, 0, 80);
  gtk_widget_set_size_request (MY_INFO_dev3_s_addr_label, 131, 16);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev3_s_addr_label), GTK_JUSTIFY_LEFT);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev3_s_addr_label), TRUE);

  MY_INFO_dev3_ok_button = gtk_button_new ();
  gtk_widget_show (MY_INFO_dev3_ok_button);
  gtk_fixed_put (GTK_FIXED (MY_INFO_dev3_fixed), MY_INFO_dev3_ok_button, 136, 180);
  gtk_widget_set_size_request (MY_INFO_dev3_ok_button, 72, 24);
  GTK_WIDGET_SET_FLAGS (MY_INFO_dev3_ok_button, GTK_CAN_DEFAULT);

  alignment64 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment64);
  gtk_container_add (GTK_CONTAINER (MY_INFO_dev3_ok_button), alignment64);

  hbox86 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox86);
  gtk_container_add (GTK_CONTAINER (alignment64), hbox86);

  image102 = gtk_image_new_from_stock ("gtk-ok", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image102);
  gtk_box_pack_start (GTK_BOX (hbox86), image102, FALSE, FALSE, 0);

  label193 = gtk_label_new_with_mnemonic (_("OK"));
  gtk_widget_show (label193);
  gtk_box_pack_start (GTK_BOX (hbox86), label193, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label193), GTK_JUSTIFY_LEFT);

  MY_INFO_dev3_frame_hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (MY_INFO_dev3_frame_hbox);
  gtk_frame_set_label_widget (GTK_FRAME (MY_INFO_dev3_frame), MY_INFO_dev3_frame_hbox);

  MY_INFO_dev3_frame_image = gtk_image_new_from_stock ("gtk-dialog-question", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (MY_INFO_dev3_frame_image);
  gtk_box_pack_start (GTK_BOX (MY_INFO_dev3_frame_hbox), MY_INFO_dev3_frame_image, TRUE, TRUE, 0);

  MY_INFO_dev3_frame_label = gtk_label_new (_(" Ad-hoc peer information "));
  gtk_widget_show (MY_INFO_dev3_frame_label);
  gtk_box_pack_start (GTK_BOX (MY_INFO_dev3_frame_hbox), MY_INFO_dev3_frame_label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (MY_INFO_dev3_frame_label), GTK_JUSTIFY_LEFT);

  MY_INFO_dev3_label = gtk_label_new (_("eth2"));
  gtk_widget_show (MY_INFO_dev3_label);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (MY_INFO_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (MY_INFO_notebook), 2), MY_INFO_dev3_label);
  gtk_label_set_line_wrap (GTK_LABEL (MY_INFO_dev3_label), TRUE);

  MY_INFO_hbuttonbox = GTK_DIALOG (MY_INFO_dialog)->action_area;
  gtk_widget_show (MY_INFO_hbuttonbox);
  gtk_widget_set_size_request (MY_INFO_hbuttonbox, -2, 1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (MY_INFO_hbuttonbox), GTK_BUTTONBOX_END);

  g_signal_connect ((gpointer) MY_INFO_dialog, "destroy",
                    G_CALLBACK (on_MY_INFO_dialog_destroy),
                    NULL);
  g_signal_connect ((gpointer) MY_INFO_dev1_ok_button, "clicked",
                    G_CALLBACK (on_MY_INFO_dev1_ok_button_clicked),
                    NULL);
  g_signal_connect ((gpointer) MY_INFO_dev2_ok_button, "clicked",
                    G_CALLBACK (on_MY_INFO_dev2_ok_button_clicked),
                    NULL);
  g_signal_connect ((gpointer) MY_INFO_dev3_ok_button, "clicked",
                    G_CALLBACK (on_MY_INFO_dev3_ok_button_clicked),
                    NULL);

  GLADE_HOOKUP_OBJECT_NO_REF (MY_INFO_dialog, MY_INFO_dialog, "MY_INFO_dialog");
  GLADE_HOOKUP_OBJECT_NO_REF (MY_INFO_dialog, MY_INFO_vbox, "MY_INFO_vbox");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_notebook, "MY_INFO_notebook");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_frame, "MY_INFO_dev1_frame");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_fixed, "MY_INFO_dev1_fixed");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_nick_entry, "MY_INFO_dev1_nick_entry");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_g_addr_textview, "MY_INFO_dev1_g_addr_textview");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_mode_entry, "MY_INFO_dev1_mode_entry");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_l_addr_textview, "MY_INFO_dev1_l_addr_textview");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_mode_label, "MY_INFO_dev1_mode_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_nick_label, "MY_INFO_dev1_nick_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_l_addr_label, "MY_INFO_dev1_l_addr_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_s_addr_textview, "MY_INFO_dev1_s_addr_textview");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_g_addr_label, "MY_INFO_dev1_g_addr_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_s_addr_label, "MY_INFO_dev1_s_addr_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_ok_button, "MY_INFO_dev1_ok_button");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, alignment58, "alignment58");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, hbox74, "hbox74");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, image90, "image90");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, label152, "label152");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_frame_hbox, "MY_INFO_dev1_frame_hbox");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_frame_image, "MY_INFO_dev1_frame_image");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_frame_label, "MY_INFO_dev1_frame_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev1_label, "MY_INFO_dev1_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_frame, "MY_INFO_dev2_frame");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_fixed, "MY_INFO_dev2_fixed");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_nick_entry, "MY_INFO_dev2_nick_entry");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_g_addr_textview, "MY_INFO_dev2_g_addr_textview");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_mode_entry, "MY_INFO_dev2_mode_entry");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_l_addr_textview, "MY_INFO_dev2_l_addr_textview");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_mode_label, "MY_INFO_dev2_mode_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_nick_label, "MY_INFO_dev2_nick_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_l_addr_label, "MY_INFO_dev2_l_addr_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_s_addr_textview, "MY_INFO_dev2_s_addr_textview");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_g_addr_label, "MY_INFO_dev2_g_addr_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_s_addr_label, "MY_INFO_dev2_s_addr_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_ok_button, "MY_INFO_dev2_ok_button");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, alignment63, "alignment63");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, hbox84, "hbox84");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, image100, "image100");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, label186, "label186");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_frame_hbox, "MY_INFO_dev2_frame_hbox");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_frame_image, "MY_INFO_dev2_frame_image");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_frame_label, "MY_INFO_dev2_frame_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev2_label, "MY_INFO_dev2_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_frame, "MY_INFO_dev3_frame");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_fixed, "MY_INFO_dev3_fixed");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_nick_entry, "MY_INFO_dev3_nick_entry");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_g_addr_textview, "MY_INFO_dev3_g_addr_textview");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_mode_entry, "MY_INFO_dev3_mode_entry");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_l_addr_textview, "MY_INFO_dev3_l_addr_textview");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_mode_label, "MY_INFO_dev3_mode_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_nick_label, "MY_INFO_dev3_nick_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_l_addr_label, "MY_INFO_dev3_l_addr_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_s_addr_textview, "MY_INFO_dev3_s_addr_textview");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_g_addr_label, "MY_INFO_dev3_g_addr_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_s_addr_label, "MY_INFO_dev3_s_addr_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_ok_button, "MY_INFO_dev3_ok_button");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, alignment64, "alignment64");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, hbox86, "hbox86");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, image102, "image102");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, label193, "label193");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_frame_hbox, "MY_INFO_dev3_frame_hbox");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_frame_image, "MY_INFO_dev3_frame_image");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_frame_label, "MY_INFO_dev3_frame_label");
  GLADE_HOOKUP_OBJECT (MY_INFO_dialog, MY_INFO_dev3_label, "MY_INFO_dev3_label");
  GLADE_HOOKUP_OBJECT_NO_REF (MY_INFO_dialog, MY_INFO_hbuttonbox, "MY_INFO_hbuttonbox");

  gtk_widget_grab_focus (MY_INFO_dialog);
  gtk_widget_grab_default (MY_INFO_dialog);
  return MY_INFO_dialog;
}


*/

