#include "scan.h"


// peer config mode
#define ADHOC_POOR_MODE						0	// nothing 
#define ADHOC_LINK_MODE						1	// wireless device[1], only peer mode
#define ADHOC_SCATTER_MODE					2	// wireless device[2+], auto scatter mode enable
#define ADHOC_PROXY_MODE						3	// wireless device[2+], proxy enable checkbutton : TURE--> proxy global mode enable

#define ADHOC_DELAY_MODE						5

// peer_addr_info address type
#define REACHABLE_LINK_SCOPE			1
#define REACHABLE_SCATTER_SCOPE		2
#define REACHABLE_GLOBAL_SCOPE		3


// proxy toggle
#define ADHOC_PROXY_ENABLE					1
#define ADHOC_PROXY_DISABLE					0


// message type
#define DISCOVER_PEERS_REQUEST		1
#define DISCOVER_PEERS_RESPONSE		2
#define DISCOVER_ADGW_REQUEST		3
#define DISCOVER_ADGW_RESPONSE		4
#define ADVERTISEMENT_ADHOCGW		5
#define ADGW_JOIN_REQUEST				6
#define ADGW_JOIN_RESPONSE			7
#define SCATTER_PEERS_SOL				8
#define SCATTER_PEERS_ADV				9
#define CHAT_MESSAGE					10

#define CONFERENCE_CONNECT_REQUEST		11
#define CONFERENCE_CONNECT_RESPONSE		12

#define CONFERENCE_ALL					1
#define CONFERENCE_VIC					2
#define CONFERENCE_RAT					3
#define MESSENGER_CHAT					4

#define CONNECTION_ACCEPT				0
#define CONNECTION_REFUSE				1



// peer status 
#define UNCONFIRMED_LINK_PEER			0
#define CONFIRMED_SCATTER_PEER		1


// device status
#ifndef IF_UP
#define IF_UP							1
#endif
#ifndef IF_DOWN
#define IF_DOWN							0
#endif


// ipv6 address find
#ifndef IPV6_ADDR_GLOBAL
#define IPV6_ADDR_GLOBAL   0x0000U
#endif

#ifndef IPV6_ADDR_SITELOCAL
#define IPV6_ADDR_SITELOCAL   0x0040U
#endif

#ifndef IPV6_ADDR_LINKLOCAL
#define IPV6_ADDR_LINKLOCAL   0x0020U
#endif

#define PATH_PROC_NET_IF_INET6 "/proc/net/if_inet6"
#define PATH_PROC_AUTOCONF "/proc/sys/net/ipv6/conf"
#define PATH_PROC_FORWARDING "/proc/sys/net/ipv6/conf/all/forwarding"

#define AUTOCONF_ENABLE "1"
#define AUTOCONF_DISABLE "0"

#define FORWARDING_ENABLE "1"
#define FORWARDING_DISABLE "0"

#define DEFAULT_PEER_LIFETIME			4

#define MAX_MSG_LEN					2048

#define SAFE_WIRELESS_WAIT_TIME		1

#define NICK_NAME_SIZE				32



/***********************************
**      packet header **
***********************************/

typedef struct  {
	unsigned char		hdr1;
	unsigned char		class;
	unsigned short	flow;
	unsigned short	len;
	unsigned char		next;
	unsigned char		limit;
	unsigned char		src[16];
	unsigned char		dst[16];
} __attribute__((packed)) ipv6Hdr;



typedef struct  {
        unsigned char	type;
        unsigned char	code;
        unsigned short	chksum;
        unsigned char	hop;
        unsigned char	opt;
        unsigned short	lifetime;
        unsigned int		reachTime;
        unsigned int		retransTime;
} __attribute__((packed)) icmpv6Hdr;




typedef struct {
	unsigned char							mac[ETH_ALEN];			// address
}__attribute__((packed)) Mac_info;	

#define MAC_INFO_SIZE			sizeof(Mac_info)


typedef struct {
	unsigned char							address[16];			// address
	int									prefixlen;			// address prefix 	
}__attribute__((packed)) Address_info;	

#define ADDRESS_INFO_SIZE			sizeof(Address_info)

typedef struct {
	unsigned short						nicklen;
	unsigned char							nick[IW_ESSID_MAX_SIZE];
} __attribute__((packed)) Nickname_info;	

#define NICKNAME_INFO_SIZE			sizeof(Nickname_info)


typedef struct 
{
	//icmp header
	unsigned char							type;		
	unsigned char							code;		//0
	unsigned short						checksum;
	unsigned short						identifier;
	unsigned short						reserved;	//0x00
	// option
} __attribute__((packed)) Dis_peers_msg;

#define	DIS_PEERS_MSG_SIZE		sizeof(Dis_peers_msg)


typedef struct {
	unsigned char							type;
	unsigned	short						length;
	unsigned char							reserved[5];	
} __attribute__((packed)) Option_header;

#define OPTOIN_HEADER_SIZE				sizeof(Option_header)


typedef struct {
	unsigned char 						mode;				// peer_mode
	unsigned char 						scope;				// reachable scope
	Mac_info								mac_address;
	Address_info							ll_address;
	Address_info							s_address;
	Address_info							g_address;
	Nickname_info						nickname;
} __attribute__((packed)) Peers_info_opt;

#define PEERS_INFO_OPT_SIZE			sizeof(Peers_info_opt)


typedef struct {
	unsigned char 						type;				// conference_all || conference_vic || conference_rat
	unsigned char 						code;				// connection_refuse || connection_accept
	Address_info							g_address;
	Nickname_info						nickname;
} __attribute__((packed)) Conference_info_opt;

#define CONFERENCE_INFO_OPT_SIZE			sizeof(Conference_info_opt)



typedef struct {
	Nickname_info						nickname;
	char									msg[MAX_MSG_LEN];
} __attribute__((packed)) Message_info_opt;







struct adhoc_windows_info
{
	int						active_adgw_window;
	int						active_proxy_window;
	int						active_config_window;
};


struct sock_info
{
	int 									bpf_sock;
	int									icmpv6_sock;
	int									udp_sock;
};


struct interface_info_entry
{
	struct list_head						list;

	int									ifmode;					// wireless || wire	
	unsigned int							ifindex;					// interface row index

	int									scope;					// reachable scope [ link scope or scatter scope]
	int									proxy_enable;			// PROXY_ENABLE or PROXY_DISABLE
	
	unsigned char 						dev_name[IFNAMSIZ + 1];
	unsigned char							mac[ETH_ALEN+1];

	struct sock_info						socks;
	struct wlan_info						curr_wlan_info;			// current wireless info

	Address_info							ll_address;					// link-local address
	Address_info							s_address;					// ad-hoc global address
	Address_info							g_address;					// scatter or proxy global real address
	Nickname_info						nickname;					// adhoc wireless nickname
};




struct peers_info_entry
{
	struct list_head						list;

	char									dev_index[IFNAMSIZ + 1];
	
	int									mode;                                       // working mode[LINK, SCATTER, PROXY]
	int									status;						// adgw joining status[unconfirmed, confirmed]
	
	unsigned char							mac_address[ETH_ALEN+1];	

	Address_info							ll_address;					// link-local address
	Address_info							s_address;					// ad-hoc global address
	Address_info							g_address;					// scatter or proxy global real address
	Nickname_info						nickname;					// adhoc wireless nickname
	
	int			          					lifetime;						 // peer expire llifetime
	void                    						(*callback_function)(struct peers_info_entry *entry); // callback function
};


struct peer_config
{
	int									mode;					// poor || link || scatter || proxy 

	int									wait_time;				// advertisement or wireless status change delay time [config file]
	int									scan_sock;				// wireless scanning or mode change socket
	int									chat_sock;				// send & recv chatting message socket
	unsigned short						udp_port;				// signaling packet [config file]
	unsigned short						chat_port;				// chatting packet

	struct list_head						devices;					// all device info list
	struct adhoc_windows_info				window;					// active winodws info
	
	struct in6_addr						default_prefix;			// default ad-hoc global address prefix[config file]
	struct in6_addr						gw_addr;
	
	struct peers_info_entry					*curr_adhoc_gw;		//  join adhoc gateway
	struct interface_info_entry 				*curr_if_focus;			// current focusing device index
	struct interface_info_entry				*proxy_if;				// current proxy setting device index

	struct list_head						adhoc_network_list;		// ad-hoc network list [ for ad-hoc config ]
	struct list_head						infra_network_list;		// infrastructure network list [for proxy config ]
	struct list_head						adgws_list;				// ad-hoc gateways list
	struct list_head						peers_list; 				// all peers list
};






//packet passingϰ   ġ ȯѴ 
int detect_packet_type(unsigned char * buf, int* len);
int handle_adhoc_msg(void);
int receive_udp_process(struct interface_info_entry *iface, int sock);
int receive_icmp_process(struct interface_info_entry *iface, int sock);
int handle_conference_connect_request(struct interface_info_entry *iface, char *buf, int len);
int handle_conference_connect_response(struct interface_info_entry *iface, char *buf, int len);
int handle_chat_message(struct interface_info_entry *iface, char *buf, int len);
int handle_dis_peers_req(struct interface_info_entry *iface, char *buf, int len);
int handle_dis_peers_res(struct interface_info_entry *iface, char *buf, int len);
int handle_dis_adgw_adv(struct interface_info_entry *iface, char *buf, int len);
int handle_adgw_join_req(struct interface_info_entry *iface, char *buf, int len);
int handle_adgw_join_res(struct interface_info_entry *iface, char *buf, int len);
int send_dis_peers_req(struct interface_info_entry *iface, struct in6_addr *target);
int send_dis_adgw_adv(struct interface_info_entry *iface, struct in6_addr *target);
int send_dis_peers_res(struct interface_info_entry *iface, unsigned short id, struct peers_info_entry *peer);
int send_adgw_join_req(struct interface_info_entry *iface, struct peers_info_entry *peer);
int send_adgw_join_res(struct interface_info_entry *iface, struct peers_info_entry *peer);
int send_message(struct interface_info_entry *iface, char *message, struct peers_info_entry *peer);
void create_opt_header(unsigned char *msg, unsigned char type, int size);
void create_peers_info_opt(unsigned char *msg, unsigned char mode, unsigned char scope, unsigned char *mac,  Address_info *ll_address, Address_info *s_address, Address_info *g_address, Nickname_info *nickname);
void create_conference_info_opt(unsigned char *msg, unsigned char type, unsigned char code, Address_info *g_address, Nickname_info *nickname);
void create_chat_message_opt(unsigned char *msg, Nickname_info *nickname, char *message);
void add_interface_entry(struct interface_info_entry *iface);
void	add_peers_entry(struct peers_info_entry *peer);
void	add_adgw_entry(struct peers_info_entry *peer);
void	del_list_entry(struct list_head *entry);
struct interface_info_entry *search_interface_by_lladdress(char *lladdr);
struct interface_info_entry *search_interface_by_name(char *name);
struct ap_entry *search_infra_by_essid(char *essid);
struct ap_entry *search_adhoc_by_essid(char *essid);
struct peers_info_entry *search_adgw_by_lladdress(char *lladdr);
struct peers_info_entry *search_adgw_by_nick(char *nickname);
struct peers_info_entry *search_peer_by_lladdress(char *lladdr);
struct peers_info_entry *search_peer_by_nick(char *nickname);
void expire_entry_time_handle();
void time_handle(struct timeval *callback_time, struct timeval *tv);
/* returns 1, 0, -1 when tv1 is greater, equal or less than tv2 */
int cmp_timeval(struct timeval *tv1, struct timeval *tv2);
void add_usecs(struct timeval *tv, unsigned int usecs);
/* identifier Ѵ */
unsigned short make_identifier();
void adhoc_advertisement(void);
void flush_scatter_peers();
int print_icmp_msg_info(char *title, char *dev, unsigned char *msg, int len, struct in6_addr *addr);

