/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <pthread.h>
#include <signal.h>


#include "dis_peers.h"
#include "callbacks.h"
#include "interface.h"
#include "scan.h"
#include "network_util.h"
#include "bpf_filter.h"

//#define USE_IPW2100 	

#ifdef USE_IPW2100
static unsigned char ipw2100_vid[3]={0x00, 0x04, 0x23};	
struct pci_ipw2100
{
	int 	use_enable;
	int	curr_mode;
	char dev[IFNAMSIZ+1];
};

struct pci_ipw2100 ipw2100;
#endif


struct peer_config config;
struct adhoc_windows window_box;


#define CONFIG_FILE_PATH "/etc/.skt_adhoc.conf"


/* Process loading of the mn_data
 * Return values: -2: consistency error, -1: error, 0: ok, 1: end */

static int process_load(void *voidptr, char *key, char *data)
{
	struct peer_config *cfg;

	cfg = voidptr;

#ifdef USE_IPW2100
	if (strcmp(key, "USE_IPW2100_WLAN") == 0) {
		if (load_bool(data, &ipw2100.use_enable) == TRUE)
			return 0;
		return -1;
	}
#endif	
	
	if (strcmp(key, "UDP_PORT") == 0) {
		if (load_int(data, (int*)&cfg->udp_port) == TRUE)
		{
			cfg->chat_port = cfg->udp_port+1;
			return 0;
		}
		return -1;
	}

	if (strcmp(key, "WAIT_TIME_DELAY") == 0) {
		if (load_int(data, (int*)&cfg->wait_time) == TRUE) 
		{
			return 0;
		}
		return -1;
	}
	if (strcmp(key, "DEFAULT_PREFIX") == 0) {
		int dummy=0;
		if (load_ip6_addr(data, &cfg->default_prefix, &dummy) == TRUE)
		{
			return 1;
		}
		return -1;
	}
	return 0;
}


int load_file(struct peer_config *cfg)
{
	FILE *file;

	cfg->udp_port = DEFAULT_UDP_PORT;

	//config reading
	file = fopen(CONFIG_FILE_PATH, "r");
	if(file == NULL) {
		fprintf(stderr,"Could not open configuration file '%s'.\n", CONFIG_FILE_PATH);
		return FALSE;
	}
	if(load_data(cfg, file, process_load)== FALSE) {
		fprintf(stderr,"client : Error while interpreting file '%s'!\n",CONFIG_FILE_PATH);
		return FALSE;
	}
	fclose(file);
	return TRUE;
}



int init_config(void)
{
	memset(&config, 0, sizeof(struct peer_config));

	if (load_file(&config) ==  FALSE) 
	{
		fprintf(stderr, "Configuration file reading failed.\n");
		return -1;
	}
	return 0;
}

void clean_up(int sig)
{
        if( !list_empty(&config.devices) )
        {
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)
		{
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

			init_global_address(entry->dev_name);
			cleanup_route_info(entry->dev_name, (struct in6_addr *)entry->s_address.address, 64);
			if((entry->scope > REACHABLE_LINK_SCOPE) || (config.mode > ADHOC_LINK_MODE))
				cleanup_route_info(entry->dev_name, (struct in6_addr *)entry->g_address.address, 64);

			default_router_del(entry->dev_name, NULL);

			change_autoconf_status(entry->dev_name, AUTOCONF_ENABLE);
		}
        }

	exit(sig);
}

void init_process()
{
	int cnt=0;

	signal(SIGHUP, clean_up);
	signal(SIGTERM, clean_up);
	signal(SIGINT, clean_up);
	signal(SIGSEGV, clean_up);

	if( init_config() < 0 )
	{
		printf("[init_process]Invalid configuration file\n");
		exit(0);
	}
	INIT_LIST_HEAD(&config.devices);

	if_devices();
	
	iw_devices();

	if( !list_empty(&config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		config.scan_sock = iw_sockets_open();
		config.chat_sock  = open_udpsock6(config.chat_port, NULL);
		
		list_for_each_safe(p, n, lead)	
		{
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);
			struct in6_addr gw_addr;

			printf("[init_process] Device name : [%s]\n", entry->dev_name);
			
			if( get_ipv6_addr(entry->dev_name, IPV6_ADDR_LINKLOCAL,(struct in6_addr *)entry->ll_address.address, &entry->ifindex) < 0 )
			{
				printf("[init_process] Not found link local address [%s]\n", entry->dev_name);
				del_list_entry((struct list_head *)entry);
				continue;
			}
			
			debug_hexdump("[init_process] Link-local address :", entry->ll_address.address, 16);
			entry->ll_address.prefixlen = 64;

			default_router_del(entry->dev_name, NULL);

			change_autoconf_status(entry->dev_name, AUTOCONF_DISABLE);
			init_global_address(entry->dev_name);

			get_mac_addr(entry->mac, entry->dev_name);
			debug_hexdump("[init_process] Mac address : \n", entry->mac, ETH_ALEN);			

			global_address_conf(entry->dev_name, &config.default_prefix, 64, entry->mac, &entry->s_address);
			sleep(SAFE_WIRELESS_WAIT_TIME*2);
			
			if( entry->ifmode == WIRELESS_INTERFACE )
			{
#ifdef USE_IPW2100
				if(ipw2100.use_enable == TRUE)
				{
					if( !memcmp(entry->mac, ipw2100_vid, 3) )
					{
						char buf[128]={0x00,};

						strcpy(ipw2100.dev, entry->dev_name);
						sprintf(buf, "ifdown %s\n", ipw2100.dev);
						system(buf);
						system("modprobe -r ipw2100");
						system("modprobe ipw2100 mode=1");
					}
				}
				else
				{
					char buf[128]={0x00,};

					sprintf(buf, "ifdown %s\n", ipw2100.dev);
					system(buf);
					system("modprobe -r ipw2100");

					printf("[init_process] Terminated IPW2100 module. dev[%s]\n", entry->dev_name);
					del_list_entry((struct list_head *)entry);
					continue;					
				}
					
#endif				
				printf("[init_process] Wirless Device [%s]\n", entry->dev_name);
				if(get_curr_wireless_info(config.scan_sock, entry->dev_name, &entry->curr_wlan_info) < 0 )
				{
					printf("[init_process] Get current wireless information fail[%s]\n", entry->dev_name);
					del_list_entry((struct list_head *)entry);
					continue;
				}
				strcpy(entry->nickname.nick, entry->curr_wlan_info.nickname);
				entry->nickname.nicklen = strlen(entry->nickname.nick);			
			}

			entry->socks.bpf_sock = open_BPF_socket(entry->dev_name);
			entry->socks.icmpv6_sock = open_icmpv6_socket();
			entry->socks.udp_sock = open_udpsock6(config.udp_port, (struct in6_addr *)entry->s_address.address);			

			entry->scope = REACHABLE_LINK_SCOPE;				
			entry->proxy_enable = ADHOC_PROXY_DISABLE;

			cnt++;
			printf("\n\n");
		}

		INIT_LIST_HEAD(&config.adhoc_network_list);
		INIT_LIST_HEAD(&config.infra_network_list);
		INIT_LIST_HEAD(&config.adgws_list);
		INIT_LIST_HEAD(&config.peers_list);
	}

	if( cnt == 0 )
	{
		printf("[init_process] Not Found Available Interface\n");
		exit (-1);
	}
	else if(cnt ==1 )
		config.mode = ADHOC_LINK_MODE;
	else if(cnt > 1 )
	{
		config.mode = ADHOC_SCATTER_MODE;
		init_scatter_autoconf();
	}
	return ;

}

int create_thread()
{
	pthread_t tid;

	pthread_create(&tid, NULL, (void *)handle_adhoc_msg, (void *) NULL);

	return tid;
}


void send_dis_all_interface(void)
{
	if(config.mode != ADHOC_LINK_MODE )
		return ;
	
	if( !list_empty(&config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)			
		{
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

			if( (entry->ifmode == WIRELESS_INTERFACE) && (entry->curr_wlan_info.mode == IW_MODE_ADHOC) )
				send_dis_peers_req(entry, NULL);
		}
	}
}



int
main (int argc, char *argv[])
{
	int cnt, result=FALSE;
	
#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif

	g_thread_init(NULL);
	gdk_threads_init();


	gtk_set_locale ();
	gtk_init (&argc, &argv);

	add_pixmap_directory (PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");

	gdk_threads_enter();

	window_box.MAIN_window = create_MAIN_window ();
	gtk_widget_show (window_box.MAIN_window);

	popup_info("Success.. Starting..\n");

	init_process();

	create_thread();

	send_dis_all_interface();


	gtk_main ();
	
	gdk_threads_leave();
	
	return 0;
}


