#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <fnmatch.h>

//for netlink
#include <asm/types.h>
//#include <net/if.h>
#include <netinet/ip.h>
#include <errno.h>
//getnameinfo
#include <sys/socket.h>
#include <netdb.h>

#include <asm/byteorder.h>
#include <time.h>


//#include <linux/if_tunnel.h>
#include <linux/netlink.h>
#include <linux/rtnetlink.h>
#include <net/if_arp.h>
#include <linux/ipv6_route.h>


#include "fileio.h"
#include "network_util.h"
#include "debug.h"
#include "dis_peers.h"

#ifndef u8
#define u8 __u8
#endif

#ifndef u16
#define u16 __u16
#endif

#ifndef u32
#define u32 __u32
#endif

#define LLADDR_MAX  16

//file open error    
int netlink_fd =0;
int change_flag_fd =0;

extern struct peer_config config;



void change_network(struct interface_info_entry *iface);
int initialize_scatter_network(struct interface_info_entry *iface);
int initialize_proxy_network(struct interface_info_entry *iface);
void global_address_conf(unsigned char *dev, 	unsigned char *prefix, int plen, unsigned char *mac_addr, Address_info *g_address);
int wait_proxy_global_address(char *device, Address_info *addr_info);
void make_address_conf(unsigned char *dev, unsigned char *prefix, unsigned short plen, unsigned char *mac_addr, Address_info *g_address);
void make_proxy_address_conf(unsigned char *dev, unsigned char *prefix, unsigned short plen, unsigned char *mac_addr, Address_info *g_address);

//////////////////////////////////////////////
/////////////////////////////////////////////
///////////////route & neighber  ////////////
/////////////////////////////////////////////
/////////////////////////////////////////////

#ifndef MSG_TRUNC
#define MSG_TRUNC 0x20
#endif


struct rtnl_handle
{
	int fd;
	struct sockaddr_nl local;
	__u32 seq;
	__u32 dump;
};

struct idxmap
{
	struct idxmap * next;
	int		index;
	int		type;
	int		alen;
	unsigned	flags;
	unsigned char	addr[8] ;
	char		name[16];
};

static struct idxmap *idxmap[16];

/**
 * rtnl_open:
 * @rth: 
 * @subscriptions: 
 * 
 * 
 * 
 * Returns: 
 */
static int rtnl_open(struct rtnl_handle *rth, unsigned subscriptions)
{
	unsigned int len;

	if(rth == NULL){
		syslog(LOG_ERR,"[rtnl_open()] rth == NULL \n");
		return -1;
	}
	if(netlink_fd ==0){
		netlink_fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
		rth->fd =netlink_fd; 
	if (rth->fd < 0) {
			syslog(LOG_ERR,"[rtnl_open()] cannot open netlink socket: %s\n",  strerror(errno));
			netlink_fd =0 ;
		return -1;
	}

	memset(&rth->local, 0, sizeof(rth->local));
	rth->local.nl_family = AF_NETLINK;
	rth->local.nl_groups = subscriptions;

	if (bind(rth->fd, (struct sockaddr *) &rth->local,
		 sizeof(rth->local)) < 0) {
			syslog(LOG_ERR, "[rtnl_open()] cannot bind netlink socket: %s\n", strerror(errno));
		close(rth->fd);
		netlink_fd =0;
		return -1;
	}

        }else{
		rth->fd = netlink_fd;
		memset(&rth->local, 0, sizeof(rth->local));
		rth->local.nl_family = AF_NETLINK;
		rth->local.nl_groups = subscriptions;

	}

	len = sizeof(rth->local);
	if (getsockname(rth->fd, (struct sockaddr *) &rth->local, &len) < 0) {
		syslog(LOG_ERR, "[rtnl_open()] cannot getsockname: %s\n", strerror(errno));
		close(rth->fd);
		netlink_fd =0;

		return -1;
	}
	if (len != sizeof(rth->local)) {
		syslog(LOG_ERR, "[rtnl_open()] wrong address length %i != %i\n", len, sizeof(rth->local));
		close(rth->fd);
		netlink_fd =0;

		return -1;
	}

	if (rth->local.nl_family != AF_NETLINK) {
		syslog(LOG_ERR, "[rtnl_open()] wrong address family %i != %i\n",  rth->local.nl_family, AF_NETLINK);
		close(rth->fd);
		netlink_fd =0;
		return -1;
	}
	rth->seq = time(NULL);
	return 0;
}


/**
 * rtnl_talk:
 * @rth: 
 * @n: 
 * @answer: 
 * 
 * 
 * 
 * Returns: 
 */
static int rtnl_talk(struct rtnl_handle *rth, struct nlmsghdr *n,
		     struct nlmsghdr *answer)
{
	int status;
	struct nlmsghdr *h;
	struct sockaddr_nl nladdr;
	struct iovec iov;
	char   buf[8192] ={0x00,};
	struct msghdr msg = {
		(void *) &nladdr, sizeof(nladdr),
		&iov, 1,
		NULL, 0,
		0
	};

	iov.iov_base = (void*) n;
	iov.iov_len = n->nlmsg_len;

	memset(&nladdr, 0, sizeof(nladdr));
	nladdr.nl_family = AF_NETLINK;
	nladdr.nl_pid = 0;
	nladdr.nl_groups = 0;

	n->nlmsg_seq = ++rth->seq;

	if (answer == NULL)
		n->nlmsg_flags |= NLM_F_ACK;

	status = sendmsg(rth->fd, &msg, 0);

	if (status < 0) {
		syslog(LOG_ERR,"[rtnl_talk()] sendmsg: %s\n", strerror(errno));
		return -1;
	}

	iov.iov_base = buf;
	iov.iov_len = sizeof(buf);

	do {
		status = recvmsg(rth->fd, &msg, 0);

		if (status < 0) {
			if (errno == EINTR)
				continue;
			syslog(LOG_ERR,"[rtnl_talk()] OVERRUN: %s\n", strerror(errno));
			continue;
		}
		if (status == 0) {
			syslog(LOG_ERR,"[rtnl_talk()] EOF on netlink\n");
			return -1;
		}
		if (msg.msg_namelen != sizeof(nladdr)) {
			syslog(LOG_ERR, "[rtnl_talk()] sender address length == %i != %i\n",
			      msg.msg_namelen, sizeof(nladdr));
			return -1;
		}
		for (h = (struct nlmsghdr*) buf; status >= sizeof(*h); ) {
			int len = h->nlmsg_len;
			pid_t pid = h->nlmsg_pid;
			int l = len - sizeof(*h);
			unsigned seq = h->nlmsg_seq;

			if (l < 0 || len > status) {
				if (msg.msg_flags & MSG_TRUNC) {
					printf("[rtnl_talk()] truncated message\n");
					return -1;
				}
				printf("[rtnl_talk()] malformed message: len=%d\n",  len);
				return -1;
			}

			if (h->nlmsg_pid != pid || h->nlmsg_seq != seq) {
				/* junk */
				continue;
			}

			if (h->nlmsg_type == NLMSG_ERROR) {
				struct nlmsgerr *err =
					(struct nlmsgerr *) NLMSG_DATA(h);
				if (l < sizeof(struct nlmsgerr)) {
					syslog(LOG_ERR, "[rtnl_talk()] ERROR truncated\n");
					return -1;
				}

				errno = -err->error;
				//coljehui : ENOBUFS ̹Ƿ 		
				//if (errno == 0 ) {
				if (errno == 0 || errno == 105) {
					if (answer)
						memcpy(answer, h,
						       h->nlmsg_len);
					return 0;
				}
				printf( "[rtnl_talk()] RTNETLINK error: "
				      "%s\n", strerror(errno));
				syslog(LOG_ERR,"[rtnl_talk()] RTNETLINK error: "
				      "%s\n", strerror(errno));
				return -1;
			}

			if (answer) {
				memcpy(answer, h, h->nlmsg_len);
				return 0;
			}

			syslog(LOG_ERR,"[rtnl_talk()] unexpected reply\n");

			status -= NLMSG_ALIGN(len);
			h = (struct nlmsghdr *)
 				(((char *) h) + NLMSG_ALIGN(len));
		}
		if (msg.msg_flags & MSG_TRUNC) {
			printf("[rtnl_talk()] message truncated\n");
			continue;
		}
		if (status) {
			syslog(LOG_ERR, "[rtnl_talk()] remnant of size %i\n", status);
			return -1;
		}
	} while (1);
}


/**
 * addattr32:
 * @n: 
 * @maxlen: 
 * @type: 
 * @data: 
 * 
 * 
 * 
 * Returns: 
 */
static int addattr32(struct nlmsghdr *n, int maxlen, int type, __u32 data)
{
	int len;
	struct rtattr *rta;

	len = RTA_LENGTH(4);
	if (NLMSG_ALIGN(n->nlmsg_len) + len > maxlen)
		return -1;
	rta = (struct rtattr *)(((char *) n) + NLMSG_ALIGN(n->nlmsg_len));
	rta->rta_type = type;
	rta->rta_len = len;
	memcpy(RTA_DATA(rta), &data, 4);
	n->nlmsg_len = NLMSG_ALIGN(n->nlmsg_len) + len;
	return 0;
}


/**
 * addattr_l:
 * @n: 
 * @maxlen: 
 * @type: 
 * @data: 
 * @alen: 
 * 
 * 
 * 
 * Returns: 
 */
 //ROUTING TABLE ATTRIBUTE  nlmsghdr ߰ 
static int addattr_l(struct nlmsghdr *n, int maxlen, int type, const void *data, int alen)
{
	int len;
	struct rtattr *rta;

	len = RTA_LENGTH(alen);
	if (NLMSG_ALIGN(n->nlmsg_len) + len > maxlen)
		return -1;
	rta = (struct rtattr *)(((char *) n) + NLMSG_ALIGN(n->nlmsg_len));
	rta->rta_type = type;
	rta->rta_len = len;
	memcpy(RTA_DATA(rta), data, alen);
	n->nlmsg_len = NLMSG_ALIGN(n->nlmsg_len) + len;
	return 0;
}


static int rtnl_wilddump_request(struct rtnl_handle *rth, int family, int type)
{
	struct {
		struct nlmsghdr nlh; //NetLink signal header
		struct rtgenmsg g;   //Route table Generation Message
	} req;
	struct sockaddr_nl nladdr;

	memset(&nladdr, 0, sizeof(nladdr));
	
	memset(&req, 0, sizeof(nladdr));
	
	//AF_ROUTE=AF_NETLINK
	nladdr.nl_family = AF_NETLINK;

	req.nlh.nlmsg_len = sizeof(req);
	req.nlh.nlmsg_type = type; 
	req.nlh.nlmsg_flags = NLM_F_ROOT | NLM_F_MATCH | NLM_F_REQUEST;  //Tree Root|return all matching|It is request message. 
	req.nlh.nlmsg_pid = 0;
	req.nlh.nlmsg_seq = rth->dump = ++rth->seq;
	req.g.rtgen_family = family;

       //sendto() struct msghdr  sendmsg() .
       
	return sendto(rth->fd, (void *) &req, sizeof(req), 0,
		      (struct sockaddr *) &nladdr, sizeof(nladdr));
}


static int rtnl_dump_filter(struct rtnl_handle *rth,
			    int (*filter)(struct sockaddr_nl *,
					  struct nlmsghdr *n, void *),
			    void *arg1)
{
	char buf[8192] ={0x00,};
	struct sockaddr_nl nladdr;
	struct iovec iov = { buf, sizeof(buf) };

	do {
		int status;
		struct nlmsghdr *h;

		struct msghdr msg = {
			(void *) &nladdr, sizeof(nladdr),
			&iov,	1,
			NULL,	0,
			0
		};

		status = recvmsg(rth->fd, &msg, 0);

		if (status < 0) {
			if (errno == EINTR)
				continue;
			syslog(LOG_ERR, "[rtnl_dump_filter()] recvmsg OVERRUN: %s\n",  strerror(errno));
			continue;
		}
		if (status == 0) {
			printf("[rtnl_dump_filter()] EOF on netlink\n");
			return -1;
		}
		
		//nladdr ==msg_name
		if (msg.msg_namelen != sizeof(nladdr)) {
			syslog(LOG_ERR,  "[rtnl_dump_filter()] sender address length "
			      "== %i\n", msg.msg_namelen);
			return -1;
		}

		h = (struct nlmsghdr *) buf;
		//NLMSG_OK : status>0 ̰ (h)->nlmsg_len >= sizeof(struct nlmsghdr) ̰ 
		//h->nlmsg_len <= status  
		while (NLMSG_OK(h, status)) {
			int err;
                       
			if (h->nlmsg_pid != rth->local.nl_pid ||
			    h->nlmsg_seq != rth->dump) {
				printf( "[rtnl_dump_filter()] junk?\n");
				h = NLMSG_NEXT(h, status);
				continue;
			}
			
                       /*NLMSG_DONE :  End of a dump	*/
			if (h->nlmsg_type == NLMSG_DONE)
				return 0;
			if (h->nlmsg_type == NLMSG_ERROR) {
				//err  , struct nlmsghdr  struct struct nlmsgerr̴ .
				//NLMSG_DATA :  struct nlmsghdr  ġ  
				struct nlmsgerr *err =
					(struct nlmsgerr *) NLMSG_DATA(h);
				if (h->nlmsg_len <
				    NLMSG_LENGTH(sizeof(struct nlmsgerr))) {
					syslog(LOG_ERR,"[rtnl_dump_filter()] ERROR truncated\n");
				} else {
					errno = -err->error;
					syslog(LOG_ERR,  "[rtnl_dump_filter()] RTNETLINK error: %s\n", strerror(errno));
				}
				return -1;
			}
			err = filter(&nladdr, h, arg1);
			if (err < 0)
				return err;

			//(status -= NLMSG_ALIGN(h->nlmsg_len) ,  (struct nlmsghdr*)(h + NLMSG_ALIGN(h->nlmsg_len))  
                       // struct nlmsghdr ŭ ü  ̰ ° struct nlmsghdr Ѵ.
			h = NLMSG_NEXT(h, status);
		}
		if (msg.msg_flags & MSG_TRUNC) {
			printf( "[rtnl_dump_filter()] message truncated\n");
			continue;
		}
		if (status) {
			syslog(LOG_ERR, "[rtnl_dump_filter()] remnant of size %d\n", status);
			return -1;
		}
	} while (1);
}

//routing table attributes rta_type index Ͽ  tb 迭 Ͽ Ѵ.
static int parse_rtattr(struct rtattr *tb[], int max, struct rtattr *rta,
			int len)
{
	while (RTA_OK(rta, len)) {
		if (rta->rta_type <= max)
			tb[rta->rta_type] = rta;
		rta = RTA_NEXT(rta,len);
	}
	if (len)
		syslog(LOG_ERR,"[parse_rtattr()] Deficit %d, rta_len=%d\n",  len, rta->rta_len);
	return 0;
}

static  int ll_remember_index(struct sockaddr_nl *who, struct nlmsghdr *n, void *arg)
{
	int h;
	struct ifinfomsg *ifi = NLMSG_DATA(n);
	struct idxmap *im, **imp;
	struct rtattr *tb[IFLA_MAX+1] ={0x00,};

	if (n->nlmsg_type != RTM_NEWLINK)
		return 0;

	if (n->nlmsg_len < NLMSG_LENGTH(sizeof(ifi)))
		return -1;


	memset(tb, 0, sizeof(tb));
	parse_rtattr(tb, IFLA_MAX, IFLA_RTA(ifi), IFLA_PAYLOAD(n));
	if (tb[IFLA_IFNAME] == NULL)
		return 0;

	h = ifi->ifi_index&0xF;

	for (imp=&idxmap[h]; (im=*imp)!=NULL; imp = &im->next)
		if (im->index == ifi->ifi_index)
			break;

	if (im == NULL) {
		im = malloc(sizeof(*im));
		if (im == NULL)
			return 0;
		im->next = *imp;
		im->index = ifi->ifi_index;
		*imp = im;
	}

	im->type = ifi->ifi_type;
	im->flags = ifi->ifi_flags;
	if (tb[IFLA_ADDRESS]) {
		int alen;
		im->alen = alen = RTA_PAYLOAD(tb[IFLA_ADDRESS]);
		if (alen > sizeof(im->addr))
			alen = sizeof(im->addr);
		memcpy(im->addr, RTA_DATA(tb[IFLA_ADDRESS]), alen);
	} else {
		im->alen = 0;
		memset(im->addr, 0, sizeof(im->addr));
	}
	strcpy(im->name, RTA_DATA(tb[IFLA_IFNAME]));
	return 0;
}

static int ll_name_to_index(char *name)
{
	static char ncache[16]= {0x00, };
	static int icache;
	struct idxmap *im;
	int i;

	if (name == NULL)
		return 0;
	if (icache && strcmp(name, ncache) == 0)
		return icache;
	for (i=0; i<16; i++) {
		for (im = idxmap[i]; im; im = im->next) {
			if (strcmp(im->name, name) == 0) {
				icache = im->index;
				strcpy(ncache, name);
				return im->index;
			}
		}
	}
	return 0;
}

static int ll_init_map(struct rtnl_handle *rth)
{
	if (rtnl_wilddump_request(rth, AF_UNSPEC, RTM_GETLINK) < 0) {
		perror("Cannot send dump request");
		exit(1);
	}

	if (rtnl_dump_filter(rth, ll_remember_index, &idxmap) < 0) {
		fprintf(stderr, "Dump terminated\n");
		exit(1);
	}
	return 0;
}


//////////////////////////////
//////////////////////////////
static int get_ctl_fd(void)
{
	int s_errno;
	int fd;

//	fd = socket(PF_INET, SOCK_DGRAM, 0);
//	if (fd >= 0)
//		return fd;
//	s_errno = errno;
	
	fd = socket(PF_INET6, SOCK_DGRAM, 0);
	if (fd >= 0)
		return fd;

	s_errno = errno;
	fd = socket(PF_PACKET, SOCK_DGRAM, 0);
	if (fd >= 0)
		return fd;

	errno = s_errno;
	perror("Cannot create control socket");
	return -1;
}

static int do_chflags(char *dev, __u32 flags, __u32 mask)
{
	struct ifreq ifr;
	int fd;
	int err;

	strcpy(ifr.ifr_name, dev);
	
	if(change_flag_fd==0){
		 change_flag_fd= get_ctl_fd();
		if (change_flag_fd < 0){
			change_flag_fd = 0;
		return -1;
		}
	}	
	fd =change_flag_fd;
	
	err = ioctl(fd, SIOCGIFFLAGS, &ifr);
	if (err) {
		perror("SIOCGIFFLAGS");
		change_flag_fd = 0;
		close(fd);
		return -1;
	}
	if ((ifr.ifr_flags^flags)&mask) {
		ifr.ifr_flags &= ~mask;
		ifr.ifr_flags |= mask&flags;
		err = ioctl(fd, SIOCSIFFLAGS, &ifr);
		if (err){
			change_flag_fd = 0;
			perror("SIOCSIFFLAGS");
	close(fd);
		}
	}
	
	return err;
}

int ip6_link_set(int cmd, char *device)
{
	__u32 mask = 0;
	__u32 flags = 0;

	if(cmd)
	{	mask |= IFF_UP;
		flags|= IFF_UP;
	}
	else
	{
		mask |= IFF_UP;
		flags &= ~IFF_UP;		
	}
	
	if (mask)
		return do_chflags(device, flags, mask);
	
	return 0;
}



//////////////////////////////////////////////////////////////
///////////////////// CLEAR ROUTE INFO ////////////////////////
//////////////////////////////////////////////////////////////


/* Like strncpy but make sure the resulting string is always 0 terminated. */  
char *safe_strncpy(char *dst, const char *src, size_t size)
{   
    dst[size-1] = '\0';
    return strncpy(dst,src,size-1);   
}

#ifndef IN6_IS_ADDR_UNSPECIFIED
#define IN6_IS_ADDR_UNSPECIFIED(a) \
        (((__u32 *) (a))[0] == 0 && ((__u32 *) (a))[1] == 0 && \
         ((__u32 *) (a))[2] == 0 && ((__u32 *) (a))[3] == 0)
#endif

#ifndef RTF_GATEWAY
#define	RTF_GATEWAY	0x0002		/* destination is a gateway	*/
#endif
#ifndef RTF_DEFAULT
#define RTF_DEFAULT	0x00010000	/* default - learned via ND	*/
#endif 
#ifndef RTF_ADDRCONF
#define RTF_ADDRCONF	0x00040000	/* addrconf route - RA		*/
#endif 
#ifndef RTF_CACHE
#define RTF_CACHE	0x01000000	/* cache entry			*/
#endif 
#ifndef RTF_UP
#define RTF_UP          0x0001          /* route usable       */
#endif 
#ifndef RTF_HOST
#define RTF_HOST        0x0004          /* host entry (net otherwise)   */
#endif 


int neigh_add(char *dev, struct in6_addr *addr, char *lladdr)
{
	printf("[neigh add()] device : %s\n", dev); 
	debug_hexdump("\t target address : ", (char*)addr, sizeof(struct in6_addr));
	debug_hexdump("\t target link address : ", lladdr, ETH_ALEN);

	return ip6_neigh_modify(RTM_NEWNEIGH, NLM_F_CREATE|NLM_F_REPLACE, 0, addr, lladdr, dev);
}

int neigh_del(char *dev, struct in6_addr *addr)
{
	printf("[neigh del()] device : %s\n", dev); 
	debug_hexdump("\t target address : ", (char*)addr, sizeof(struct in6_addr));

	return ip6_neigh_modify(RTM_DELNEIGH, 0, 0, addr, NULL, dev);
}



int neigh_add_proxy(char *dev, struct in6_addr *addr)
{
	printf("[neigh add proxy()] device : %s\n", dev); 
	debug_hexdump("\t target address : ", (char*)addr, sizeof(struct in6_addr));

	return ip6_neigh_modify(RTM_NEWNEIGH, NLM_F_CREATE|NLM_F_REPLACE, NTF_PROXY, addr, NULL, dev);
}

int neigh_del_proxy(char *dev, struct in6_addr *addr)
{
	printf("[neigh del proxy()] device : %s\n", dev); 
	debug_hexdump("\t target address : ", (char*)addr, sizeof(struct in6_addr));

	return ip6_neigh_modify(RTM_DELNEIGH, 0, NTF_PROXY, addr, NULL, dev);
}



int ip6_neigh_modify(int cmd, int flags, int type, struct in6_addr *target_addr, char *target_link_addr, char *device)
{
	struct rtnl_handle rth;
	struct {
		struct nlmsghdr 	n;
		struct ndmsg 		ndm;
		char   			buf[256];
	} req;
	inet_prefix addr;

	memset(&req, 0, sizeof(req));

	req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct ndmsg));
	req.n.nlmsg_flags = NLM_F_REQUEST|flags;
	req.n.nlmsg_type = cmd;
	req.ndm.ndm_family = AF_INET6;

	if(cmd ==RTM_DELNEIGH){
		req.ndm.ndm_state = 0x00;
	}else{
//		req.ndm.ndm_state = NUD_PERMANENT;	
		req.ndm.ndm_state = NUD_REACHABLE;
	}
	if(type)
		req.ndm.ndm_flags |= type;
		
	addr.family = AF_INET6;
	addr.bytelen = 16;
	addr.bitlen = 128;
	memcpy(addr.data, target_addr, sizeof(struct in6_addr));

	addattr_l(&req.n, sizeof(req), NDA_DST, &addr.data, addr.bytelen);

	if( target_link_addr )
		addattr_l(&req.n, sizeof(req), NDA_LLADDR, target_link_addr, ETH_ALEN);

	if (rtnl_open(&rth, 0) < 0)
		return -1;

	ll_init_map(&rth);

	if ((req.ndm.ndm_ifindex = ll_name_to_index(device)) == 0) {
		fprintf(stderr, "Cannot find device \"%s\"\n", device);
		return -1;
	}

	if (rtnl_talk(&rth, &req.n,  NULL) < 0)
		return -1;

	return 0;
}


static void ipv6_addr_set(struct in6_addr *addr, 
				     __u32 w1, __u32 w2,
				     __u32 w3, __u32 w4)
{
	addr->s6_addr32[0] = w1;
	addr->s6_addr32[1] = w2;
	addr->s6_addr32[2] = w3;
	addr->s6_addr32[3] = w4;
}
 
//ff02::0001 ּҸ 
// : 	ff01::0001 - interface-local all node
//		ff02::0001 - link-local 
void ipv6_addr_all_nodes(struct in6_addr *addr)
{
	ipv6_addr_set(addr,
		      __constant_htonl(0xFF020000), 0, 0,
		      __constant_htonl(0x1));
}

void ipv6_addr_all_routers(struct in6_addr *addr)
{
	ipv6_addr_set(addr,
		      __constant_htonl(0xFF020000), 0, 0,
		      __constant_htonl(0x2));
}

int addr_del(char *dev, struct in6_addr *addr, int prefix_len)
{
	printf("[addr_del()] device : %s, plen %d\n", dev, prefix_len); 
	debug_hexdump("\t target address : ", (char*)addr, sizeof(struct in6_addr));

	return ip6_addr_del(RTM_DELADDR,  addr, prefix_len, dev);
}

int addr_add(char *dev, struct in6_addr *addr, int prefix_len)
{
	printf("[addr_add()] device : %s, plen %d\n", dev, prefix_len); 
	debug_hexdump("\t target address : ", (char*)addr, sizeof(struct in6_addr));

	return ip6_addr_add(RTM_NEWADDR,  addr, prefix_len, dev);
}


int ip6_addr_del(int cmd, struct in6_addr *addr, int prefix_len, char *device)
{
	struct rtnl_handle rth;
	struct {
		struct nlmsghdr 	n;
		struct ifaddrmsg 	ifa;
		char   			buf[256];
	} req;

	inet_prefix laddr;

	memset(&req, 0, sizeof(req));

	req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct ifaddrmsg));
	req.n.nlmsg_flags = NLM_F_REQUEST|NLM_F_EXCL;
	req.n.nlmsg_type = cmd;
	req.ifa.ifa_family = AF_INET6;	

	laddr.family = AF_INET6;
	laddr.bytelen = 16;
	laddr.bitlen = prefix_len;
	memcpy(laddr.data, addr, sizeof(struct in6_addr));

	addattr_l(&req.n, sizeof(req), IFA_ADDRESS, &laddr.data, laddr.bytelen);
	req.ifa.ifa_prefixlen = laddr.bitlen;

	if (rtnl_open(&rth, 0) < 0)
		return -1;

	ll_init_map(&rth);

	if ((req.ifa.ifa_index = ll_name_to_index(device)) == 0) {
		fprintf(stderr, "Cannot find device \"%s\"\n", device);
		return -1;
	}

	if (rtnl_talk(&rth, &req.n, NULL ) < 0)
		return -1;

	return 0;
}


int ip6_addr_add(int cmd, struct in6_addr *addr, int prefix_len, char *device)
{
	struct rtnl_handle rth;
	struct {
		struct nlmsghdr 	n;
		struct ifaddrmsg 	ifa;
		char   			buf[256];
	} req;

	inet_prefix laddr;

	memset(&req, 0, sizeof(req));

	req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct ifaddrmsg));
	req.n.nlmsg_flags = NLM_F_REQUEST|NLM_F_EXCL;
	req.n.nlmsg_type = cmd;
	req.ifa.ifa_family = AF_INET6;	

	laddr.family = AF_INET6;
	laddr.bytelen = 16;
	laddr.bitlen = prefix_len;

	memcpy(laddr.data, addr, sizeof(struct in6_addr));
	addattr_l(&req.n, sizeof(req), IFA_ADDRESS, &laddr.data, laddr.bytelen);
	req.ifa.ifa_prefixlen = laddr.bitlen;

	if (rtnl_open(&rth, 0) < 0)
		return -1;

	ll_init_map(&rth);

	if ((req.ifa.ifa_index = ll_name_to_index(device)) == 0) {
		fprintf(stderr, "Cannot find device \"%s\"\n", device);
		return -1;
	}

	if (rtnl_talk(&rth, &req.n, NULL ) < 0)
		return -1;

	return 0;
}


/*
int default_router_add(char *dev, struct in6_addr *addr)
{
	struct in6_addr dflt_addr;

	memset(&dflt_addr, 0, sizeof(struct in6_addr));
	inet_pton(AF_INET6, DEFAULT_GW_ADDRESS, &dflt_addr);

	printf("[default_router_add()] device : %s\n", dev); 
	debug_hexdump("\t default gw address : ", (char*)dflt_addr, sizeof(struct in6_addr));
	debug_hexdump("\t target address : ", (char*)addr, sizeof(struct in6_addr));

	return ip6_route_modify(RTM_NEWROUTE, NLM_F_CREATE|NLM_F_REPLACE, RTN_UNICAST, FALSE,
	                                         &dflt_addr, 3, 0, dev, IP6_RT_PRIO_ADDRCONF, addr, 0);
}

int default_router_del(char *dev)
{
	struct in6_addr dflt_addr;

	memset(&dflt_addr, 0, sizeof(struct in6_addr));
	inet_pton(AF_INET6, DEFAULT_GW_ADDRESS, &dflt_addr);

	printf("[default_router_del()] device : %s \n", dev); 
	debug_hexdump("\t target address : ", (char*)&dflt_addr, sizeof(struct in6_addr));

	return ip6_route_modify(RTM_DELROUTE, 0, RTN_UNICAST, FALSE, 
	                                         &dflt_addr, 0, 0, dev,  IP6_RT_PRIO_ADDRCONF, NULL, 0);
}
*/



int ip6_route_modify(int cmd, unsigned flags, int type, int reverse, struct in6_addr *prefix, int prefix_len, int table, char *device, int priority , struct in6_addr *gateway,  int gw_len )
{
	struct rtnl_handle rth;
	struct {
		struct nlmsghdr 	n;
		struct rtmsg 		r;
		char   			buf[1024];
	} req;

	memset(&req, 0, sizeof(req));

	req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
	req.n.nlmsg_flags = NLM_F_REQUEST|flags;
	req.n.nlmsg_type = cmd;
	req.r.rtm_family = AF_INET6;
	req.r.rtm_table = RT_TABLE_MAIN;
	req.r.rtm_scope = RT_SCOPE_NOWHERE;
	addattr32(&req.n, sizeof(req), RTA_PRIORITY, priority);

	if (cmd != RTM_DELROUTE) {
		req.r.rtm_protocol = RTPROT_BOOT;
		req.r.rtm_scope = RT_SCOPE_UNIVERSE;
		req.r.rtm_type = RTN_UNICAST;
	}
	
	if(type)
		req.r.rtm_type = type;

	inet_prefix addr;
	memset(&addr, 0, sizeof(inet_prefix));
	addr.family = AF_INET6;
	addr.bytelen = 16;
	addr.bitlen = prefix_len;
	memcpy(addr.data, prefix, sizeof(struct in6_addr));		

	req.r.rtm_dst_len = addr.bitlen;
	if (addr.bytelen)
		addattr_l(&req.n, sizeof(req), RTA_DST, &addr.data, addr.bytelen);

	if(gateway !=NULL){
		inet_prefix addr;
		memset(&addr, 0, sizeof(inet_prefix));
		addr.family = AF_INET6;
		addr.bytelen = 16;
		addr.bitlen = gw_len;
		memcpy(addr.data, gateway, sizeof(struct in6_addr));
	
		req.r.rtm_dst_len = addr.bitlen;
		if (addr.bytelen)
			addattr_l(&req.n, sizeof(req), RTA_GATEWAY, &addr.data, addr.bytelen);
	}

	if (rtnl_open(&rth, 0) < 0)
			return -1;
	
	if (device )
	{
		int idx;

		ll_init_map(&rth);

		if (device) {
			if ((idx = ll_name_to_index(device)) == 0) {
				syslog(LOG_ERR,"[rtnl_modify_route()] Cannot find device \"%s\"\n", device);
				return -1;
			}
			addattr32(&req.n, sizeof(req), RTA_OIF, idx);
		}
	}

	if (rtnl_talk(&rth, &req.n, NULL) < 0)
		return -1;

	return 0;
}

// ϸ fdϿ  ߻ - ҷ fd   ϶ 
void get_mac_addr(unsigned char *mac, char *if_name)
{
	int             fd;
	struct sockaddr *hw_addr;
	struct ifreq    ifr;

	if ((fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
		perror("get_MAC_addr: fail to open socket");
		exit(1);
	}

	memset(&ifr, 0, sizeof(ifr));
	ifr.ifr_addr.sa_family = AF_INET;
	strncpy(ifr.ifr_name, if_name, sizeof(ifr.ifr_name));

	if (ioctl(fd, SIOCGIFHWADDR, (char *)&ifr) < 0) {
		perror("get_MAC_addr: fail SIOCGIFHWADDR ioctl");
		close(fd);
		exit(1);
	}

	hw_addr = (struct sockaddr *) &ifr.ifr_hwaddr;

	memcpy(mac, (unsigned char *)hw_addr->sa_data, ETH_ALEN);
	mac[ETH_ALEN+1] = '\0';
	
	close(fd);
}




int send_udp_msg(int sock, unsigned short port, char *msg, int len, int ifindex, struct in6_addr *src_addr, struct in6_addr *dest)
{

	struct sockaddr_in6 addr;
	struct in6_pktinfo *pkt_info =NULL;	
	struct msghdr mhdr;
	struct cmsghdr *cmsg=NULL;
	struct iovec iov;
	char chdr[CMSG_SPACE(sizeof(struct in6_pktinfo))]={0x00,};
	int res;	

	printf("sock[%d], port[%d], ifindex[%d]\n", sock, port, ifindex);
	
	memset((void *)&addr, 0, sizeof(addr));
	memset(&mhdr, 0, sizeof(mhdr));
	memset(&iov, 0, sizeof(iov));
	
	addr.sin6_family = AF_INET6;
	addr.sin6_port = htons(port);
	memcpy(&addr.sin6_addr, dest, sizeof(struct in6_addr));

	iov.iov_len  = len;
	iov.iov_base = (caddr_t) msg;

	cmsg = (struct cmsghdr *) chdr;
	
	cmsg->cmsg_len   = CMSG_LEN(sizeof(struct in6_pktinfo));
	cmsg->cmsg_level = IPPROTO_IPV6;
	cmsg->cmsg_type  = IPV6_PKTINFO;

	pkt_info = (struct in6_pktinfo *)CMSG_DATA(cmsg);
	pkt_info->ipi6_ifindex = ifindex;
	
//	if(src_addr)
//		memcpy(&pkt_info->ipi6_addr, src_addr, sizeof(struct in6_addr));
	
	mhdr.msg_name = (caddr_t)&addr;
	mhdr.msg_namelen = sizeof(struct sockaddr_in6);
	mhdr.msg_iov = &iov;
	mhdr.msg_iovlen = 1;
	mhdr.msg_control = (void *) cmsg;
	mhdr.msg_controllen = sizeof(chdr);

	debug_hexdump("[send_udp_msg] Destination :", (unsigned char *)&addr.sin6_addr, 16);

	res = sendmsg(sock, &mhdr, 0);
	
	if (res < 0) {
		perror("[send_udp_msg()] sendmsg");
		return -1;
	}

	return res;
}


int send_icmpv6_msg(int sock, char *msg, int len, int ifindex, struct in6_addr *src_addr, struct in6_addr *dest)
{
	struct sockaddr_in6 addr;
	struct in6_pktinfo *pkt_info =NULL;	
	struct msghdr mhdr;
	struct cmsghdr *cmsg=NULL;
	struct iovec iov;
	char chdr[CMSG_SPACE(sizeof(struct in6_pktinfo))]={0x00,};
	int res;	


	memset((void *)&addr, 0, sizeof(addr));
	memset(&mhdr, 0, sizeof(mhdr));
	memset(&iov, 0, sizeof(iov));
	addr.sin6_family = AF_INET6;
	addr.sin6_port = htons(IPPROTO_ICMPV6);
	memcpy(&addr.sin6_addr, dest, sizeof(struct in6_addr));

	iov.iov_len  = len;
	iov.iov_base = (caddr_t) msg;

	cmsg = (struct cmsghdr *) chdr;
	
	cmsg->cmsg_len   = CMSG_LEN(sizeof(struct in6_pktinfo));
	cmsg->cmsg_level = IPPROTO_IPV6;
	cmsg->cmsg_type  = IPV6_PKTINFO;

	pkt_info = (struct in6_pktinfo *)CMSG_DATA(cmsg);
	pkt_info->ipi6_ifindex = ifindex;
	
	if(src_addr)
		memcpy(&pkt_info->ipi6_addr, src_addr, sizeof(struct in6_addr));
	
	mhdr.msg_name = (caddr_t)&addr;
	mhdr.msg_namelen = sizeof(struct sockaddr_in6);
	mhdr.msg_iov = &iov;
	mhdr.msg_iovlen = 1;
	mhdr.msg_control = (void *) cmsg;
	mhdr.msg_controllen = sizeof(chdr);

	res = sendmsg(sock, &mhdr, 0);
	
	if (res < 0) {
		perror("[send_icmpv6_msg()] sendmsg");
		return -1;
	}

	return res;
}


int open_icmpv6_socket(void)
{
	int sock;
	int err, val;

	sock = socket(AF_INET6, SOCK_RAW, IPPROTO_ICMPV6);
	
	if (sock < 0)
	{
		perror("can't create socket(AF_INET6):");
		return (-1);
	}

	val = 1;
	err = setsockopt(sock, IPPROTO_IPV6, IPV6_PKTINFO, &val, sizeof(int));
	if (err < 0)
	{
		perror("setsockopt(IPV6_PKTINFO): ");
		return (-1);
	}


	val = 2;

	err = setsockopt(sock, IPPROTO_RAW, IPV6_CHECKSUM, &val, sizeof(int));

	if (err < 0)
	{
		perror("setsockopt(IPV6_CHECKSUM): ");
		return (-1);
	}

	val = 255;
	err = setsockopt(sock, IPPROTO_IPV6, IPV6_UNICAST_HOPS, &val, sizeof(int));
	if (err < 0)
	{
		perror("setsockopt(IPV6_UNICAST_HOPS): ");
		return (-1);
	}

	val = 255;
	err = setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_HOPS, &val, sizeof(int));
	if (err < 0)
	{
		perror("setsockopt(IPV6_MULTICAST_HOPS): ");
		return (-1);
	}

#ifdef IPV6_HOPLIMIT
	val = 1;
	err = setsockopt(sock, IPPROTO_IPV6, IPV6_HOPLIMIT, &val, sizeof(int));
	if (err < 0)
	{
		perror("setsockopt(IPV6_HOPLIMIT): ");
		return (-1);
	}
#endif

	return sock;
}



int send_udp_packet(int sock, unsigned short port, char *msg, int len, struct in6_addr *addr)
{
	int size;
	struct sockaddr_in6 to6;
	
	bzero((char*)&to6, sizeof(to6));
	to6.sin6_family = AF_INET6;
	memcpy(&to6.sin6_addr, addr, sizeof(struct in6_addr));
	to6.sin6_port = htons(port);

	debug_hexdump("[send_udp_packet] Destination :", (unsigned char *)addr, 16);

	size = sendto(sock, msg, len, 0,(struct sockaddr*)&to6, sizeof(to6));

	if( size < 0 ) {
		printf("[send_udp_packet] : Send error [%s]\n", strerror(errno));
		return -1;
	}

	return size;
}


int open_udpsock6(unsigned short port, struct in6_addr *s_addr)
{
	int s;			/* socket fd */
	int one = 1, err;
	char buf[100];
	struct sockaddr_in6 	addr;

	if ((s = socket(AF_INET6, SOCK_DGRAM, 0)) < 0) {
			printf("[open_udpsock6()] : Socket open error. \n");
			return -1;
	}

	if ( setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one)) == -1 ) {
			printf("%s\n", strerror(errno));
			return (-1);
	}

	one = 1;
	err = setsockopt(s, IPPROTO_IPV6, IPV6_PKTINFO, &one, sizeof(int));
	if (err < 0)
	{
		perror("setsockopt(IPV6_PKTINFO): ");
		return (-1);
	}
/*
	one = 1;
	err = setsockopt(s, IPPROTO_IPV6, IPV6_HOPLIMIT, &one, sizeof(int));
	if (err < 0)
	{
		perror("setsockopt(IPV6_HOPLIMIT): ");
		return (-1);
	}
*/
	bzero((char*)&addr, sizeof(addr));
	addr.sin6_family = AF_INET6;
	if(s_addr != NULL )
		memcpy(&addr.sin6_addr , s_addr, sizeof(struct in6_addr));
	addr.sin6_port = htons(port);

	if (bind(s, (struct sockaddr *) &addr, sizeof(struct sockaddr_in6)) < 0)
	{
		printf("[open_udpsocket6()] Binding Error[%s]\n", strerror(errno));
		printf("[open_udpsock6()] bind addr : %s\n", inet_ntop(AF_INET6,&addr.sin6_addr,buf,INET6_ADDRSTRLEN));
		return -1;
	}

	printf("[open_udpsock6()] Socket fd : %d  port : %d\n", s, ntohs(addr.sin6_port));
	printf("[open_udpsock6()] bind addr : %s\n", inet_ntop(AF_INET6,&addr.sin6_addr,buf,INET6_ADDRSTRLEN));

	return s;
}







void change_network(struct interface_info_entry *iface)
{
	printf("[change_network] device[%s]\n", iface->dev_name);
	if( !list_empty(&config.peers_list) )
	{
		struct list_head* lead = &config.peers_list;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)		
		{
			struct peers_info_entry *entry = list_entry(p, typeof(*entry), list);

			if( !strcmp( iface->dev_name, entry->dev_index ))
			{
				printf("Delete peer [%s] confirmed[%d]\n", entry->nickname.nick, entry->status);
				debug_hexdump("Ad-hoc Address : ", iface->s_address.address, 16);
				debug_hexdump("Global  Address : ", iface->g_address.address, 16);
				del_list_entry((struct list_head *)entry);				
			}			
		}
	}	
		
}



#ifndef IN6_IS_ADDR_UNSPECIFIED
#define IN6_IS_ADDR_UNSPECIFIED(a) \
        (((__u32 *) (a))[0] == 0 && ((__u32 *) (a))[1] == 0 && \
         ((__u32 *) (a))[2] == 0 && ((__u32 *) (a))[3] == 0)
#endif


static int INET6_rresolve(char *name, struct sockaddr_in6 *sin6, int numeric)
{
    int s;

    /* Grmpf. -FvK */
    if (sin6->sin6_family != AF_INET6) {
	syslog(LOG_ERR,"INET6_rresolve: unsupport address family %d !\n",
		sin6->sin6_family);
	errno = EAFNOSUPPORT;
	return (-1);
    }
    if (numeric & 0x7FFF) {
	inet_ntop(AF_INET6, &sin6->sin6_addr, name, 80);
	return (0);
    }
    if (IN6_IS_ADDR_UNSPECIFIED(&sin6->sin6_addr)) {
        if (numeric & 0x8000)
	    strcpy(name, "default");
	else
	    strcpy(name, "*");
	return (0);
    }

    if ((s = getnameinfo((struct sockaddr *) sin6, sizeof(struct sockaddr_in6),
			 name, 255 /* !! */ , NULL, 0, 0))) {
	syslog(LOG_ERR,"getnameinfo failed\n");
	return -1;
    }
    return (0);
}



/* Display an Internet socket address. */
/* dirty! struct sockaddr usually doesn't suffer for inet6 addresses, fst. */
static char *INET6_sprint(struct sockaddr *sap, int numeric)
{
    static char buff[128]={0x00,};

    if (sap->sa_family == 0xFFFF || sap->sa_family == 0)
	return safe_strncpy(buff, "[NONE SET]", sizeof(buff));
    if (INET6_rresolve(buff, (struct sockaddr_in6 *) sap, numeric) != 0)
	return safe_strncpy(buff, "[UNKNOWN]", sizeof(buff));
    return (buff);
}


static int INET6_getsock(char *bufp, struct sockaddr *sap)
{
    struct sockaddr_in6 *sin6;

    sin6 = (struct sockaddr_in6 *) sap;
    sin6->sin6_family = AF_INET6;
    sin6->sin6_port = 0;

    if (inet_pton(AF_INET6, bufp, sin6->sin6_addr.s6_addr) <= 0)
	return (-1);

    return 16;			/* ?;) */
}

static int INET6_input(char *bufp, struct sockaddr *sap)
{
	return (INET6_getsock(bufp, sap));;
}




int default_router_add(char *dev, struct in6_addr *addr)
{
	struct in6_rtmsg rt;
	struct ifreq ifr;
	struct sockaddr_in6 sa6;
	char *dflt_net = "2000::0";
	int metric, prefix_len;
	char *cp;
	int skfd;

	prefix_len = 3;
	memset(&sa6, 0, sizeof(sa6));
	INET6_input( dflt_net, (struct sockaddr *) &sa6);	
	
	memset((char *) &rt, 0, sizeof(struct in6_rtmsg));

	memcpy(&rt.rtmsg_dst, sa6.sin6_addr.s6_addr, sizeof(struct in6_addr));

	rt.rtmsg_flags = RTF_UP;
	rt.rtmsg_metric = 1;
	rt.rtmsg_dst_len = prefix_len;


	rt.rtmsg_flags |= RTF_GATEWAY;
	memcpy(&rt.rtmsg_gateway, addr, sizeof(struct in6_addr));

	if ((skfd = socket(AF_INET6, SOCK_DGRAM, 0)) < 0) {
		perror("socket");
		return (-1);
	}

	memset(&ifr, 0, sizeof(ifr));
	strcpy(ifr.ifr_name, dev);

	if (ioctl(skfd, SIOGIFINDEX, &ifr) < 0) {
		perror("SIOGIFINDEX");
		return (-1);
	}
	rt.rtmsg_ifindex = ifr.ifr_ifindex;

	if (ioctl(skfd, SIOCADDRT, &rt) < 0) {
		perror("SIOCADDRT");
		close(skfd);
		return (-1);
	}

    (void) close(skfd);
    return (0);
}

int default_router_del(char *dev, struct in6_addr *addr)
{
	struct in6_rtmsg rt;
	struct ifreq ifr;
	struct sockaddr_in6 sa6;
	char *dflt_net = "2000::0";
	int metric, prefix_len;
	char *cp;
	int skfd;

	prefix_len = 3;
	memset(&sa6, 0, sizeof(sa6));
	INET6_input( dflt_net, (struct sockaddr *) &sa6);	
	
	memset((char *) &rt, 0, sizeof(struct in6_rtmsg));

	memcpy(&rt.rtmsg_dst, sa6.sin6_addr.s6_addr, sizeof(struct in6_addr));

	rt.rtmsg_flags = RTF_UP;
	rt.rtmsg_metric = 1;
	rt.rtmsg_dst_len = prefix_len;


	if( addr != NULL )
	{
		rt.rtmsg_flags |= RTF_GATEWAY;
		memcpy(&rt.rtmsg_gateway, addr, sizeof(struct in6_addr));
	}
	
	if ((skfd = socket(AF_INET6, SOCK_DGRAM, 0)) < 0) {
		perror("socket");
		return (-1);
	}

	memset(&ifr, 0, sizeof(ifr));
	strcpy(ifr.ifr_name, dev);

	if (ioctl(skfd, SIOGIFINDEX, &ifr) < 0) {
		perror("SIOGIFINDEX");
		return (-1);
	}
	rt.rtmsg_ifindex = ifr.ifr_ifindex;

	if (ioctl(skfd, SIOCDELRT, &rt) < 0) {
		perror("SIOCDELRT");
		close(skfd);
		return (-1);
	}

    (void) close(skfd);
    return (0);
}


//int get_default_route_info(char * tgt_dev, char *default_gw_string)
int get_default_route_info(char * tgt_dev, struct in6_addr *gw_addr)
{
    char buff[4096]={0x00}, iface[16]={0x00,}, flags[16]={0x00};
    char naddr6[128]={0x00};
    struct sockaddr_in6 snaddr6;
    int num, iflags, metric, refcnt, use, prefix_len, slen;
    char addr6p[8][5], saddr6p[8][5], naddr6p[8][5];

    FILE *fp = fopen("/proc/net/ipv6_route", "r");

    if (!fp) {
        syslog(LOG_ERR, "[get_default_route_info] INET6 (IPv6) not configured in this system.\n");
	return -1;
    }

/*
    DEBUG("\t[get_default_route_info]\n");
    DEBUG("Destination                                 "
	     "Next Hop                                "
	     "Flags Metric Ref    Use Iface\n");
*/
    while (fgets(buff, 1023, fp)) {
	num = sscanf(buff, "%4s%4s%4s%4s%4s%4s%4s%4s %02x %4s%4s%4s%4s%4s%4s%4s%4s %02x %4s%4s%4s%4s%4s%4s%4s%4s %08x %08x %08x %08x %s\n",
		     addr6p[0], addr6p[1], addr6p[2], addr6p[3],
		     addr6p[4], addr6p[5], addr6p[6], addr6p[7],
		     &prefix_len,
		     saddr6p[0], saddr6p[1], saddr6p[2], saddr6p[3],
		     saddr6p[4], saddr6p[5], saddr6p[6], saddr6p[7],
		     &slen,
		     naddr6p[0], naddr6p[1], naddr6p[2], naddr6p[3],
		     naddr6p[4], naddr6p[5], naddr6p[6], naddr6p[7],
		     &metric, &use, &refcnt, &iflags, iface);
	
		//check dev :  ̽ 츸 pass
		if ((strcmp(tgt_dev, iface) ==0) && (iflags & RTF_GATEWAY) && (iflags & RTF_DEFAULT) &&(strncmp(naddr6, "fe80", 4)))
		{
			snprintf(naddr6, sizeof(naddr6), "%s:%s:%s:%s:%s:%s:%s:%s",
				 naddr6p[0], naddr6p[1], naddr6p[2], naddr6p[3],
				 naddr6p[4], naddr6p[5], naddr6p[6], naddr6p[7]);
			INET6_input( naddr6, (struct sockaddr *) &snaddr6);	

			memset(gw_addr, 0, sizeof(struct in6_addr));
			memcpy(gw_addr, &snaddr6.sin6_addr, sizeof(struct in6_addr));

			printf("[get_default_route_info] default gw address : %s\n", naddr6);

			return 0;
		}

    	}
    (void) fclose(fp);
    return (-1);
}




// route о tgt_dev prefix    Ѵ.
int cleanup_route_info(char * tgt_dev, struct in6_addr *prefix, int pre_len)
{
    char buff[4096]={0x00}, iface[16]={0x00,}, flags[16]={0x00};
    char addr6[128]={0x00}, naddr6[128]={0x00};
    struct sockaddr_in6 saddr6, snaddr6;
    int num, iflags, metric, refcnt, use, prefix_len, slen;
    struct in6_addr *gw_addr =NULL;
    char addr6p[8][5], saddr6p[8][5], naddr6p[8][5];

    FILE *fp = fopen("/proc/net/ipv6_route", "r");

    if (!fp) {
       printf("[cleanup_route_info] INET6 (IPv6) not configured in this system.\n");
	return 1;
    }


#ifdef DEBUG_NETWORK
    DEBUG("\t[cleanup_route_info]\n");
    DEBUG("Destination                                 "
	     "Next Hop                                "
	     "Flags Metric Ref    Use Iface\n");
#endif

    while (fgets(buff, 1023, fp)) {
	num = sscanf(buff, "%4s%4s%4s%4s%4s%4s%4s%4s %02x %4s%4s%4s%4s%4s%4s%4s%4s %02x %4s%4s%4s%4s%4s%4s%4s%4s %08x %08x %08x %08x %s\n",
		     addr6p[0], addr6p[1], addr6p[2], addr6p[3],
		     addr6p[4], addr6p[5], addr6p[6], addr6p[7],
		     &prefix_len,
		     saddr6p[0], saddr6p[1], saddr6p[2], saddr6p[3],
		     saddr6p[4], saddr6p[5], saddr6p[6], saddr6p[7],
		     &slen,
		     naddr6p[0], naddr6p[1], naddr6p[2], naddr6p[3],
		     naddr6p[4], naddr6p[5], naddr6p[6], naddr6p[7],
		     &metric, &use, &refcnt, &iflags, iface);

	//check dev :  ̽ 츸 pass
	if (strcmp(tgt_dev, iface) !=0)
		continue;
	
	/* Fetch and resolve the target address. */
	snprintf(addr6, sizeof(addr6), "%s:%s:%s:%s:%s:%s:%s:%s",
		 addr6p[0], addr6p[1], addr6p[2], addr6p[3],
		 addr6p[4], addr6p[5], addr6p[6], addr6p[7]);
	INET6_input( addr6, (struct sockaddr *) &saddr6);

	snprintf(addr6, sizeof(addr6), "%s/%d", INET6_sprint((struct sockaddr *) &saddr6, 1),prefix_len);

	//check prefix : prefix   Ǵ Default   
	if (!prefix_compare((struct in6_addr *)&saddr6.sin6_addr, prefix, pre_len) ){

			/* Fetch and resolve the nexthop address. */
			snprintf(naddr6, sizeof(naddr6), "%s:%s:%s:%s:%s:%s:%s:%s",
				 naddr6p[0], naddr6p[1], naddr6p[2], naddr6p[3],
				 naddr6p[4], naddr6p[5], naddr6p[6], naddr6p[7]);
			INET6_input( naddr6, (struct sockaddr *) &snaddr6);

#ifdef DEBUG_NETWORK
			snprintf(naddr6, sizeof(naddr6), "%s/%d",
				 INET6_sprint((struct sockaddr *) &snaddr6, 1),slen);

			/* Decode the flags. */
			strcpy(flags, "U");
			if (iflags & RTF_GATEWAY)
			    strcat(flags, "G");
			if (iflags & RTF_HOST)
			    strcat(flags, "H");
			if (iflags & RTF_DEFAULT)
			    strcat(flags, "D");
			if (iflags & RTF_ADDRCONF)
			    strcat(flags, "A");
			if (iflags & RTF_CACHE)
			    strcat(flags, "C");

			/* Print the info. */
			printf("%-43s %-39s %-5s %-6d %-2d %7d %-8s\n",
			       addr6, naddr6, flags, metric, refcnt, use, iface);

			debug_hexdump("\t target address : ", (char*)&saddr6.sin6_addr, prefix_len/8);
#endif
			if(!IN6_IS_ADDR_UNSPECIFIED(&snaddr6.sin6_addr))
				gw_addr = (struct in6_addr *)&snaddr6.sin6_addr;


			if(gw_addr != NULL){
#ifdef DEBUG_NETWORK	
				debug_hexdump("\t gatway address : ", (char*)&snaddr6.sin6_addr,16);
#endif
				//next hop  dest addr   neigh  
				if (!memcmp(gw_addr, &saddr6.sin6_addr, sizeof(struct in6_addr)))
					ip6_neigh_modify(RTM_DELNEIGH, 0, 0, gw_addr, NULL, tgt_dev);

			}
			//   Ѵ.
			ip6_route_modify(RTM_DELROUTE, 0, RTN_UNICAST, FALSE, 
		                                (struct in6_addr *)&saddr6.sin6_addr, prefix_len, 0, tgt_dev,  metric,
		                                 gw_addr,gw_addr==NULL?  0:128);
			gw_addr = NULL;
		
	}else{
		continue;
	}
	

    }

    (void) fclose(fp);
    return (0);
}



int initialize_scatter_network(struct interface_info_entry *iface)
{
	if(config.mode != ADHOC_PROXY_MODE)
	{
		char buf[128]={0x00,};

		sprintf(buf, "Not Proxy mode.[%s]\n", config.proxy_if->dev_name);	
		popup_error(buf);						
		return -1;
	}

	config.mode = ADHOC_DELAY_MODE;

	if( !list_empty(&config.peers_list) )
	{
		struct list_head* lead = &config.peers_list;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)	
		{
			struct peers_info_entry *entry = list_entry(p, typeof(*entry), list);

			if( entry->status == CONFIRMED_SCATTER_PEER )
			{
				printf("Proxy neigh delete peer [%s]\n", entry->nickname.nick);
				neigh_del_proxy(config.proxy_if->dev_name, (struct in6_addr *)entry->g_address.address);
			}			
		}
	}	

	if( !list_empty(&config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)			
		{
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

			addr_del(entry->dev_name, (struct in6_addr *)entry->g_address.address, entry->g_address.prefixlen);
			cleanup_route_info(entry->dev_name, (struct in6_addr *)entry->g_address.address, 64);
	
			global_address_conf(entry->dev_name, NULL, 64, entry->mac, &entry->g_address);

			printf("[initialize_scatter_network] Scatter enable [%s]\n", entry->dev_name);
			debug_hexdump("Scatter Global address : ", entry->g_address.address, 16);			
		}
	}

	
	global_address_conf(iface->dev_name, (unsigned char *)&config.default_prefix, 64, iface->mac, &iface->g_address);
	iface->proxy_enable = ADHOC_PROXY_DISABLE;	

	config.mode = ADHOC_SCATTER_MODE;
	
	return 0;
}


int initialize_proxy_network(struct interface_info_entry *iface)
{
	if( config.mode < ADHOC_SCATTER_MODE )
	{
		char buf[128]={0x00,};

		sprintf(buf, "Not support Proxy mode.[%s]\n", config.curr_if_focus->dev_name);	
		popup_error(buf);	
		return -1;
	}
	
	init_global_address(iface->dev_name);
	cleanup_route_info(iface->dev_name, (struct in6_addr *)iface->g_address.address, 64);
	send_ndisc_rs(iface->socks.icmpv6_sock, iface->ifindex, iface->mac);
	
	if( wait_proxy_global_address(iface->dev_name, &iface->g_address) < 0 )
	{
		char buf[128]={0x00,};

		sprintf(buf, "Proxy global address initialize fail.[%s]\n", config.curr_if_focus->dev_name);	
		popup_error(buf);

		global_address_conf(iface->dev_name,(unsigned char *)&config.default_prefix, 64, iface->mac, &iface->s_address);
		global_address_conf(iface->dev_name, NULL, 64, iface->mac, &iface->g_address);

		debug_hexdump("Undo Global address : ", iface->g_address.address, 16);
		
		return -1;
	}
	else
	{
		get_default_route_info(iface->dev_name, &config.gw_addr);
		default_router_add(iface->dev_name, &config.gw_addr);
	}
	
	iface->proxy_enable = ADHOC_PROXY_ENABLE;

	printf("[initialize_proxy_network] Proxy enable [%s]\n", iface->dev_name);
	debug_hexdump("Proxy Global address : ", iface->g_address.address, 16);

	if( !list_empty(&config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)		
		{
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

			if( entry->proxy_enable != ADHOC_PROXY_ENABLE )
			{
				entry->proxy_enable = ADHOC_PROXY_DISABLE;
//				addr_del(entry->dev_name, (struct in6_addr *)entry->s_address.address, entry->s_address.prefixlen);
//				init_global_address(entry->dev_name);

				addr_del(entry->dev_name, (struct in6_addr *)entry->g_address.address, entry->g_address.prefixlen);
				cleanup_route_info(entry->dev_name, (struct in6_addr *)entry->g_address.address, 64);
	
				global_address_conf(entry->dev_name, iface->g_address.address, 80, entry->mac, &entry->g_address);

				send_dis_adgw_adv(entry, NULL);
				printf("[initialize_proxy_network] Reconfiguration [%s]\n", entry->dev_name);
				debug_hexdump("Proxy Global address : ", entry->g_address.address, 16);				
			}
		}
	}

	return 0;
}



void make_address_conf(unsigned char *dev, unsigned char *prefix, unsigned short plen, unsigned char *mac_addr, Address_info *g_address)
{
	struct in6_addr *addr;

	memset(g_address, 0, ADDRESS_INFO_SIZE);
	
	if( prefix == NULL )
	{
		addr  = (struct in6_addr *)g_address->address;
		memcpy(addr, &config.default_prefix, sizeof(struct in6_addr));
		memcpy(addr->s6_addr + 2, mac_addr, 6);
	}
	else
	{
		memset(g_address, 0, sizeof(Address_info));
		addr  = (struct in6_addr *)g_address->address;
		memcpy(addr, prefix, sizeof(struct in6_addr));	
	}

	memcpy(	addr->s6_addr + 8, mac_addr, 3);
	memcpy(	addr->s6_addr + 13, mac_addr+3, 3);
	addr->s6_addr[11] = 0xFF;
	addr->s6_addr[12] = 0xFE;
	addr->s6_addr[8] ^= 2;

	g_address->prefixlen = plen;
	
	return ;
}

void make_proxy_address_conf(unsigned char *dev, unsigned char *prefix, unsigned short plen, unsigned char *mac_addr, Address_info *g_address)
{
	struct in6_addr *addr;

	memset(g_address, 0, ADDRESS_INFO_SIZE);
	
	memset(g_address, 0, sizeof(Address_info));
	addr  = (struct in6_addr *)g_address->address;
	memcpy(addr, prefix, sizeof(struct in6_addr));	

	memcpy(	addr->s6_addr + 13, mac_addr+3, 3);
	addr->s6_addr[11] = 0xFF;
	addr->s6_addr[12] = 0xFE;

	g_address->prefixlen = plen;
	
	return ;
}


void proxy_global_address_conf(unsigned char *dev, 	unsigned char *prefix, int plen, unsigned char *mac_addr, Address_info *g_address)
{
	struct in6_addr *addr;

	memset(g_address, 0, ADDRESS_INFO_SIZE);
	
	if( prefix == NULL )
	{
		addr  = (struct in6_addr *)g_address->address;
		memcpy(addr, &config.default_prefix, sizeof(struct in6_addr));
		memcpy(addr->s6_addr + 2, mac_addr, 6);
	}
	else
	{
		addr  = (struct in6_addr *)g_address->address;
		memcpy(addr, prefix, sizeof(struct in6_addr));	
	}

	memcpy(	addr->s6_addr + 13, mac_addr+3, 3);
	addr->s6_addr[11] = 0xFF;
	addr->s6_addr[12] = 0xFE;

	g_address->prefixlen = plen;
	
	addr_add(dev, addr, plen);

	return ;
}


void global_address_conf(unsigned char *dev, 	unsigned char *prefix, int plen, unsigned char *mac_addr, Address_info *g_address)
{
	struct in6_addr *addr;

	memset(g_address, 0, ADDRESS_INFO_SIZE);
	
	if( prefix == NULL )
	{
		addr  = (struct in6_addr *)g_address->address;
		memcpy(addr, &config.default_prefix, sizeof(struct in6_addr));
		memcpy(addr->s6_addr + 2, mac_addr, 6);
	}
	else
	{
		addr  = (struct in6_addr *)g_address->address;
		memcpy(addr, prefix, sizeof(struct in6_addr));	
	}

	memcpy(	addr->s6_addr + 8, mac_addr, 3);
	memcpy(	addr->s6_addr + 13, mac_addr+3, 3);
	addr->s6_addr[11] = 0xFF;
	addr->s6_addr[12] = 0xFE;
	addr->s6_addr[8] ^= 2;

	g_address->prefixlen = plen;
	
	addr_add(dev, addr, plen);

	return ;
}

int wait_proxy_global_address(char *device, Address_info *addr_info)
{
	FILE *fp;
	char str_addr[128];
	unsigned int plen, pscope, dad_status, if_idx;
	char devname[IFNAMSIZ+1];
	int t=1;
	int scope=IPV6_ADDR_GLOBAL;
	struct in6_addr addr;

	memset(addr_info, 0, ADDRESS_INFO_SIZE);

	change_autoconf_status(device, AUTOCONF_ENABLE);		
	ifup_ipv6_restart(device);
	change_forwarding_status(FORWARDING_ENABLE);

	while(1)
	{
		if ((fp = fopen(PATH_PROC_NET_IF_INET6, "r")) == NULL)
		{
			printf("can't open %s: %s", PATH_PROC_NET_IF_INET6,
				strerror(errno));
			change_autoconf_status(device, AUTOCONF_DISABLE);			

			return (-1);	
		}
		while (fscanf(fp, "%32s %02x %02x %02x %02x %15s\n",
			      str_addr, &if_idx, &plen, &pscope, &dad_status,
			      devname) != EOF)
		{
			if (pscope == scope && strcmp(devname, device) == 0)
			{
				unsigned int ap;
				int i;

				for (i=0; i<16; i++)
				{
					sscanf(str_addr + i * 2, "%02x", &ap);
					addr.s6_addr[i] = (unsigned char)ap;
				}

				memcpy(addr_info->address, &addr, sizeof(struct in6_addr));
				addr_info->prefixlen = plen;
				fclose(fp);
				change_autoconf_status(device, AUTOCONF_DISABLE);
				return 0;
			}
		}

		if( t < PUBLIC_RADV_WAIT_TIME )
		{
//			wireless_commit(config.scan_sock, device);
			t++;
			fclose(fp);
		}
		else
			break;
	}
	
	fclose(fp);
	change_autoconf_status(device, AUTOCONF_DISABLE);
	return -1;	
}


int get_ipv6_addr(unsigned char *device, int scope,  struct in6_addr *addr, unsigned int *ifindex)
{
	FILE *fp;
	char str_addr[128];
	unsigned int plen, pscope, dad_status, if_idx;
	char devname[IFNAMSIZ+1];
	int err=1;
	
	if ((fp = fopen(PATH_PROC_NET_IF_INET6, "r")) == NULL)
	{
		printf("can't open %s: %s", PATH_PROC_NET_IF_INET6,
			strerror(errno));
		return (-1);	
	}
	
	while (fscanf(fp, "%32s %02x %02x %02x %02x %15s\n",
		      str_addr, &if_idx, &plen, &pscope, &dad_status,
		      devname) != EOF)
	{
		 if (pscope == scope && strcmp(devname, device) == 0)
		{
			unsigned int ap;
			int i;
			
			for (i=0; i<16; i++)
			{
				sscanf(str_addr + i * 2, "%02x", &ap);
				addr->s6_addr[i] = (unsigned char)ap;
			}

			*ifindex = if_idx;
			err=0;			
		}
	}

	fclose(fp);
	
	if(err)
	{
		printf("no linklocal address configured for %s", device);
		return (-1);	
	}
	return 0;	
}

void init_scatter_autoconf()
{
	if( !list_empty(&config.devices) )
	{
		struct list_head* lead = &config.devices;
		struct list_head *p,*n;

		list_for_each_safe(p, n, lead)	
		{
			struct interface_info_entry *entry = list_entry(p, typeof(*entry), list);

//			sleep(config.wait_time);		
//			entry->socks.udp_sock = open_udpsock6(config.udp_port, (struct in6_addr *)entry->s_address.address);				

			global_address_conf(entry->dev_name, NULL, 64, entry->mac, &entry->g_address);
			debug_hexdump("[init_scatter_autoconf] Assign Satter address :", entry->g_address.address, 16);
			
			if( (entry->ifmode == WIRELESS_INTERFACE) && (entry->curr_wlan_info.mode == IW_MODE_ADHOC) )
			{
				send_dis_adgw_adv(entry, NULL);
//				send_ra(entry->socks.icmpv6_sock, entry->ifindex, NULL, (struct in6_addr *)entry->s_address.address, 64, entry->mac);
			}
		}
	}
	change_forwarding_status(FORWARDING_ENABLE);
}


int init_global_address(unsigned char *device)
{
	FILE *fp;
	char str_addr[128];
	unsigned int plen, pscope, dad_status, if_idx;
	char devname[IFNAMSIZ+1];
	int scope = IPV6_ADDR_GLOBAL;
	struct in6_addr addr;
	
	if ((fp = fopen(PATH_PROC_NET_IF_INET6, "r")) == NULL)
	{
		printf("can't open %s: %s", PATH_PROC_NET_IF_INET6,
			strerror(errno));
		return (-1);	
	}
	
	while (fscanf(fp, "%32s %02x %02x %02x %02x %15s\n",
		      str_addr, &if_idx, &plen, &pscope, &dad_status,
		      devname) != EOF)
	{
		 if (pscope == scope && strcmp(devname, device) == 0)
		{
			unsigned int ap;
			int i;
			
			for (i=0; i<16; i++)
			{
				sscanf(str_addr + i * 2, "%02x", &ap);
				addr.s6_addr[i] = (unsigned char)ap;
			}
			//    ִ global address  .
			addr_del(device, &addr, plen);
		}
	}
//	default_router_del(device);
	fclose(fp);
	return 0;	
}



//nprefix̸ŭ addr prefix Ͽ  0 
int prefix_compare(struct in6_addr *addr, struct in6_addr *prefix, unsigned int nprefix)
{
	int i;

	if (nprefix > 128)
		return -1;

	for (i = 0; nprefix > 0; nprefix -= 32, i++) {
		if (nprefix >= 32) {
			if (addr->s6_addr32[i] != prefix->s6_addr32[i])
				return 1;
		} else {
			if (((addr->s6_addr32[i] ^ prefix->s6_addr32[i]) &
			     ((~0) << (32 - nprefix))) != 0)
				return 1;
			return 0;
		}
	}

	return 0;
}


int change_autoconf_status(char *dev, char *status)
{
	char path[128]={0x00,};

	sprintf(path, "echo %s > %s/%s/autoconf", status, PATH_PROC_AUTOCONF, dev);

	system(path);

	return 0;
}

int change_forwarding_status(char *status)
{
	char path[128]={0x00,};

	sprintf(path, "echo %s > %s", status, PATH_PROC_FORWARDING);

	system(path);

	return 0;
}

int ifup_ipv6_restart(char *dev)
{
	char path[128]={0x00,};

	sprintf(path, "/etc/sysconfig/network-scripts/ifup-ipv6 %s", dev);

	system(path);
	sleep(SAFE_WIRELESS_WAIT_TIME);
	return 0;
}


void send_ra(int sock, int ifindex, struct in6_addr *dest, struct in6_addr *prefix, int plen, unsigned char * lladdr)
{
	struct sockaddr_in6 addr;
	struct in6_addr dst;
	struct nd_router_advert *radvert = NULL;
	unsigned char buff[MAX_MSG_LEN] ={0x00,};
	int len = 0;
	struct nd_opt_prefix_info *pinfo= NULL;

	unsigned char  LL_addr[LLADDR_MAX] ={0x00, };
	unsigned char *ucp =NULL;
	int if_hwaddr_len = ETH_ALEN;

	if (dest == NULL)
		ipv6_addr_all_nodes(&dst);
	else
		memcpy(&dst, dest, sizeof(struct in6_addr));
	
	addr.sin6_family = AF_INET6;
	addr.sin6_port = htons(IPPROTO_ICMPV6);
	memcpy(&addr.sin6_addr, &dst, sizeof(struct in6_addr));

	radvert = (struct nd_router_advert *) buff;
	radvert->nd_ra_type  = ND_ROUTER_ADVERT;
	radvert->nd_ra_code  = 0;
	radvert->nd_ra_cksum = 0;

	radvert->nd_ra_curhoplimit	= 0x40;
	radvert->nd_ra_flags_reserved	= 0;
	radvert->nd_ra_flags_reserved	= 0;

	radvert->nd_ra_router_lifetime	= 300;

	radvert->nd_ra_reachable  = htonl(0);
	radvert->nd_ra_retransmit = htonl(0);

	len = sizeof(struct nd_router_advert);

	/*
	 *	add prefix options
	 */
	pinfo = (struct nd_opt_prefix_info *) (buff + len);
	pinfo->nd_opt_pi_type	     = ND_OPT_PREFIX_INFORMATION;
	pinfo->nd_opt_pi_len	     = 4;
	pinfo->nd_opt_pi_prefix_len  = plen;
	pinfo->nd_opt_pi_flags_reserved  = ND_OPT_PI_FLAG_ONLINK;
	pinfo->nd_opt_pi_flags_reserved	|= ND_OPT_PI_FLAG_AUTO;
	pinfo->nd_opt_pi_flags_reserved	|= ND_OPT_PI_FLAG_RADDR;


	 //Mobile IPv6 ext 
	pinfo->nd_opt_pi_flags_reserved |= 0;
	pinfo->nd_opt_pi_valid_time	= htonl(DFLT_AdvValidLifetime);
	pinfo->nd_opt_pi_preferred_time = htonl(DFLT_AdvPreferredLifetime);
	pinfo->nd_opt_pi_reserved2	= 0;
	memcpy(&pinfo->nd_opt_pi_prefix, prefix, sizeof(struct in6_addr));

	len += sizeof(struct nd_opt_prefix_info);

	/*
	 * add Source Link-layer Address option
	 */
	 memcpy((char * )LL_addr,(char * )lladdr,if_hwaddr_len);
	ucp = (unsigned char *) (buff + len);

	*ucp++  = ND_OPT_SOURCE_LINKADDR;
	*ucp++  = 0x01;   //len field

	len += 2 * sizeof(unsigned char );

	memcpy(buff + len, LL_addr, 6);
	len += 6;

	send_icmpv6_msg(sock, buff, len, ifindex, NULL, &dst);
//		(int sock, char * msg, int len, int ifindex, struct in6_addr * src_addr, struct in6_addr * dest)

#ifdef DEBUG_NETWORK
			debug_hexdump("[send_ra()] sended RA packet", buff, len);
#endif

}


#ifndef NDISC_ROUTER_SOLICITATION
#define NDISC_ROUTER_SOLICITATION	133
#define NDISC_NEIGHBOUR_ADVERTISEMENT	136
#endif

#ifndef ND_OPT_SOURCE_LL_ADDR
#define ND_OPT_SOURCE_LL_ADDR		1
#define ND_OPT_TARGET_LL_ADDR		2
#endif




struct nd_msg {
	struct nd_neighbor_advert nd_na;
	__u8		 opt[0];
};

#define NDISC_OPT_SPACE(len) (((len)+2+7)&~7)

static __u8 *ndisc_fill_option(__u8 *opt, int type, void *data, int data_len)
{
	int space = NDISC_OPT_SPACE(data_len);

	opt[0] = type;
	opt[1] = space>>3;
	memcpy(opt+2, data, data_len);
	data_len += 2;
	opt += data_len;
	if ((space -= data_len) > 0)
		memset(opt, 0, space);
	return opt + space;
}
	
void send_ndisc_na(int sockfd, int ifindex, char *dev_mac, struct in6_addr *target_addr , int is_router ) 
{
	struct in6_addr addr;
	struct nd_msg msg;	
	unsigned char lladdr[ETH_ALEN] ={0x00};
	int len=0;

	memset(&msg, 0, sizeof(struct nd_msg));
	
	/* NEIGHBOUR_ADVERTISEMENT */
	msg.nd_na.nd_na_type = NDISC_NEIGHBOUR_ADVERTISEMENT;
	msg.nd_na.nd_na_code = 0;
	msg.nd_na.nd_na_cksum = 0;
	
	msg.nd_na.nd_na_flags_reserved = 0;
	msg.nd_na.nd_na_flags_reserved |= ND_NA_FLAG_OVERRIDE;
	if(is_router)
		msg.nd_na.nd_na_flags_reserved |=	ND_NA_FLAG_ROUTER;
	
	len += sizeof(struct nd_msg);
	
	ipv6_addr_all_nodes(&addr);
	memcpy(lladdr, dev_mac,ETH_ALEN);
	ndisc_fill_option(msg.opt, ND_OPT_TARGET_LL_ADDR, lladdr, ETH_ALEN);

	/* Set the target address. */
	memcpy(&msg.nd_na.nd_na_target, target_addr, sizeof(struct in6_addr));
	len += 8;

	send_icmpv6_msg(sockfd, (char *)&msg, len, ifindex, NULL, &addr);

	debug_hexdump("[send_ndisc_na] sended NA packet", (char * )&msg, len);
}        



void send_ndisc_rs(int sockfd, int ifindex, char *dev_mac)
{
	struct in6_addr addr;
	struct nd_msg msg;	
	unsigned char lladdr[ETH_ALEN] ={0x00};
	int len=0;

	memset(&msg, 0, sizeof(struct nd_msg));
	
	/* NEIGHBOUR_ADVERTISEMENT */
	msg.nd_na.nd_na_type = NDISC_ROUTER_SOLICITATION;
	msg.nd_na.nd_na_code = 0;
	msg.nd_na.nd_na_cksum = 0;
	
	msg.nd_na.nd_na_flags_reserved = 0;
	msg.nd_na.nd_na_flags_reserved = 0;
	msg.nd_na.nd_na_flags_reserved = 0;
	
	len += sizeof(struct nd_msg);
	
	ipv6_addr_all_nodes(&addr);
	memcpy(lladdr, dev_mac,ETH_ALEN);
	ndisc_fill_option(msg.opt, ND_OPT_SOURCE_LL_ADDR, lladdr, ETH_ALEN);

	/* Set the target address. */
//	memcpy(&msg.nd_na.nd_na_target, target_addr, sizeof(struct in6_addr));
//	len += 8;

	send_icmpv6_msg(sockfd, (char *)&msg, len, ifindex, NULL, &addr);

	debug_hexdump("[send_ndisc_rs] sended RS packet", (char * )&msg, len);
}

