#include "scan.h"



/************************* SETTING ROUTINES **************************/

/*------------------------------------------------------------------*/
/*
 * Macro to handle errors when setting WE
 * Print a nice error message and exit...
 * We define them as macro so that "return" do the right thing.
 * The "do {...} while(0)" is a standard trick
 */
#define ERR_SET_EXT(rname, request) \
	fprintf(stderr, "Error for wireless request \"%s\" (%X) :\n", \
		rname, request)

#define ABORT_ARG_NUM(rname, request) \
	do { \
		ERR_SET_EXT(rname, request); \
		fprintf(stderr, "    too few arguments.\n"); \
		return(-1); \
	} while(0)

#define ABORT_ARG_TYPE(rname, request, arg) \
	do { \
		ERR_SET_EXT(rname, request); \
		fprintf(stderr, "    invalid argument \"%s\".\n", arg); \
		return(-2); \
	} while(0)

#define ABORT_ARG_SIZE(rname, request, max) \
	do { \
		ERR_SET_EXT(rname, request); \
		fprintf(stderr, "    argument too big (max %d)\n", max); \
		return(-3); \
	} while(0)

/*------------------------------------------------------------------*/
/*
 * Wrapper to push some Wireless Parameter in the driver
 * Use standard wrapper and add pretty error message if fail...
 */
#define IW_SET_EXT_ERR(skfd, ifname, request, wrq, rname) \
	do { \
	if(iw_set_ext(skfd, ifname, request, wrq) < 0) { \
		ERR_SET_EXT(rname, request); \
		fprintf(stderr, "    SET failed on device %-1.8s ; %s.\n", \
			ifname, strerror(errno)); \
	}  } while(0)

/*------------------------------------------------------------------*/
/*
 * Wrapper to extract some Wireless Parameter out of the driver
 * Use standard wrapper and add pretty error message if fail...
 */
#define IW_GET_EXT_ERR(skfd, ifname, request, wrq, rname) \
	do { \
	if(iw_get_ext(skfd, ifname, request, wrq) < 0) { \
		ERR_SET_EXT(rname, request); \
		fprintf(stderr, "    GET failed on device %-1.8s ; %s.\n", \
			ifname, strerror(errno)); \
		return(-6); \
	} } while(0)

/*------------------------------------------------------------------*/
/*
 * Set the wireless options requested on command line
 * This function is too long and probably should be split,
 * because it look like the perfect definition of spaghetti code,
 * but I'm way to lazy
 */

void wireless_commit(int sock, char *dev)
{
	struct iwreq		wrq;
	
	memset(&wrq, 0, sizeof(struct iwreq));
	IW_SET_EXT_ERR(sock, dev, SIOCSIWCOMMIT, &wrq, "Commit changes");
}


int ap_link_change(int scan_sock, int mode, char* apessid, int channel, char *nick, char *ifname)
{
	struct iwreq		wrq;
	char		essid[IW_ESSID_MAX_SIZE + 1];
       
	if(strlen(apessid) > IW_ESSID_MAX_SIZE)
		ABORT_ARG_SIZE("Set ESSID", SIOCSIWESSID, IW_ESSID_MAX_SIZE);
	else
	{
		double freq;

		wrq.u.essid.flags = 1;
		
		strcpy(essid, apessid);	/* Size checked, all clear */
		wrq.u.essid.pointer = (caddr_t) essid;
		wrq.u.essid.length = strlen(essid) + 1;
		IW_SET_EXT_ERR(scan_sock, ifname, SIOCSIWESSID, &wrq, "Set ESSID");

		wrq.u.mode = mode;
		IW_SET_EXT_ERR(scan_sock, ifname, SIOCSIWMODE, &wrq, "Set Mode");

		if (mode == IW_MODE_ADHOC )		
		{
			freq = (double)channel;	
			iw_float2freq(freq, &(wrq.u.freq));
			IW_SET_EXT_ERR(scan_sock, ifname, SIOCSIWFREQ, &wrq, "Set Frequency");
		}

		if( nick != NULL )
		{
			wrq.u.essid.pointer = (caddr_t) nick;
			wrq.u.essid.length = strlen(nick) + 1;
			IW_SET_EXT_ERR(scan_sock, ifname, SIOCSIWNICKN, &wrq, "Set Nickname");
		}		

//		wrq.u.power.flags |= IW_POWER_ALL_R;
//		IW_SET_EXT_ERR(scan_sock, ifname, SIOCSIWPOWER, &wrq,"Set Power Management");		

//		memset(&wrq, 0, sizeof(struct iwreq));
//		IW_SET_EXT_ERR(scan_sock, ifname, SIOCSIWCOMMIT, &wrq, "Commit changes");
	}

	return(0);
}
		
/***************************** SCANNING *****************************/
/*
 * This one behave quite differently from the others
 */
#if WIRELESS_EXT > 13
/*------------------------------------------------------------------*/


static int make_aplist(int mode, struct list_head *ap_list, char *ifname, struct iw_event *	event,	/* Extracted token */
		     int		 *ap_num,	/* AP number  = return ΰ????*/
		     struct iw_range *	iwrange,	/* Range info */
		     int		has_range)
{

	
	char		buffer[128] = {0x00,};	/* Temporary buffer */
	/* Now, let's decode the event */
	switch(event->cmd)
	{
		case SIOCGIWAP:
		{
			struct ap_entry* entry=NULL;	

			/* ap list  AP ʱȭ ϰ ap_list head  δ  */
			entry = malloc(sizeof(struct ap_entry));
	
			if(entry == NULL)
			{
				printf("entry is NULL\n");
				return -1;
				
			}
			memset(entry, 0x00, sizeof(struct ap_entry));
			memcpy(entry->ap_link_addr, (const unsigned char*)event->u.ap_addr.sa_data,6); 
			(*ap_num) ++;		//entry->id = ap_num ΰҵ  ׸ ap_num۰ 0  
			entry->id = (*ap_num);
			strcpy(entry->device, ifname);
			list_add(&entry->list, ap_list);
			/* debug print */
	      		printf("[make_aplist()]  ap cell %02d - Address: %s, ", *ap_num, iw_pr_ether(buffer, event->u.ap_addr.sa_data));
      			break;
	       }
       
		case SIOCGIWESSID:
	       {
			int flag = 0;
			struct ap_entry	*lead = NULL;
			struct list_head * p;

			/* 1. ap_num  ap_entryã´. */
			list_for_each(p, ap_list)		
			{	
				lead = list_entry(p, typeof(*lead), list);
				if(lead->id==(*ap_num))
				{	
					flag=1;
					break;
				}
			}
			if(flag==0)	//° .. 켱 ׳  
			{
				printf("[make_aplist()] can't search SIOCGIWESSID\n");
				break;
			}
			/* 2. ã :  lead ã ap_entryŰ ִ 
					 essidִ´ */
			memcpy(lead->essid, event->u.essid.pointer, event->u.essid.length);
			lead->essid[event->u.essid.length] = '\0';

			printf(" id = %d, essid = %s\n", lead->id, lead->essid);	
     			break;
		}
		case SIOCGIWMODE:
		{
			int flag = 0;
			struct ap_entry	*lead = NULL;
			struct list_head * p;

			// 1. ap_num  ap_entryã´. 
			list_for_each(p, ap_list)		
			{	
				lead = list_entry(p, typeof(*lead), list);
				if(lead->id==(*ap_num))
				{	
					flag=1;
					break;
				}
			}
			if(flag==0)	//° .. 켱 ׳  
			{
				printf("[make_aplist()] can't search SIOCGIWMODE\n");
				break;
			}			
			strcpy( lead->mode, iw_operation_mode[event->u.mode] );
			lead->mode_t = event->u.mode;

			if( mode == IW_MODE_MASTER )
			{
				if( (lead->mode_t != IW_MODE_MASTER) && (lead->mode_t != IW_MODE_INFRA) )
				{
					list_del((struct list_head*)lead);
					free(lead);				
				}
			}
			else
			{
				if( lead->mode_t != IW_MODE_ADHOC )
				{
					list_del((struct list_head*)lead);
					free(lead);
				}
			}
			
			printf("  mode %8s, ", lead->mode);
			break;		
		}
		case SIOCGIWFREQ:
		{
			double	freq;
			int flag = 0;
			struct ap_entry	*lead = NULL;
			struct list_head * p;
			char			buffer[128];	/* Temporary buffer */

			// 1. ap_num  ap_entryã´. 
			list_for_each(p, ap_list)		
			{	
				lead = list_entry(p, typeof(*lead), list);
				if(lead->id==(*ap_num))
				{	
					flag=1;
					break;
				}
			}
			if(flag==0)	//° .. 켱 ׳  
			{
				printf("[make_aplist()] can't search SIOCGIWFREQ\n");
				break;
			}
			// 2. ã :  lead ã ap_entryŰ ִ . frequence  ִ´ .
			freq = iw_freq2float(&(event->u.freq));	
			iw_print_freq(buffer, freq);
			lead->channel = iw_freq_to_channel(freq, iwrange);
			printf("Channel %02d,  %s, ", lead->channel, buffer);
			break;
		}
		case IWEVQUAL:
		{
			int flag=0;
			struct ap_entry	*ptr = NULL, *lead = NULL;
			struct list_head * p;

			/* 1. ap_num  ap_entryã´. */
			list_for_each(p, ap_list)		
			{	
				lead = list_entry(p, typeof(*lead), list);
				if(lead->id==(*ap_num))
				{	
					flag=1;
					break;
				}
			}
			if(flag==0)	//° .. 켱 ׳  
					{
				printf("[make_aplist()] can't search IWEVQUAL\n");
				break;
			}
			/* 2. ã :  lead ã ap_entryŰ ִ 
					1. strengthְ 2. Ʈ װ 3. Ѵ */
			flag=0;
			lead->strength = event->u.qual.level-0x100;	/* 1.strengthְ ,, level 8bit̴.  ̳ʽ 0x100Ծ..*/
			/* debug print */
      		       event->u.qual.updated = 0x0;	/* Not that reliable, disable */
			iw_print_stats(buffer, &event->u.qual, iwrange, has_range);
			printf("%s\n", buffer);

			list_del_init((struct list_head*)lead);  				/* 2. list . */
			/* ap_list m head ٷ  */
			if(list_empty(ap_list))
			{
				list_add(&lead->list, ap_list);
				break;
			}
			list_for_each(p, ap_list)  				/* 3. */
			{
				ptr = list_entry(p, typeof(*ptr), list);
				if(abs(lead->strength) < abs(ptr->strength))	// ã´.
				{
					flag=1;
					list_add_tail(&lead->list, &ptr->list);
						break;  
				}
			}		 

   		    /* ã .. ap_list ǳ(ptr) δ(   ) */
			if(flag==0)
				list_add(&lead->list, &ptr->list);
			break;		
		}
		
 		case SIOCGIWNWID:
		case SIOCGIWENCODE:
		case SIOCGIWRATE:
		default:
			break;
	}	/* switch(event->cmd) */

	
	/* May have changed */
	return(*ap_num);
}





/*------------------------------------------------------------------*/

/*
 * Perform a scanning on one device
 * Ȱ : ap scanning Ͽ ap mac, strength ap_list strength Ѵ. 
 * Ҹ κ : 
 	1> scanõ(timeout ) 5ʵϴ° ʹ . ⸦ ٿ° ..
*/
static int scan(int mode, struct list_head *ap_list, int		skfd,
		    char *	ifname,
		    char *	args[],		/* Command line args */
		    int		count)		/* Args count */	
{
	struct iwreq		wrq;
	unsigned char		buffer[IW_SCAN_MAX_DATA];	/* Results */
	struct timeval	tv;				/* Select timeout */
	int			timeout = 5000000;		/* 5s */

	/* Avoid "Unused parameter" warning */
	args = args; 
	count = count;

	/* Init timeout value -> 250ms*/
	tv.tv_sec = 0;
	tv.tv_usec = 250000;

 	/*
 	* Here we should look at the command line args and set the IW_SCAN_ flags
	* properly
	*/
	wrq.u.param.flags = IW_SCAN_DEFAULT;
	wrq.u.param.value = 0;		/* Later */

	/* Initiate Scanning ....⼭ scan requestϰ???*/
	if(iw_set_ext(skfd, ifname, SIOCSIWSCAN, &wrq) < 0)
	{
		if(errno != EPERM)
		{
			fprintf(stderr, "%-8.8s  Interface doesn't support scanning : %s\n\n",
			ifname, strerror(errno));
			return(-1);
		}
		/* If we don't have the permission to initiate the scan, we may
		* still have permission to read left-over results.
		* But, don't wait !!! */
#if 0
      /* Not cool, it display for non wireless interfaces... */
      fprintf(stderr, "%-8.8s  (Could not trigger scanning, just reading left-over results)\n", ifname);
#endif
		tv.tv_usec = 0;
	}
	sleep(1);
	timeout -= tv.tv_usec;

	/* Forever scan request  ٸ...*/
	while(1)
	{
		fd_set		rfds;		/* File descriptors for select */
		int		last_fd;	/* Last fd */
		int		ret;

		/* Guess what ? We must re-generate rfds each time */
		FD_ZERO(&rfds);
		last_fd = -1;

		/* In here, add the rtnetlink fd in the list */

		/* Wait until something happens ׻ tv  ȴ Init timeout value -> 250ms*/
		ret = select(last_fd + 1, &rfds, NULL, NULL, &tv);

		/* Check if there was an error */
		if(ret < 0)
		{
			if(errno == EAGAIN || errno == EINTR)
				continue;
			fprintf(stderr, "Unhandled signal - exiting...\n");
			return(-1);
		}

		/* Check if there was a timeout */
		if(ret == 0)
		{
			/* Try to read the results */
			wrq.u.data.pointer = buffer; // buffer data
			wrq.u.data.flags = 0;
			wrq.u.data.length = sizeof(buffer);
			/*scan request   wrp.u.data.pointer(=buffer) ȴ.*/ 
			if(iw_get_ext(skfd, ifname, SIOCGIWSCAN, &wrq) < 0) //Ⱑ ߿ 
			{
				/* Check if results not available yet */
				if(errno == EAGAIN)
				{
					/* Restart timer for only 100ms   100ms ٿ ٽ scanѴ ٵ 
					Ż timeoutð 5ʶ... ʹ  ƴѰ?  */
					tv.tv_sec = 0;
					tv.tv_usec = 100000;
					timeout -= tv.tv_usec;
					if(timeout > 0)
						continue;	/* Try again later */
				}

				/* Bad error */
				fprintf(stderr, "%-8.8s  Failed to read scan data : %s\n\n",
				ifname, strerror(errno));
				return(-2);
			}
			else	/* We have the results, go to process them */
				break;
		}

		/* In here, check if event and event type
		* if scan event, read results. All errors bad & no reset timeout */
	}

	/* 1. scan request   wrp.u.data.(=buffer) Ǿִ*/ 
	if(wrq.u.data.length)	/* length buffer size */
	{
		struct iw_event		iwe;
		struct stream_descr	stream;
		int			ap_num = 0;
		int			ret, cnt=0;
		struct iw_range		range;
		int			has_range;
		has_range = (iw_get_range_info(skfd, ifname, &range) >= 0);
		printf("%-8.8s  Scan completed :\n", ifname);
		iw_init_event_stream(&stream, buffer, wrq.u.data.length);//stream current=data, end=data+length
		do
		{
	  		/* Extract an event and print it */
	  		/* event stream   event  iwe scan ϳ ap  ִ */ 
			memset( &iwe, 0, sizeof(struct iw_event));
	  		ret = iw_extract_event_stream(&stream, &iwe);
			/* 2.  Լ scanning  ap_list ϰ, Ұ̴. */
		  	if(ret > 0)
				cnt = make_aplist(mode, ap_list, ifname, &iwe, &ap_num, &range, has_range);	
		}
		while(ret > 0);
		printf("\n");
	}
	else
		printf("%-8.8s  No scan results\n", ifname);

	return(0);
}

int scanning(int mode, int scan_sock, struct list_head *ap_list, char *ifname)
{
	free_ap_list(ap_list);
	return scan(mode, ap_list, scan_sock, ifname, NULL, 0);
 
 }

/*
*Ȱ : ap_list ϸ  freeѴ 
*/
int free_ap_list(struct list_head *ap_list)
{
	struct list_head *head = ap_list;
	struct list_head *pos, *n;
	struct ap_entry *entry;
	
	list_for_each_safe(pos, n, head)
	{
			entry = list_entry(pos, typeof(*entry), list);

			list_del_init((struct list_head*)entry);
			free(entry);
	}

	return 1;
}


int get_curr_wireless_info(int skfd, char *ifname, struct wlan_info *curr_info)
{
	struct wlan_info info;
	struct iwreq		wrq;
	double freq;

	memset((char *) &info, 0, sizeof(struct wlan_info));

	/* Get frequency / channel */
	if(iw_get_ext(skfd, ifname, SIOCGIWFREQ, &wrq) >= 0)
	{
		freq = iw_freq2float(&(wrq.u.freq));
		info.channel = iw_freq_to_channel(freq, NULL);
	}
	else
	{
		printf("SIOCGIWFREQ error\n");
		goto err;
	}
	/* Get ESSID */
	wrq.u.essid.pointer = (caddr_t) info.essid;
	wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
	wrq.u.essid.flags = 0;
	if(iw_get_ext(skfd, ifname, SIOCGIWESSID, &wrq) < 0)
	{
		printf("SIOCGIWESSID error\n");
		goto err;
	}

	/* Get NickName */
	wrq.u.essid.pointer = (caddr_t) info.nickname;
	wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
	wrq.u.essid.flags = 0;
	if(iw_get_ext(skfd, ifname, SIOCGIWNICKN, &wrq) < 0)
	{
		printf("SIOCGIWNICKN error\n");
		goto err;
	}
	/* Get operation mode */
	if(iw_get_ext(skfd, ifname, SIOCGIWMODE, &wrq) >= 0)
		info.mode = wrq.u.mode;
	else
	{
		printf("SIOCGIWMODE error\n");
		goto err;
	}

	printf("[get_curr_wireless_info] mode %d\n", info.mode);
	printf("[get_curr_wireless_info] channel %d\n", info.channel);
	printf("[get_curr_wireless_info] essid %s\n", info.essid);
	printf("[get_curr_wireless_info] nick %s\n", info.nickname);

	memcpy(curr_info, &info, sizeof(struct wlan_info));
	return 0;

	
err:
	memset((char *) curr_info, 0, sizeof(struct wlan_info));	
	return -1;
}
/* Get AP address */
char* get_curr_ap_addr(int skfd, char* ifname, char* ap_addr)
{
	struct wireless_info	info;
	struct iwreq		wrq;

	if(iw_get_ext(skfd, ifname, SIOCGIWAP, &wrq) >= 0)
    	{
	      info.has_ap_addr = 1;
	      memcpy(&(info.ap_addr), &(wrq.u.ap_addr), sizeof (sockaddr));
   	}

	memcpy(ap_addr,(char*)&(info.ap_addr.sa_data),ETH_ALEN);
	
#ifdef DEBUG_NETWORK
	debug_hexdump("[get_curr_ap_addr()] ap addr \n", ap_addr, ETH_ALEN);
#endif

	return ap_addr;
}



#endif /* wireless _ext 13 */


